@@ -6,33 +6,19 @@
 //
 // Design and Implementation by Bjoern Lemke               
 //                                                         
-// (C)opyright 2000-2010 Bjoern Lemke                        
-//
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
-// 
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; see the file COPYING.  If not, write to
-// the Free Software Foundation, 59 Temple Place - Suite 330,
-// Boston, MA 02111-1307, USA.
+// (C)opyright 2000-2016 Bjoern Lemke                        
 //
 // IMPLEMENTATION MODULE
 //
-// Class: Main
+// Class: main
 // 
-// Description: 
+// Description: The cego administration console program 
 //
-// Status: QG-2.6
+// Status: CLEAN
 //
 ///////////////////////////////////////////////////////////////////////////////
 
+// LFC INCLUDES
 #include <lfcbase/Chain.h>
 #include <lfcbase/Exception.h>
 #include <lfcbase/GetLongOpt.h>
@@ -40,38 +26,38 @@
 #include <lfcbase/Timer.h>
 #include <lfcbase/Tokenizer.h>
 #include <lfcbase/Net.h>
-
 #include <lfcxml/Element.h>
 #include <lfcxml/Document.h>
 #include <lfcxml/XMLSuite.h>
 
+// CEGO INCLUDES
 #include "CegoDefs.h"
 #include "CegoAdmAction.h"
+#ifndef CGNOCURSES
 #include "CegoAdmMon.h"
+#endif
 
+// POSIX INCLUDES
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <readline/readline.h>
 #include <readline/history.h>
 
-
 #define USAGE "Usage: cgadm --user=<user>/<password>\n\
-          [ --server=<host>]\n			      \
-          [ --port=<port> ]\n			      \
-          [ --batchfile=<batchfile> ]\n		      \
-          [ --cmd=<command> ]\n			      \
-          [ --logfile=<logfile> ]\n		      \
-          [ --raw ]\n				      \
-          [ --verbose ]\n			      \
-          [ --mon ]\n				      \
-          [ --ignore ]\n			      \
-          [ --debug ] [ --version  ] [ --help ]"
-
+   [ --server=<host>]\n\
+   [ --port=<port> ]\n\
+   [ --batchfile=<batchfile> ]\n\
+   [ --cmd=<command> ]\n\
+   [ --logfile=<logfile> ]\n\
+   [ --raw ]\n\
+   [ --verbose ]\n\
+   [ --mon ]\n\
+   [ --ignore ]\n\
+   [ --debug ] [ --version  ] [ --help ]"
 
 #define DEFAULTSERVER "localhost"
 #define DEFAULTPORT 2000
-#define DEFPITFORMAT "%Y-%m-%d %H:%M:%S"
 
 #define HISTFILE ".cgadm_history"
 #define PROMPT "CGADM > "
@@ -126,7 +112,7 @@ int main(int argc, char **argv)
     Chain password;
     
 
-#ifdef HAVE_MINGW32
+#ifdef HAVE_MINGW
     Chain homeDir(getenv("HOMEPATH"));
     Chain histPath = homeDir + "\\" + Chain(HISTFILE);
 #else
@@ -160,9 +146,9 @@ int main(int argc, char **argv)
 
     if ( longOpt.isSet( Chain("version") ) )
     {
-	cout << CEGO_PRODUCT << " (" << sizeof(long) * 8 << " bit), Version " << CEGO_VERSION 
-	     << " [ lfc : " << __lfcVersionString  
-	     << ", lfcxml : " <<  __lfcxmlVersionString << " ]" << endl;
+	cout << CEGO_PRODUCT << " Admin Client (" << sizeof(long) * 8 << " bit), Version " << CEGO_VERSION 
+	     << " [ lfc: " << __lfcVersionString  
+	     << ", lfcxml: " <<  __lfcxmlVersionString << " ]" << endl;
 	cout << CEGO_COPYRIGHT << endl;
 	exit(0);
     }
@@ -209,7 +195,7 @@ int main(int argc, char **argv)
 	    throw Exception(EXLOC, "Password not set");
 	       
 	if ( logFile.length() > 0 )
-	    pModule = new CegoModule(logFile);
+	    pModule = new CegoModule(logFile, Chain(""));
 	else
 	    pModule = new CegoModule();
 		
@@ -245,14 +231,6 @@ int main(int argc, char **argv)
 
     CegoAdmAction* pAdm = new CegoAdmAction(pAH, pModule, resultMode, rawMode);
 
-    Chain pitFormat(DEFPITFORMAT);    
-    char *pPF = getenv("CGPITFORMAT");
-    if ( pPF )
-    {
-	pitFormat = Chain(pPF);
-    }
-    pAdm->setPitFormat(pitFormat);
-
     int exitCode = 0;
 
     try
@@ -262,8 +240,12 @@ int main(int argc, char **argv)
 	{
 	case MONITOR:
 	{	    
+#ifndef CGNOCURSES
 	    CegoAdmMon mon(pAH);
 	    mon.showMonitor();
+#else
+	    cout << "Curses not enabled" << endl;
+#endif
 	    break;	    	    
 	}
 	case BATCH:
@@ -534,19 +516,28 @@ void helpOverview()
     cout << "   [t18] correct tableset <tableset>" << endl;
     cout << "   [t19] set checkpoint <intval> for <tableset>" << endl;
     cout << "   [t20] set initfile '<initfile>' for <tableset>" << endl;
-    cout << "   [t21] set syssize <intval> for <tableset>" << endl;
-    cout << "   [t22] set tmpsize <intval> for <tableset>" << endl;
-    cout << "   [t23] set tsroot '<root path>' for <tableset>" << endl;
-    cout << "   [t24] set tsticket '<ticket path>' for <tableset>" << endl;
-    cout << "   [t25] set lognum <intval> for <tableset>" << endl;
-    cout << "   [t26] set logsize <intval> for <tableset>" << endl;
-    cout << "   [t27] set sortareasize <intval> for <tableset>" << endl;
-    cout << "   [t28] sync tableset <tableset> [ with '<command>' [ <timeout> ] ] [ message 'system message' ]" << endl;
-    cout << "   [t39] list datafile for <tableset>" << endl;
-    cout << "   [t30] list logfile for <tableset>" << endl;
-    cout << "   [t31] list transaction for <tableset>" << endl;
-    cout << "   [t32] list object for <tableset>" << endl;
-    cout << "   [t33] reset tableset <tableset>" << endl;
+    cout << "   [t21] set appsize <intval> for <tableset>" << endl;
+    cout << "   [t22] set syssize <intval> for <tableset>" << endl;
+    cout << "   [t23] set tmpsize <intval> for <tableset>" << endl;
+    cout << "   [t24] set tsroot '<root path>' for <tableset>" << endl;
+    cout << "   [t25] set tsticket '<ticket path>' for <tableset>" << endl;
+    cout << "   [t26] set lognum <intval> for <tableset>" << endl;
+    cout << "   [t27] set logsize <intval> for <tableset>" << endl;
+    cout << "   [t28] set sortareasize <intval> for <tableset>" << endl;
+    cout << "   [t29] sync tableset <tableset> [ with '<command>' [ <timeout> ] ] [ message 'system message' ]" << endl;
+    cout << "   [t30] list datafile for <tableset>" << endl;
+    cout << "   [t31] list logfile for <tableset>" << endl;
+    cout << "   [t32] list transaction for <tableset>" << endl;
+    cout << "   [t33] list object for <tableset>" << endl;
+    cout << "   [t34] reset tableset <tableset>" << endl;
+    cout << "   [t35] [ enable | disable ] querycache for <tableset>" << endl;
+    cout << "   [t36] list querycache for <tableset>" << endl;
+    cout << "   [t37] set querycache entry <intval> for <tableset>" << endl;
+    cout << "   [t38] set querycache size <intval> for <tableset>" << endl;
+    cout << "   [t39] [ enable | disable ] tablecache for <tableset>" << endl;
+    cout << "   [t40] list tablecache for <tableset>" << endl;
+    cout << "   [t41] set tablecache entry <intval> for <tableset>" << endl;
+    cout << "   [t42] set tablecache size <intval> for <tableset>" << endl;
 
     cout << endl;
     cout << "   User and role administration" << endl;
@@ -560,10 +551,11 @@ void helpOverview()
     cout << "   [u5] list role" << endl;
     cout << "   [u6] create role <role>" << endl;
     cout << "   [u7] show role <role>" << endl;
-    cout << "   [u8] set permission <perm> with <permlist> to role <role>" << endl;
-    cout << "   [u9] assign role <role> to <userid>" << endl;
-    cout << "   [u10] remove role <role> from user <userid>" << endl;
-    cout << "   [u11] drop role <role>" << endl;
+    cout << "   [u8] set permission <perm> with <permlist> for <role>" << endl;
+    cout << "   [u9] remove permission <perm> from <role>" << endl;
+    cout << "   [u10] assign role <role> to <userid>" << endl;
+    cout << "   [u11] remove role <role> from user <userid>" << endl;
+    cout << "   [u12] drop role <role>" << endl;
 
     cout << endl;
     cout << "   Backup and Recovery" << endl;
@@ -571,19 +563,21 @@ void helpOverview()
     cout << "   [b0] set logmng '<progname>' [ timeout <intval> ]" << endl;
     cout << "   [b1] list archlog for <tableset>" << endl;
     cout << "   [b2] enable archlog for <tableset>" << endl;
-    cout << "   [b3] disable archlog for <tableset>" << endl;
-    cout << "   [b4] add archlog <archid> path '<archpath>' to <tableset>" << endl;
-    cout << "   [b5] remove archlog <archid> from <tableset>" << endl;
-    cout << "   [b6] begin backup for <tableset> [ message 'system message' ]" << endl;
-    cout << "   [b7] end backup for <tableset> [ message 'system message' ] [ keep ticket ]" << endl;
-    cout << "   [b8] recover tableset <tableset> [ to <point in time> ]" << endl;
-    cout << "   [b9] export [ xml | binary | plain ]  table <table> from <tableset> to '<filename>'" << endl;
-    cout << "   [b10] export [ xml | binary | plain ] tableset [ structure ] <tableset> to '<filename>'" << endl;
-    cout << "   [b11] import [ xml | binary | plain ] table <table> to <tableset> from '<filename>' [ nologging ]" << endl;
-    cout << "   [b12] import [ xml | binary | plain ] tableset [ structure ] <tableset> from '<filename>' [ nologging ]" << endl;
-    cout << "   [b13] list bustat for <tableset>" << endl;
-    cout << "   [b14] reset bustat for <tableset>" << endl;
-    cout << "   [b15] show logmng" << endl;
+    cout << "   [b3] enable autocorrect for <tableset>" << endl;
+    cout << "   [b4] disable archlog for <tableset>" << endl;
+    cout << "   [b5] add archlog <archid> path '<archpath>' to <tableset>" << endl;
+    cout << "   [b6] remove archlog <archid> from <tableset>" << endl;
+    cout << "   [b7] begin backup for <tableset> [ message 'system message' ]" << endl;
+    cout << "   [b8] end backup for <tableset> [ message 'system message' ] [ keep ticket ]" << endl;
+    cout << "   [b9] recover tableset <tableset> [ to <point in time> ]" << endl;
+    cout << "   [b10] export [ xml | binary | plain ]  table <table> from <tableset> to '<filename>'" << endl;
+    cout << "   [b11] export [ xml | binary | plain ] tableset [ structure ] <tableset> to '<filename>'" << endl;
+    cout << "   [b12] import [ xml | binary | plain ] table <table> to <tableset> from '<filename>' [ nologging ]" << endl;
+    cout << "   [b13] import [ xml | binary | plain ] tableset [ structure ] <tableset> from '<filename>' [ nologging ]" << endl;
+    cout << "   [b14] list bustat for <tableset>" << endl;
+    cout << "   [b15] reset bustat for <tableset>" << endl;
+    cout << "   [b16] show logmng" << endl;
+    cout << "   [b17] list bufile for <tableset>" << endl;
     cout << endl;
     cout << "   You can get a detailed help for each command by typing \"help <topic>\"," << endl;
     cout << "   where topic is the identifier in in the square brackets ( e.g. help t1; )" << endl;
@@ -836,8 +830,8 @@ void helpTopic(const Chain& topic)
     else if ( topic == Chain("u8"))
     {
 	cout << endl;
-	cout << "set permission <permid> with <permlist> to role <role>" << endl;
-	cout << "----------------------------------------------------" << endl;
+	cout << "set permission <permid> with <permlist> for <role>" << endl;
+	cout << "------------------------------------------------------" << endl;
 	cout << endl;
 	cout << "   Sets a permission to the specified role" << endl;
 	cout << "   The permission id (permid) is any valid identifier" << endl;
@@ -847,12 +841,23 @@ void helpTopic(const Chain& topic)
 	cout << "   It consists of a triple describing tableset, filter and right" << endl;
 	cout << "   The triple must be specified in a comma separated list" << endl;
 	cout << endl;
-	cout << "        Sample : set permission p1 with tableset=TS1,filter='*T',right=READ to role R1" << endl;
+	cout << "        Sample : set permission p1 with tableset=TS1,filter='*T',right=READ for R1" << endl;
 	cout << endl;
 	cout << "    Valid right values are READ,WRITE,MODIFY,EXEC or ALL" << endl;
 	cout << endl;
     }
     else if ( topic == Chain("u9"))
+    {
+	cout << endl;
+	cout << "remove permission <permid> from <role>" << endl;
+	cout << "-------------------------------------------" << endl;
+	cout << endl;
+	cout << "   Removes an existing permission from the specified role" << endl;
+	cout << endl;
+	cout << "        Sample : remove permission p1 from R1" << endl;
+	cout << endl;
+    }
+    else if ( topic == Chain("u10"))
     {	
 	cout << endl;
 	cout << "assign role <role> to <userid>" << endl;
@@ -862,7 +867,7 @@ void helpTopic(const Chain& topic)
 	cout << endl;
 	
     }
-    else if ( topic == Chain("u10"))
+    else if ( topic == Chain("u11"))
     {
 	cout << endl;
 	cout << "remove role <role> from <userid>" << endl;
@@ -871,7 +876,7 @@ void helpTopic(const Chain& topic)
 	cout << "   Removes the specified user role from user" << endl;
 	cout << endl;
     }
-    else if ( topic == Chain("u11"))
+    else if ( topic == Chain("u12"))
     {
 	cout << endl;
 	cout << "drop role <role>" << endl;
@@ -907,9 +912,9 @@ void helpTopic(const Chain& topic)
 	cout << "   Define a new tableset. The following attributes must be specified:"<< endl;
 	cout << endl;
 	cout << "   tsroot          - Root directory of the database system and control files." << endl;
-	cout << "   tsticket        - Name of the online backup tableset ticket." << endl;
 	cout << "   syssize         - Size of the system space in number of pages." << endl;
 	cout << "   tmpsize         - Size of the tempory space in number of pages." << endl;
+	cout << "   appsize         - Size of the application space in number of pages." << endl;
 	cout << "   logname         - Prefix of the configured redolog files." << endl;
 	cout << "   logsize         - Size of a redolog file in bytes." << endl;
 	cout << "   lognum          - Number of configured redolog files." << endl;
@@ -921,7 +926,7 @@ void helpTopic(const Chain& topic)
 	cout << endl;
 	cout << "   Sample:" << endl;
 	cout << endl;
-	cout << "      define tableset ts1 tsroot='/cego/ts1' syssize=100 tmpsize=100 logname='/cego/ts1/redolog'" << endl; 
+	cout << "      define tableset ts1 tsroot='/cego/ts1' syssize=100 tmpsize=100 appsize=3000 logname='/cego/ts1/redolog'" << endl; 
 	cout << "             logsize=100000 lognum=3 primary='geek' secondary='sam';" << endl;
 	cout << endl;
     }
@@ -938,7 +943,7 @@ void helpTopic(const Chain& topic)
     {
 	cout << endl;
 	cout << "info tableset <tableset>" << endl;
-	cout << "--------------------------" << endl;
+	cout << "------------------------" << endl;
 	cout << endl;
 	cout << "   Prints a detailed listing of the current tableset configuration values" << endl;
 	cout << endl;
@@ -1107,24 +1112,35 @@ void helpTopic(const Chain& topic)
     else if ( topic == Chain("t21"))
     {
 	cout << endl;
-	cout << "set syssize <intval> for <tableset>" << endl;
+	cout << "set appsize <intval> for <tableset>" << endl;
 	cout << "----------------------------------------" << endl;
 	cout << endl;
-	cout << "   Sets the initial system page size." << endl;
+	cout << "   Sets the initial number of application pages." << endl;
 	cout << "   To perform this operation, the corresponding tableset must be in status defined" << endl;
 	cout << endl;
     }
+
     else if ( topic == Chain("t22"))
+    {
+	cout << endl;
+	cout << "set syssize <intval> for <tableset>" << endl;
+	cout << "----------------------------------------" << endl;
+	cout << endl;
+	cout << "   Sets the initial number of system pages." << endl;
+	cout << "   To perform this operation, the corresponding tableset must be in status defined" << endl;
+	cout << endl;
+    }
+    else if ( topic == Chain("t23"))
     {
 	cout << endl;
 	cout << "set tmpsize <intval> for <tableset>" << endl;
 	cout << "----------------------------------------" << endl;
 	cout << endl;
-	cout << "   Sets the initial temp page size." << endl;
+	cout << "   Sets the initial number of temp pages." << endl;
 	cout << "   To perform this operation, the corresponding tableset must be in status defined" << endl;
 	cout << endl;
     }
-    else if ( topic == Chain("t23"))
+    else if ( topic == Chain("t24"))
     {
 	cout << endl;
 	cout << "set tsroot '<tableset root path>' for <tableset>" << endl;
@@ -1134,7 +1150,7 @@ void helpTopic(const Chain& topic)
 	cout << "   To perform this operation, the corresponding tableset must be in status defined" << endl;
 	cout << endl;
     }
-    else if ( topic == Chain("t24"))
+    else if ( topic == Chain("t25"))
     {
 	cout << endl;
 	cout << "set tsticket '<ticket file>' for <tableset>" << endl;
@@ -1143,7 +1159,7 @@ void helpTopic(const Chain& topic)
 	cout << "   Sets the ticket file for the tableset online backup." << endl;
 	cout << endl;
     }
-    else if ( topic == Chain("t25"))
+    else if ( topic == Chain("t26"))
     {
 	cout << endl;
 	cout << "set lognum <intval> for <tableset>" << endl;
@@ -1153,7 +1169,7 @@ void helpTopic(const Chain& topic)
 	cout << "   To perform this operation, the corresponding tableset must be in status defined" << endl;
 	cout << endl;
     }
-    else if ( topic == Chain("t26"))
+    else if ( topic == Chain("t27"))
     {
 	cout << endl;
 	cout << "set logsize <intval> for <tableset>" << endl;
@@ -1164,7 +1180,7 @@ void helpTopic(const Chain& topic)
 	cout << endl;
 
     }
-    else if ( topic == Chain("t27"))
+    else if ( topic == Chain("t28"))
     {
 	cout << endl;
 	cout << "set sortareasize <intval> for <tableset>" << endl;
@@ -1175,7 +1191,7 @@ void helpTopic(const Chain& topic)
 	cout << "   The new value ist activated when the tableset is restarted" << endl;
 	cout << endl;
     }
-    else if ( topic == Chain("t28"))
+    else if ( topic == Chain("t29"))
     {
 	cout << endl;
 	cout << "sync tableset <tableset> [ with '<command>' ] [ <timeout> ] ] [ message 'system message' ]" << endl;
@@ -1187,7 +1203,7 @@ void helpTopic(const Chain& topic)
 	cout << "   A comment can be added to the sync command, which is stored in the backup stat table" << endl;
 	cout << endl;
     }
-    else if ( topic == Chain("t29"))
+    else if ( topic == Chain("t30"))
     {
 	cout << endl;
 	cout << "list datafile for <tableset>" << endl;
@@ -1197,7 +1213,7 @@ void helpTopic(const Chain& topic)
 	cout << endl;	
     }
 
-    else if ( topic == Chain("t30"))
+    else if ( topic == Chain("t31"))
     {
 	cout << endl;
 	cout << "list logfile for <tableset>" << endl;
@@ -1206,7 +1222,7 @@ void helpTopic(const Chain& topic)
 	cout << "   List all configured datafiles for the given tableset" << endl;
 	cout << endl;	
     }
-    else if ( topic == Chain("t31"))
+    else if ( topic == Chain("t32"))
     {	
 	cout << endl;
 	cout << "list transaction for <tableset>" << endl;
@@ -1214,7 +1230,7 @@ void helpTopic(const Chain& topic)
 	cout << endl;
 	cout << endl;	
     }
-    else if ( topic == Chain("t32"))
+    else if ( topic == Chain("t33"))
     {
 	cout << endl;
 	cout << "list object for <tableset>" << endl;
@@ -1222,7 +1238,7 @@ void helpTopic(const Chain& topic)
 	cout << endl;
 	cout << endl;	
     }
-    else if ( topic == Chain("t33"))
+    else if ( topic == Chain("t34"))
     {
 	cout << endl;
 	cout << "reset tableset <tableset>" << endl;
@@ -1232,7 +1248,78 @@ void helpTopic(const Chain& topic)
 	cout << "   The command just should be executed in case of any undetermined exception." << endl;	
 	cout << endl;	
     }
-
+    else if ( topic == Chain("t35"))
+    {
+	cout << endl;
+	cout << "[ enable | disable ] querycache for <tableset>" << endl;
+	cout << "--------------------------------" << endl;
+	cout << endl;
+	cout << "   Enables or disables querycache for given tableset" << endl;
+	cout << endl;
+    }
+    else if ( topic == Chain("t36"))
+    {
+	cout << endl;
+	cout << "list querycache for <tableset>" << endl;
+	cout << "------------------------------" << endl;
+	cout << endl;
+	cout << "   Lists all queries, which are available in the query cache" << endl;
+	cout << endl;
+    }
+    else if ( topic == Chain("t37"))
+    {
+	cout << endl;
+	cout << "set querycache entry <intval> for <tableset>" << endl;
+	cout << "--------------------------------------------" << endl;
+	cout << endl;
+	cout << "   Sets the maximum number of entries for the query cache" << endl;
+	cout << endl;
+    }
+    else if ( topic == Chain("t38"))
+    {
+	cout << endl;
+	cout << "set querycache size <intval> for <tableset>" << endl;
+	cout << "-------------------------------------------" << endl;
+	cout << endl;
+	cout << "   Sets the maximum size for a single query cache entry ( in byte )" << endl;
+	cout << endl;
+    }
+    else if ( topic == Chain("t39"))
+    {
+	cout << endl;
+	cout << "[ enable | disable ] tablecache for <tableset>" << endl;
+	cout << "--------------------------------" << endl;
+	cout << endl;
+	cout << "   Enables or disables table cache for given tableset" << endl;
+	cout << endl;
+    }
+    else if ( topic == Chain("t40"))
+    {
+	cout << endl;
+	cout << "list tablecache for <tableset>" << endl;
+	cout << "------------------------------" << endl;
+	cout << endl;
+	cout << "   Lists all tables, which are available in the table cache" << endl;
+	cout << endl;
+    }
+    else if ( topic == Chain("t41"))
+    {
+	cout << endl;
+	cout << "set tablecache entry <intval> for <tableset>" << endl;
+	cout << "--------------------------------------------" << endl;
+	cout << endl;
+	cout << "   Sets the maximum number of entries for the table cache" << endl;
+	cout << endl;
+    }
+    else if ( topic == Chain("t42"))
+    {
+	cout << endl;
+	cout << "set tablecache size <intval> for <tableset>" << endl;
+	cout << "-------------------------------------------" << endl;
+	cout << endl;
+	cout << "   Sets the maximum siue for a single table cache entry ( in byte )" << endl;
+	cout << endl;
+    }
     else if ( topic == Chain("b0"))
     {
 	cout << endl;
@@ -1260,6 +1347,15 @@ void helpTopic(const Chain& topic)
 	cout << endl;
     }
     else if ( topic == Chain("b3"))
+    {
+	cout << endl;
+	cout << "enable autocorrect for <tableset>" << endl;
+	cout << "-------------------------------------------------------------" << endl;
+	cout << endl;
+	cout << "   Enables automatic index correction for the given tableset." << endl;
+	cout << endl;
+    }
+    else if ( topic == Chain("b4"))
     {	
 	cout << endl;
 	cout << "disable archlog for <tableset>" << endl;
@@ -1268,7 +1364,7 @@ void helpTopic(const Chain& topic)
 	cout << "   Disables archive log mode for the given tableset." << endl;
 	cout << endl;
     }
-    else if ( topic == Chain("b4"))
+    else if ( topic == Chain("b5"))
     {
 	cout << endl;
 	cout << "add archlog <archid> path '<archpath>' to <tableset>" << endl;
@@ -1277,7 +1373,7 @@ void helpTopic(const Chain& topic)
 	cout << "   Adds the specified archlog destination to the given tableset" << endl;
 	cout << endl;
     }
-    else if ( topic == Chain("b5"))
+    else if ( topic == Chain("b6"))
     {
 	cout << endl;
 	cout << "remove archlog <archlog> from <tableset>" << endl;
@@ -1285,7 +1381,7 @@ void helpTopic(const Chain& topic)
 	cout << "   Removes the specified archlog destination from the given tableset" << endl;
 	cout << endl;
     }
-    else if ( topic == Chain("b6"))
+    else if ( topic == Chain("b7"))
     {
 	cout << endl;
 	cout << "begin backup for <tableset>" << endl;
@@ -1296,7 +1392,7 @@ void helpTopic(const Chain& topic)
 	cout << "   If any recovery is required, the original page is restored during logfile recovery" << endl;
 	cout << endl;
     }
-    else if ( topic == Chain("b7"))
+    else if ( topic == Chain("b8"))
     {
 	cout << endl;
 	cout << "end backup for <tableset>" << endl;
@@ -1306,7 +1402,7 @@ void helpTopic(const Chain& topic)
 	cout << "   Normally this is done, after backup has been finished for all datafiles." << endl;
 	cout << endl;
     }
-    else if ( topic == Chain("b8"))
+    else if ( topic == Chain("b9"))
     {
 	cout << endl;
 	cout << "recover tableset <tableset> [ to <point in time>]" << endl;
@@ -1317,17 +1413,16 @@ void helpTopic(const Chain& topic)
 	cout << "   All required backup datafiles and offline redo logs have to be provided in the appropriate locations." << endl;
 	cout << "   If an online backup should be restored, a valid backup ticket file has to be provided." << endl;
 	cout << endl;
-	cout << "   If point-in-time is used, an appropriate time stamp has to be provided in the following format" << endl;
+	cout << "   If point-in-time is used, an appropriate time stamp has to be provided with the server supported date format." << endl;
+	cout << "   The default format value is <dd.mm.YYYY HH:MM:SS>" << endl;
 	cout << endl;
-	cout << "      <YYYY-mm-dd> HH:MM:SS Z>" << endl;
-	cout << endl;
-	cout << "   Sample : recover tableset TS1 to '2007-08-12 08:12:34 CEST';" << endl;
+	cout << "   Sample : recover tableset TS1 to '08:12:34 09.07.2016';" << endl;
 	cout << endl;
 	cout << "   After the ceovery has been finished, the corresponding tableset can be started up." << endl;
 	cout << endl;
     }
 
-    else if ( topic == Chain("b9"))
+    else if ( topic == Chain("b10"))
     {
 	cout << endl;
 	cout << "export [ xml | binary | plain ] table <tablename> from <tableset> to '<filename>'" << endl;
@@ -1337,7 +1432,7 @@ void helpTopic(const Chain& topic)
 	cout << "   Either xml, binary or plain export mode can be used. If no mode is specified, xml mode is used." << endl;
 	cout << endl;
     }
-    else if ( topic == Chain("b10"))
+    else if ( topic == Chain("b11"))
     {
 	cout << endl;
 	cout << "export [ xml | binary | plain ] tableset [ structure ] <tableset> to '<filename>'" << endl;
@@ -1350,29 +1445,33 @@ void helpTopic(const Chain& topic)
 	cout << endl;
 	
     }
-    else if ( topic == Chain("b11"))
+    else if ( topic == Chain("b12"))
     {
 	cout << endl;
-	cout << "import [ xml | binary | plain ] table <tablename> to <tableset> from '<filename>'" << endl;
-	cout << "---------------------------------------------------------------------------------" << endl;
+	cout << "import [ xml | binary | plain ] table <tablename> to <tableset> from '<filename>' [ nologging ]" << endl;
+	cout << "-----------------------------------------------------------------------------------------------" << endl;
 	cout << endl;
 	cout << "   Import of a single table from the given export file to the given tableset ." << endl;
 	cout << "   Either xml, binary or plain export mode can be used. If no mode is specified, xml mode is used." << endl;
 	cout << endl;
+	cout << "   If the nologging option is added, the online redo log writing is disabled. This results in better import performance." << endl;
+	cout << endl;
     }
-    else if ( topic == Chain("b12"))
+    else if ( topic == Chain("b13"))
     {
 	cout << endl;
-	cout << "import [ xml | binary | plain ] tableset [ structure ] <tableset> from '<filename>'" << endl;
-	cout << "-----------------------------------------------------------------------------------" << endl;
+	cout << "import [ xml | binary | plain ] tableset [ structure ] <tableset> from '<filename>' [ nologging ]" << endl;
+	cout << "-------------------------------------------------------------------------------------------------" << endl;
 	cout << endl;
 	cout << "   Import of a tableset from an export file." << endl;
 	cout << "   For consistency reasons, the name of the given tableset must match with the tableset in the import file." << endl;
 	cout << "   If the structure option is given, no table content is imported." << endl;
 	cout << "   Either xml, binary or plain export mode can be used. If no mode is specified, xml mode is used." << endl;
 	cout << endl;
+	cout << "   If the nologging option is added, the online redo log writing is disabled. This results in better import performance." << endl;
+	cout << endl;
     }
-    else if ( topic == Chain("b13"))
+    else if ( topic == Chain("b14"))
     {
 	cout << endl;
 	cout << "list bustat for <tableset>" << endl;
@@ -1381,7 +1480,7 @@ void helpTopic(const Chain& topic)
 	cout << "   Lists the online backup statistics for the given tableset." << endl;
 	cout << endl;
     }
-    else if ( topic == Chain("b14"))
+    else if ( topic == Chain("b15"))
     {
 	cout << endl;
 	cout << "reset bustat for <tableset>" << endl;
@@ -1390,7 +1489,7 @@ void helpTopic(const Chain& topic)
 	cout << "   Resets the online backup statistics for the given tableset." << endl;
 	cout << endl;
     }
-    else if ( topic == Chain("b15"))
+    else if ( topic == Chain("b16"))
     {
 	cout << endl;
 	cout << "show logmng" << endl;
@@ -1399,11 +1498,17 @@ void helpTopic(const Chain& topic)
 	cout << "   Prints the current log manager configuration" << endl;
 	cout << endl;
     }
-
+    else if ( topic == Chain("b17"))
+    {
+	cout << endl;
+	cout << "list bufile for <tableset>" << endl;
+	cout << "-----------------------------------------------------------------------------------" << endl;
+	cout << endl;
+	cout << "   Lists all relevant files for an online backup for the given tableset" << endl;
+	cout << endl;
+    }
     else
     {
 	helpOverview();
     }
 }
-
-
