@@ -6,51 +6,37 @@
 //     
 // Design and Implementation by Bjoern Lemke
 //     
-// (C)opyright 2000-2010 Bjoern Lemke
-//
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
-// 
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; see the file COPYING.  If not, write to
-// the Free Software Foundation, 59 Temple Place - Suite 330,
-// Boston, MA 02111-1307, USA.
+// (C)opyright 2000-2016 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
 // Class: CegoDatabaseManager
 // 
-// Description: 
+// Description: General database manager
 //
-// Status: QG-2.6
+// Status: CLEAN
 //
 ///////////////////////////////////////////////////////////////////////////////
 
+// LFC INCLUDES
 #include <lfcbase/ThreadLock.h>
 #include <lfcbase/File.h>
-#include <lfcbase/Host.h>
 #include <lfcbase/Datetime.h>
 #include <lfcbase/Sleeper.h>
 #include <lfcbase/Net.h>
 #include <lfcbase/NetHandler.h>
-
 #include <lfcxml/XMLSuite.h>
 #include <lfcxml/Element.h>
 #include <lfcxml/Document.h>
 
+// CEGO INCLUDES
 #include "CegoDatabaseManager.h"
 #include "CegoTypeConverter.h"
 #include "CegoDefs.h"
 #include "CegoXMLdef.h"
 #include "CegoXMLSpace.h"
 
+// POSIX INCLUDES
 #include <string.h>
 #include <stdlib.h>
 
@@ -114,16 +100,26 @@ CegoDatabaseManager::ObjectUseMode CegoDatabaseManager::ObjectRecord::getMode()
     return _mode;
 }
 
-void CegoDatabaseManager::ObjectRecord::setTid(long tid)
+void CegoDatabaseManager::ObjectRecord::setTid(unsigned long long tid)
 {
     _tid = tid;
 } 
 
-long CegoDatabaseManager::ObjectRecord::getTid() const
+unsigned long long CegoDatabaseManager::ObjectRecord::getTid() const
 {
     return _tid;
 }
 
+const Chain& CegoDatabaseManager::ObjectRecord::getName() const
+{
+    return _objName;
+}
+
+const CegoObject::ObjectType CegoDatabaseManager::ObjectRecord::getType() const
+{
+    return _type;
+}
+
 CegoDatabaseManager::ObjectRecord& CegoDatabaseManager::ObjectRecord::operator = ( const CegoDatabaseManager::ObjectRecord& t)
 {
     _tabSetId = t._tabSetId;
@@ -137,7 +133,24 @@ CegoDatabaseManager::ObjectRecord& CegoDatabaseManager::ObjectRecord::operator =
 
 bool CegoDatabaseManager::ObjectRecord::operator == ( const CegoDatabaseManager::ObjectRecord& r)
 {
-    if ( _objName == r._objName && _type == r._type)
+
+    bool typeMatch=false;
+    if ( ( _type == CegoObject::PAVLTREE || _type == CegoObject::UAVLTREE || _type == CegoObject::AVLTREE )
+	 && ( r._type == CegoObject::PAVLTREE || r._type == CegoObject::UAVLTREE || r._type == CegoObject::AVLTREE ) )
+    {
+	typeMatch=true;
+    }
+    else if ( ( _type == CegoObject::PBTREE || _type == CegoObject::UBTREE || _type == CegoObject::BTREE ) 
+	&& ( r._type == CegoObject::PBTREE || r._type == CegoObject::UBTREE || r._type == CegoObject::BTREE ) )
+    {
+	typeMatch=true;
+    }
+    else 
+    {
+	typeMatch = _type == r._type;
+    }
+
+    if ( _tabSetId == r._tabSetId && _objName == r._objName && typeMatch )
 	return true;
     return false;
 }
@@ -210,7 +223,6 @@ void CegoDatabaseManager::CopyRecord::setId(int id)
     _id = id;
 }
 
-
 CegoDatabaseManager::CopyRecord& CegoDatabaseManager::CopyRecord::operator = ( const CegoDatabaseManager::CopyRecord& cr)
 {
     _id = cr._id;
@@ -321,7 +333,7 @@ bool CegoDatabaseManager::DbSessionRecord::operator == ( const CegoDatabaseManag
 // CegoDatabaseManager //
 /////////////////////////
 
-CegoDatabaseManager::CegoDatabaseManager(const Chain& xmlDef, const Chain& lckFileName, const Chain& logFile, CegoDbHandler::ProtocolType protType) :  CegoBufferPool(xmlDef, logFile)
+CegoDatabaseManager::CegoDatabaseManager(const Chain& xmlDef, const Chain& lckFileName, const Chain& logFile, const Chain& progName, CegoDbHandler::ProtocolType protType) :  CegoBufferPool(xmlDef, logFile, progName)
 {
 
     _protType = protType;
@@ -346,11 +358,14 @@ CegoDatabaseManager::CegoDatabaseManager(const Chain& xmlDef, const Chain& lckFi
     for ( int i=0; i<TABMNG_MAXTABSET; i++)
     {
 	_recoveryMode[i] = OFF;
+	_pQueryCache[i] = 0;
+	_pTableCache[i]= 0;
     }
     _nextCopyId=1;
 
     _logConfigured=false;
 
+    
     _modId = getModId("CegoDatabaseManager");
 }
 
@@ -358,6 +373,104 @@ CegoDatabaseManager::~CegoDatabaseManager()
 {
     File lckFile(_lckFileName);
     lckFile.remove();
+
+    for ( int i=0; i<TABMNG_MAXTABSET; i++)
+    {
+	if ( _pQueryCache[i] )
+	    delete _pQueryCache[i];
+	if ( _pTableCache[i] )
+	    delete _pTableCache[i];	
+    }
+}
+
+void CegoDatabaseManager::allocateQueryCache(const Chain& tableSet)
+{
+    if ( getQueryCacheMode(tableSet) )
+    {	
+	int numEntry = getMaxQueryCacheEntry(tableSet);
+	int cacheSize = getMaxQueryCacheSize(tableSet);
+
+	int tabSetId = getTabSetId(tableSet);
+	
+	if ( _pQueryCache[tabSetId] )
+	    delete _pQueryCache[tabSetId];
+	
+	_pQueryCache[tabSetId] = new CegoQueryCache(numEntry,cacheSize);
+    }
+}
+
+void CegoDatabaseManager::allocateTableCache(const Chain& tableSet)
+{
+    if ( getTableCacheMode(tableSet) )
+    {
+
+	int numEntry = getMaxTableCacheEntry(tableSet);
+	int cacheSize = getMaxTableCacheSize(tableSet);
+
+	int tabSetId = getTabSetId(tableSet);
+
+	if ( _pTableCache[tabSetId] )
+	    delete _pTableCache[tabSetId];
+	_pTableCache[tabSetId] = new CegoTableCache(numEntry,cacheSize, this);
+    }
+}
+
+void CegoDatabaseManager::releaseQueryCache(const Chain& tableSet)
+{
+
+    int tabSetId = getTabSetId(tableSet);
+    
+    if ( _pQueryCache[tabSetId] )
+    {
+	delete _pQueryCache[tabSetId];
+	_pQueryCache[tabSetId]=0;
+    }
+}
+
+void CegoDatabaseManager::releaseTableCache(const Chain& tableSet)
+{
+    int tabSetId = getTabSetId(tableSet);
+    
+    if ( _pTableCache[tabSetId] )
+    {
+	delete _pTableCache[tabSetId];
+	_pTableCache[tabSetId]=0;
+    }
+}
+
+void CegoDatabaseManager::cleanCache(int tabSetId, CegoObject::ObjectType objType, const Chain& objName)
+{
+    if ( _pQueryCache[tabSetId] )
+    {
+	_pQueryCache[tabSetId]->invalidate(CegoObject(objType, objName, tabSetId));
+    }
+    
+    if ( _pTableCache[tabSetId] && objType == CegoObject::TABLE )
+    {
+	_pTableCache[tabSetId]->invalidate(tabSetId, objName);
+    }
+}
+
+CegoQueryCache* CegoDatabaseManager::getQueryCache(const Chain& tableSet) const
+{
+    int tabSetId = getTabSetId(tableSet);
+    return _pQueryCache[tabSetId];
+}
+
+CegoQueryCache* CegoDatabaseManager::getQueryCache(int tabSetId) const
+{
+    return _pQueryCache[tabSetId];
+}
+
+CegoTableCache* CegoDatabaseManager::getTableCache(const Chain& tableSet) const
+{
+    int tabSetId = getTabSetId(tableSet);
+    return _pTableCache[tabSetId];
+}
+
+CegoTableCache* CegoDatabaseManager::getTableCache(int tabSetId) const
+{
+    return _pTableCache[tabSetId];
 }
 
 void CegoDatabaseManager::beat()
@@ -436,7 +549,6 @@ bool CegoDatabaseManager::nextRecovery(Chain& tableSet)
     return false;
 }
 
-
 bool CegoDatabaseManager::nextCopy(int& id, Chain& tableSet, Chain& targetHost, Chain& mediatorHost, Chain& user, Chain& passwd)
 {
     PW();
@@ -481,7 +593,6 @@ void CegoDatabaseManager::setCopyStatus(int id, const Chain& msg)
     }
     V();
     return;
-
 }
 
 Element* CegoDatabaseManager::getCopyInfo()
@@ -510,20 +621,20 @@ Element* CegoDatabaseManager::getCopyInfo()
     return pCopyInfo;
 }
 
-
-void CegoDatabaseManager::useObject(int tabSetId, const Chain& objName, CegoObject::ObjectType type, ObjectUseMode mode, long tid)
+void CegoDatabaseManager::useObject(int tabSetId, const Chain& objName, CegoObject::ObjectType type, ObjectUseMode mode, unsigned long long tid)
 {
 
+    // In SHARED mode, an object could be use in parallel by several threads. Normally, this is done, if the object is accessed read only.
 
     if ( mode == SHARED )
     {
-	// cout << "Using object " << objName << " " << type << " shared" << endl;
+
         PW();
         ObjectRecord *pOR = _objList.Find(ObjectRecord(tabSetId, objName, type));
         if ( pOR == 0 )
         {
             V();
-	    Chain msg = Chain("Cannot access object ") + objName;
+	    Chain msg = Chain("Cannot access object <") + objName + Chain(">");
             throw Exception(EXLOC, msg); 
         }
 
@@ -534,66 +645,50 @@ void CegoDatabaseManager::useObject(int tabSetId, const Chain& objName, CegoObje
 	}
 	else
 	{
-	    while ( pOR && ( pOR->getMode() == EXCLUSIVE || pOR->getMode() == EXCLUSIVE_WRITE ) )
+	    int numTries=0;
+
+	    while ( pOR && pOR->getMode() == EXCLUSIVE_WRITE && numTries < DBM_MAXLOCKTRIES )
 	    {
 		V();
+
+		if ( numTries > 0 )
+		{
+		    log(_modId, Logger::NOTICE, Chain("Shared lock delay on ") + objName + Chain(" ( ") + Chain(numTries) + Chain(" tries )"));
+		    Sleeper ns;
+		    ns.milliSleep(DBM_LOCKDELAY);
+		}
+
 		PW();
+
 		pOR = _objList.Find(ObjectRecord(tabSetId, objName, type));
+		
+		if ( pOR == 0 )
+		{
+		    V();
+		    Chain msg = Chain("Cannot access object ") + objName;
+		    throw Exception(EXLOC, msg);
+		}
+		numTries++;
 	    }
-	    if ( pOR == 0 )
+
+	    if ( numTries == DBM_MAXLOCKTRIES )
 	    {
 		V();
-		Chain msg = Chain("Cannot access object ") + objName;
+		Chain msg = Chain("Access timeout on object ") + objName;
 		throw Exception(EXLOC, msg); 
 	    }
-	    
-	    
+
 	    pOR->incUsed();
 	    V();
 	}
     }
-    else if ( mode == EXCLUSIVE )
-    {
-	// cout << "Using object " << objName << " " << type << " exclusive" << endl;
-        PW();
-        ObjectRecord *pOR = _objList.Find(ObjectRecord(tabSetId, objName, type));
-
-        if ( pOR == 0 )
-        {
-            V();
-            Chain msg = Chain("Cannot access object ") + objName;
-            throw Exception(EXLOC, msg);
-        }
-	
-	pOR->setMode(mode);
-	pOR->setTid(tid);
-	    
-	while ( pOR && pOR->numUsed() != 0 )
-	{
-	    V();
-	    Sleeper ns;
-	    ns.nanoSleep(10000);
-	    PW();
-	    
-	    pOR = _objList.Find(ObjectRecord(tabSetId, objName, type));            
-	}
-
-        if ( pOR == 0 )
-        {
-            V();
-            Chain msg = Chain("Cannot access object ") + objName;
-
-            throw Exception(EXLOC, msg);
-        }
-        pOR->incUsed();
-        V();
-    }
+    
+    // In EXCLUSIVE_WRITE mode, an object is accessed exclusively by one thread only
+    
     else if ( mode == EXCLUSIVE_WRITE )
     {
-	// cout << "Using object " << objName << " " << type << " exclusive_write " << endl;
         PW();
         ObjectRecord *pOR = _objList.Find(ObjectRecord(tabSetId, objName, type));
-
         if ( pOR == 0 )
         {
             V();
@@ -601,10 +696,6 @@ void CegoDatabaseManager::useObject(int tabSetId, const Chain& objName, CegoObje
             throw Exception(EXLOC, msg);
         }
 	
-	// cout << "NumUsed = " << pOR->numUsed() << endl;
-
-	// if ( pOR->getMode() == SHARED  )
-
 	if ( pOR->getMode() == SHARED && pOR->numUsed() == 0 )
 	{
 	    pOR->setMode(mode);
@@ -612,43 +703,55 @@ void CegoDatabaseManager::useObject(int tabSetId, const Chain& objName, CegoObje
 	}
 	else
 	{	    
-	    // while ( pOR && pOR->getMode() != SHARED )
-	    while ( pOR && pOR->numUsed() != 0 )
+	    int numTries=0;
+	    while ( pOR && pOR->numUsed() != 0 && numTries < DBM_MAXLOCKTRIES )
 	    {
+
 		V();
-		Sleeper ns;
-		ns.nanoSleep(10000);
+
+		if ( numTries > 0 )
+		{
+		    log(_modId, Logger::NOTICE, Chain("Exclusive write lock delay on ") + objName + Chain(" ( ") + Chain(numTries) + Chain(" tries )"));
+		    Sleeper ns;
+		    ns.milliSleep(DBM_LOCKDELAY);
+		    		    
+		}
+		
 		PW();
 
-		pOR = _objList.Find(ObjectRecord(tabSetId, objName, type));            
+		pOR = _objList.Find(ObjectRecord(tabSetId, objName, type));
+		if ( pOR == 0 )
+		{
+		    V();
+		    Chain msg = Chain("Cannot access object ") + objName;
+		    throw Exception(EXLOC, msg);
+		}
+
+		numTries++;
+
 	    }
 
-	    if ( pOR == 0 )
+	    if ( numTries == DBM_MAXLOCKTRIES )
 	    {
 		V();
-		Chain msg = Chain("Cannot access object ") + objName;
-		throw Exception(EXLOC, msg);
-	    }
+		Chain msg = Chain("Access timeout on object ") + objName;
+		throw Exception(EXLOC, msg); 
+	    }	       		
 
 	    pOR->setMode(mode);
+	    pOR->setTid(tid);
 
 	}
 
-        if ( pOR == 0 )
-        {
-            V();
-	    Chain msg = Chain("Cannot access object ") + objName;
-            throw Exception(EXLOC, msg);
-        }
         pOR->incUsed();
         V();
     }
     return;
 }
 
-void CegoDatabaseManager::unuseObject(int tabSetId, const Chain& objName, CegoObject::ObjectType type, ObjectUseMode mode)
+void CegoDatabaseManager::unuseObject(int tabSetId, const Chain& objName, CegoObject::ObjectType type)
 {
-    // cout << "Unusing object " << objName << " " << type  << endl;
+    
     PW();
     ObjectRecord *pOR = _objList.Find(ObjectRecord(tabSetId, objName, type));
 
@@ -661,12 +764,10 @@ void CegoDatabaseManager::unuseObject(int tabSetId, const Chain& objName, CegoOb
     }
     
     pOR->decUsed();
-
-    if ( mode != SHARED )
-    {
-        pOR->setMode(SHARED);
-	pOR->setTid(0);
-    }
+    
+    pOR->setMode(SHARED);
+    pOR->setTid(0);
+    
     V();    
 }
 
@@ -686,13 +787,9 @@ void CegoDatabaseManager::removeObject(int tabSetId, const Chain& objName, CegoO
 
 bool CegoDatabaseManager::objectExists(int tabSetId, const Chain& objName, CegoObject::ObjectType type)
 {
-
     PR();
-    
     ObjectRecord *pTR = _objList.Find(ObjectRecord(tabSetId, objName, type));
-
-    V();
-    
+    V();    
     if ( pTR ) 
     {	
 	return true;
@@ -700,8 +797,19 @@ bool CegoDatabaseManager::objectExists(int tabSetId, const Chain& objName, CegoO
     else
     {
 	return false;
-    }    
+    }
+}
 
+void CegoDatabaseManager::printObjectList()
+{
+    PW();
+    ObjectRecord *pOR = _objList.First();
+    while ( pOR )
+    {
+	cout << "ObjListEntry : " << pOR->getName() << " Type = " << pOR->getType() << endl;
+	pOR = _objList.Next();
+    }
+    V();
 }
 
 void CegoDatabaseManager::setThreadInfo(int numDbThread, int numAdmThread, int numLogThread)
@@ -755,22 +863,7 @@ void CegoDatabaseManager::decreaseActiveLogThread()
     _activeLogThread--;
 }
 
-Chain CegoDatabaseManager::getSysFileName(const Chain& tableSet)
-{
-    Chain tsRoot = CegoXMLSpace::getTSRoot(tableSet);
-    Chain dbName = CegoXMLSpace::getDbName();
-    return tsRoot + Chain("/") + dbName + Chain("_") + Chain(tableSet) + Chain(SYSSUFFIX);
-}
-
-
-Chain CegoDatabaseManager::getTmpFileName(const Chain& tableSet)
-{
-    Chain tsRoot = CegoXMLSpace::getTSRoot(tableSet);
-    Chain dbName = CegoXMLSpace::getDbName();
-    return tsRoot + Chain("/") + dbName + Chain("_") + Chain(tableSet) + Chain(TEMPSUFFIX);
-}
-
-void CegoDatabaseManager::getDBMLockStat(Chain& lockName, long& lockCount, long &numRdLock, long &numWrLock, long &sumRdDelay, long &sumWrDelay)
+void CegoDatabaseManager::getDBMLockStat(Chain& lockName, int& lockCount, unsigned long long &numRdLock, unsigned long long &numWrLock, unsigned long long &sumRdDelay, unsigned long long &sumWrDelay)
 {
     lockName = dbmLock.getId();
     lockCount= dbmLock.numLockTry();
@@ -784,20 +877,17 @@ void CegoDatabaseManager::getDBMLockStat(Chain& lockName, long& lockCount, long
 	sumRdDelay = dbmLock.sumReadDelay() / LCKMNG_DELRES;
     if ( dbmLock.numWriteLock() > 0 )
 	sumWrDelay = dbmLock.sumWriteDelay() / LCKMNG_DELRES;
-
 }
 
-
 CegoDistDbHandler* CegoDatabaseManager::allocateSession(const Chain& hostName, const Chain& tableSet,
 							const Chain& userName, const Chain& password)
 {
+    PW();    
 
-    PW();
-
+    CegoDistDbHandler* pSession = 0;
     
     try
     {
-    
 	DbSessionRecord *pSR = _dbSessionList.First();
 	while ( pSR )
 	{
@@ -819,24 +909,22 @@ CegoDistDbHandler* CegoDatabaseManager::allocateSession(const Chain& hostName, c
 	
 	// create new session
 
-	CegoDistDbHandler* pSession = createSession(hostName, tableSet, userName, password);
+	pSession = createSession(hostName, tableSet, userName, password);
 	_dbSessionList.Insert( DbSessionRecord(hostName, tableSet, userName, pSession));
 
-	V();
-
-	return pSession;
-
     }
     catch ( Exception e )
     {
 	V();
 	throw Exception(EXLOC, "Cannot allocate db session", e);
     }
+
+    V();
+    return pSession;
 }
 
 void CegoDatabaseManager::releaseSession(CegoDistDbHandler* pHandler)
 {
-
     PW();
     
     DbSessionRecord *pSR = _dbSessionList.First();
@@ -857,12 +945,10 @@ void CegoDatabaseManager::releaseSession(CegoDistDbHandler* pHandler)
     
     Chain msg = Chain("Cannot release session for unknown db handle"); 
     throw Exception(EXLOC, msg);
-
 }
 
 void CegoDatabaseManager::cleanSession(int lifetime)
 {
-
     PW();
     
     Datetime dt;
@@ -882,19 +968,16 @@ void CegoDatabaseManager::cleanSession(int lifetime)
 	pSR = _dbSessionList.Next();
     }
     
-    V();
-    
+    V();   
 }
 
-
 Element* CegoDatabaseManager::getSessionInfo(int lifetime)
 {
     Element* pSessionInfo = new Element(XML_DBSESSIONINFO_ELEMENT);
     
     DbSessionRecord *pSR = _dbSessionList.First();
     while ( pSR )
-    {
-	
+    {	
 	Element *pN = new Element(XML_DBSESSION_ELEMENT);
 	pN->setAttribute(XML_HOSTNAME_ATTR, pSR->getHostName());
 	pN->setAttribute(XML_TABLESET_ATTR, pSR->getTableSet());
@@ -911,12 +994,9 @@ Element* CegoDatabaseManager::getSessionInfo(int lifetime)
 
 	pSR = _dbSessionList.Next();
     }
-
-
     return pSessionInfo;
 }
 
-
 CegoDistDbHandler* CegoDatabaseManager::createSession(const Chain& hostName, const Chain& tableSet,
 						  const Chain& userName, const Chain& password)
 {
@@ -975,7 +1055,6 @@ void CegoDatabaseManager::closeSession(CegoDistDbHandler* pSH)
 
     delete pSH;
     delete pN;
-
 }
 
 void CegoDatabaseManager::configureLogger(Logger::LogLevel level)
@@ -1012,6 +1091,13 @@ void CegoDatabaseManager::configureLogger()
     }
 }
 
+bool CegoDatabaseManager::verifyJDBC(const Chain& user)
+{
+    SetT<Chain> roleSet;
+    getRoleSet(user, roleSet);
+    return roleSet.Find(Chain(ROLE_JDBC));
+}
+
 bool CegoDatabaseManager::verifyAccess(const int tabSetId, const Chain& objName, CegoObject::ObjectType type, CegoXMLSpace::AccessMode mode, const Chain& user)
 {
 
@@ -1080,7 +1166,6 @@ void CegoDatabaseManager::initLogFiles(const Chain& tableSet, bool overwrite)
 
 void CegoDatabaseManager::releaseLogFiles(const Chain& tableSet, bool waitForArch)
 {
-
     ListT<Chain> lfList;
     ListT<int> sizeList;
     ListT<Chain> statusList;
@@ -1093,11 +1178,10 @@ void CegoDatabaseManager::releaseLogFiles(const Chain& tableSet, bool waitForArc
 
     while ( pLog && pStatus ) 
     {	
-
 	if ( *pStatus == Chain(XML_ACTIVE_VALUE) )
 	{
 	    setLogFile(tabSetId, *pLog, true);
-	    long minlsn = getMinLSN(tabSetId);
+	    unsigned long long minlsn = getMinLSN(tabSetId);
 	    if ( minlsn > 0 )
 	    {
 		log(_modId, Logger::NOTICE, Chain("Releasing logfile ") + *pLog + Chain(" LSN=") + Chain(minlsn));
@@ -1108,7 +1192,6 @@ void CegoDatabaseManager::releaseLogFiles(const Chain& tableSet, bool waitForArc
 	pLog = lfList.Next();	
     }
 
-
     if ( waitForArch )
     {
 
@@ -1135,20 +1218,12 @@ void CegoDatabaseManager::releaseLogFiles(const Chain& tableSet, bool waitForArc
 		pStatus = statusList.Next();
 	    }
 
-
 	    lfList.Empty();
 	    sizeList.Empty();
 	    statusList.Empty();
 
 	    Sleeper s;
 	    s.secSleep(LOGMNG_RECOVERY_DELAY);
-
 	}
     }
-
 }
-
-
-
-
-
