@@ -6,44 +6,31 @@
 //                                                         
 // Design and Implementation by Bjoern Lemke               
 //     
-// (C)opyright 2000-2012 Bjoern Lemke
-//
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
-// 
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; see the file COPYING.  If not, write to
-// the Free Software Foundation, 59 Temple Place - Suite 330,
-// Boston, MA 02111-1307, USA.
+// (C)opyright 2000-2016 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
 // Class: CegoDbHandler
 // 
-// Description: 
+// Description: Database handler class to the access the database backend via network 
 //
-// Status: QG-2.13
+// Status: CLEAN
 //
 ///////////////////////////////////////////////////////////////////////////////
 
+// LFC INCLUDES
 #include <lfcbase/AESCrypt.h>
 #include <lfcbase/Datetime.h>
-
 #include <lfcxml/Element.h>
 
+// CEGO INCLUDES
 #include "CegoTypeConverter.h"
 #include "CegoDbHandler.h"
 #include "CegoXMLdef.h"
 #include "CegoXMLHelper.h"
 #include "CegoDefs.h"
 
+// POSIX INCLUDES
 #include <string.h>
 #include <stdlib.h>
 
@@ -68,11 +55,16 @@
 
 #define SER_BLOBINFO "bli"
 #define SER_BLOBSIZE "bls"
-
 #define SER_PUTBLOB "blp"
 #define SER_GETBLOB "blg"
 #define SER_DELBLOB "bld"
 
+#define SER_CLOBINFO "cli"
+#define SER_CLOBSIZE "cls"
+#define SER_PUTCLOB "clp"
+#define SER_GETCLOB "clg"
+#define SER_DELCLOB "cld"
+
 // distributed requests
 #define SER_CREATETABLE "createtable"
 #define SER_CREATEVIEW "creatview"
@@ -104,8 +96,6 @@ CegoDbHandler::CegoDbHandler(NetHandler *pN, CegoDbHandler::ProtocolType pt, Ceg
     _pModule = pModule;
     _pRow = 0;
 
-    _pSer = new CegoSerial(_pN);
-
     _protType = pt;
     
     if ( _protType == CegoDbHandler::XML )
@@ -114,9 +104,12 @@ CegoDbHandler::CegoDbHandler(NetHandler *pN, CegoDbHandler::ProtocolType pt, Ceg
 	pDoc->setAttribute(XML_VERSION_ATTR, XML_VERSION_VALUE);    
 	_xml.setDocument(pDoc);
     }
-
+    else
+    {
+	_pSer = new CegoSerial(_pN);
+    }
+  
     _wasReset = false;
-
     _modId = _pModule->getModId("CegoDbHandler");
 }
 
@@ -128,13 +121,15 @@ CegoDbHandler::~CegoDbHandler()
 	pDoc->clear();
 	delete pDoc;
     }
-    if ( _pSer )
-	delete _pSer;
+    else
+    {
+	if ( _pSer )
+	    delete _pSer;
+    }
 }
 
 CegoDbHandler::ResultType CegoDbHandler::requestSession(const Chain& tableSet, const Chain& user, const Chain& password, bool doEncrypt)
 {   
-
 #ifdef CGDEBUG
     _pModule->log(_modId, Logger::DEBUG, Chain("Request session for user ") + user + Chain("/") + password);
 #endif
@@ -182,8 +177,7 @@ CegoDbHandler::ResultType CegoDbHandler::requestSession(const Chain& tableSet, c
 	_pSer->writeChain(user);
 	_pSer->writeChain(pwd);
 	
-	return sendSerialReq();
-	
+	return sendSerialReq();	
     }
 }
 
@@ -212,7 +206,6 @@ bool CegoDbHandler::acceptSession()
 
 	if ( _protType == CegoDbHandler::XML )
 	{
-	    
 	    _xml.getDocument()->clear();
 	    _xml.setChain( _pN->getMsg() );	
 #ifdef CGDEBUG
@@ -276,15 +269,11 @@ bool CegoDbHandler::acceptSession()
 	}
 	else
 	{
-
 #ifdef CGDEBUG
-
 	    _pModule->log(_modId, Logger::DEBUG, Chain("--- SER ---"));
 	    _pModule->log(_modId, Logger::DEBUG, Chain(_pN->getMsg(), _pN->getMsgSize()));
 	    _pModule->log(_modId, Logger::DEBUG, Chain("--- --- ---"));
-
 #endif
-
 	    _pSer->reset();
 	    Chain req = _pSer->readChain();
 
@@ -306,7 +295,6 @@ bool CegoDbHandler::acceptSession()
 		// cout << "SESSION OK" << endl;
 		return true;
 	    }
-
 	}	
     }
     catch ( Exception e)
@@ -319,17 +307,13 @@ bool CegoDbHandler::acceptSession()
 	
 	_pN->sendNack();
 	return false;
-	
     }
 }
 
 CegoDbHandler::ResultType CegoDbHandler::reqQueryOp(const Chain& cmd)
 {
-
-
     if ( _protType == CegoDbHandler::XML )
     {
-		
 	_xml.getDocument()->clear();
 	
 	Element* pRoot = new Element(XML_FRAME_ELEMENT);
@@ -386,17 +370,14 @@ CegoDbHandler::ResultType CegoDbHandler::reqQueryOp(const Chain& cmd)
 	_pSer->reset();
 	_pSer->writeChain(Chain(SER_QUERY));
 	_pSer->writeChain(cmd);
-	return sendSerialReq();	
-	
+	return sendSerialReq();
     }
 }
 
-CegoDbHandler::ResultType CegoDbHandler::reqQueryAbort(long idx)
+CegoDbHandler::ResultType CegoDbHandler::reqQueryAbort(unsigned long long idx)
 {    
-    
     if ( _protType == CegoDbHandler::XML )
-    {
-	
+    {	
 	Element* pRoot = new Element(XML_FRAME_ELEMENT);
 	pRoot->setAttribute(XML_TID_ATTR, Chain(idx));
 	return sendXMLReq(XML_QUERYABORT_REQUEST, pRoot);
@@ -415,45 +396,39 @@ const Chain& CegoDbHandler::getMsg()
     return _serMsg;
 }
 
-
-long CegoDbHandler::getAffected()
+unsigned long long CegoDbHandler::getAffected()
 {
     if ( _protType == CegoDbHandler::XML )
     {
-    
 	Element *pRoot = _xml.getDocument()->getRootElement();
 	
 	if ( pRoot )
 	{
-	    _serAffected =  pRoot->getAttributeValue(XML_AFFECTED_ATTR).asLong();
+	    _serAffected =  pRoot->getAttributeValue(XML_AFFECTED_ATTR).asUnsignedLongLong();
 	}
     }
-
     return _serAffected;
 }
 
-long CegoDbHandler::getTid()
+unsigned long long CegoDbHandler::getTid() const
 {
-    if ( _protType == CegoDbHandler::XML )
-    {
-
-	Element *pRoot = _xml.getDocument()->getRootElement();
-	
-	if ( pRoot )
-	{
-	    return pRoot->getAttributeValue(XML_TID_ATTR).asLong();
-	}
-	else
-	{
-	    throw Exception(EXLOC, "No root element found");
-	}
-    }
     return _serTid;
-    {
-	return _serTid;
-    }
 }
 
+const Chain& CegoDbHandler::getDbName()
+{
+    return _serDbName;
+}
+
+const Chain& CegoDbHandler::getDbVersion()
+{
+    return _serDbVersion;
+}
+
+const Chain& CegoDbHandler::getDateFormat()
+{
+    return _serDateFormat;
+}
 
 const Chain& CegoDbHandler::getTableSet()
 {
@@ -474,38 +449,30 @@ const Chain& CegoDbHandler::getFormat()
 {
     if ( _protType == CegoDbHandler::XML )
     {
-
-	Element *pRoot = _xml.getDocument()->getRootElement();
-	
+	Element *pRoot = _xml.getDocument()->getRootElement();	
 	if ( pRoot )
 	{
 	    _serFormat = pRoot->getAttributeValue(XML_FORMAT_ATTR);
 	}
     }
-
     return _serFormat;
 }
 
 const ListT<CegoField>& CegoDbHandler::getSchema()
 {
-
     if ( _protType == CegoDbHandler::XML )
-    {
-	
-	Element *pRoot = _xml.getDocument()->getRootElement();
-	
+    {	
+	Element *pRoot = _xml.getDocument()->getRootElement();	
 	if ( pRoot )
 	{
-	    
 	    _serSchema.Empty();
-
+	    
 	    Chain tableName = pRoot->getAttributeValue(XML_TABLENAME_ATTR);
 	    
 	    ListT<Element*> childList = pRoot->getChildren(Chain(XML_SCHEMA_ELEMENT));
 	    Element **pRow = childList.First();
 	    while ( pRow ) 
-	    {
-		
+	    {	
 		Chain colTable = (*pRow)->getAttributeValue(XML_TABLENAME_ATTR);
 		Chain colName = (*pRow)->getAttributeValue(XML_COLNAME_ATTR);
 		Chain colType = (*pRow)->getAttributeValue(XML_COLTYPE_ATTR);
@@ -535,9 +502,7 @@ const ListT<CegoField>& CegoDbHandler::getSchema()
 	    }
 	}
     }
-
     return _serSchema;
-
 }
 
 Element* CegoDbHandler::getObjElement()
@@ -564,7 +529,6 @@ Element* CegoDbHandler::getObjElement()
 
 CegoObject::ObjectType CegoDbHandler::getObjType()
 {
-
     if ( _protType == CegoDbHandler::XML )
     {
 	Element *pRoot = _xml.getDocument()->getRootElement();
@@ -655,7 +619,6 @@ CegoDbHandler::ResultType CegoDbHandler::receiveTableData(const ListT<CegoField>
 
 CegoDbHandler::ResultType CegoDbHandler::receiveTableData(const ListT<CegoField>& schema, ListT<CegoFieldValue>& fvl)
 {
-
 #ifdef CGDEBUG
     _pModule->log(_modId, Logger::DEBUG, Chain("Receiving table data ..."));
 #endif
@@ -778,7 +741,6 @@ CegoDbHandler::ResultType CegoDbHandler::receiveTableData(const ListT<CegoField>
     }
     else
     {
-	
 	// cout << "Num Ahead = " << _pSer->numAhead() << endl;
 
 	if ( _pSer->numAhead() > 0 )
@@ -803,7 +765,6 @@ CegoDbHandler::ResultType CegoDbHandler::receiveTableData(const ListT<CegoField>
 	    }
 	    return res;
 	}
-
     }
 }
 
@@ -848,7 +809,6 @@ void CegoDbHandler::abortQuery()
     {
 	_pSer->reset();
     }
-    
 }
 
 void CegoDbHandler::resetQuery()
@@ -861,12 +821,10 @@ void CegoDbHandler::resetQuery()
 
     _rowList.Empty();
     _pRow = 0;
-
 }
 
 void CegoDbHandler::sendCollectedData()
 {
-
     char c = QUERY_MOREDATA;
 
     if ( _protType == CegoDbHandler::XML )
@@ -920,7 +878,7 @@ void CegoDbHandler::sendCollectedData()
 	}
 	else
 	{
-
+	    
 #ifdef CGDEBUG
 	    _pModule->log(_modId, Logger::DEBUG, Chain("--- SER ---"));
 	    _pModule->log(_modId, Logger::DEBUG, Chain(_pN->getMsg(), _pN->getMsgSize()));
@@ -936,7 +894,6 @@ void CegoDbHandler::sendCollectedData()
     {
 	throw Exception(EXLOC, "Unknown query sync");
     }
-
     return;
 }
 
@@ -952,12 +909,11 @@ bool CegoDbHandler::wasReset()
 	_wasReset = false;
 	return false;
     }
-    
 }
+
 void CegoDbHandler::sendFinishData()
 {
-
-    char c = QUERY_MOREDATA;
+    char c;
 
     if ( _protType == CegoDbHandler::XML )
     {
@@ -1023,8 +979,17 @@ void CegoDbHandler::sendFinishData()
 
 void CegoDbHandler::sendErrorData(const Chain& msg)
 {
+    char c;
 
-    char c = _pN->recvChar();
+    if ( _protType == CegoDbHandler::XML )
+    {
+	c = _pN->recvChar();
+    }
+    else
+    {
+	if ( _serSync )
+	    c = _pN->recvChar();
+    }
     
     if ( c == QUERY_ABORT )
     {
@@ -1037,14 +1002,11 @@ void CegoDbHandler::sendErrorData(const Chain& msg)
     }
     else if ( c == QUERY_MOREDATA )
     {
-	// we send error back anyway
-	
+	// we send error back anyway	
     }
 
     if ( _protType == CegoDbHandler::XML )
     {
-
-	
 	_xml.getDocument()->clear();
 	_xml.getDocument()->setDocType(XML_ERROR_DOC);
 	
@@ -1065,7 +1027,6 @@ void CegoDbHandler::sendErrorData(const Chain& msg)
     }
     else
     {
-
 	_pSer->reset();
 	_pSer->writeChain(SER_ERROR);
 	_pSer->writeChain(msg);
@@ -1081,26 +1042,23 @@ void CegoDbHandler::sendErrorData(const Chain& msg)
     _pN->writeMsg();
 }
 
-
 void CegoDbHandler::sendSessionConfirm(const Chain& msg, 
-				       long tid, 
+				       unsigned long long tid, 
 				       const Chain& dbName, 
-				       const Chain& dbVersion)
-{
-
-    
+				       const Chain& dbVersion,
+				       const Chain& dateTimeFormat)
+{    
     if ( _protType == CegoDbHandler::XML )
     {
-
-    
 	_xml.getDocument()->clear();
-	_xml.getDocument()->setDocType(XML_OK_DOC);
+	_xml.getDocument()->setDocType(XML_SACK_DOC);
 	
 	Element* pRoot = new Element(XML_FRAME_ELEMENT);
 	pRoot->setAttribute(XML_MSG_ATTR, msg);
 	pRoot->setAttribute(XML_TID_ATTR, Chain(tid));
 	pRoot->setAttribute(XML_DBPRODNAME_ATTR, dbName);
 	pRoot->setAttribute(XML_DBPRODVERSION_ATTR, dbVersion);
+	pRoot->setAttribute(XML_DATETIMEFORMAT_ATTR, dateTimeFormat);
 	
 	_xml.getDocument()->setRootElement(pRoot);
 	
@@ -1124,7 +1082,8 @@ void CegoDbHandler::sendSessionConfirm(const Chain& msg,
 	_pSer->writeChain(Chain(tid));
 	_pSer->writeChain(dbName);
 	_pSer->writeChain(dbVersion);
-
+	_pSer->writeChain(dateTimeFormat);
+	
 #ifdef CGDEBUG
 	_pModule->log(_modId, Logger::DEBUG, Chain("--- SER ---"));
 	_pModule->log(_modId, Logger::DEBUG, Chain(_pN->getMsg(), _pN->getMsgSize()));
@@ -1134,17 +1093,12 @@ void CegoDbHandler::sendSessionConfirm(const Chain& msg,
     }
     
     _pN->writeMsg();
-    
 }
 
-
-
-void CegoDbHandler::sendResponse(const Chain& msg, long affected)
+void CegoDbHandler::sendResponse(const Chain& msg, unsigned long long affected)
 {
-
     if ( _protType == CegoDbHandler::XML )
     {
-
 	_xml.getDocument()->clear();
 	_xml.getDocument()->setDocType(XML_OK_DOC);
 	
@@ -1163,8 +1117,7 @@ void CegoDbHandler::sendResponse(const Chain& msg, long affected)
 	_pModule->log(_modId, Logger::DEBUG, Chain("--- --- ---"));
 #endif
 	
-	_pN->setMsg(xmlString, xmlString.length());
-	
+	_pN->setMsg(xmlString, xmlString.length());	
     }
     else
     {
@@ -1181,15 +1134,12 @@ void CegoDbHandler::sendResponse(const Chain& msg, long affected)
 	
     }
     _pN->writeMsg();
-    
 }
 
 void CegoDbHandler::sendError(const Chain& msg)
 {
-
     if ( _protType == CegoDbHandler::XML )
     {
-
 	_xml.getDocument()->clear();
 	_xml.getDocument()->setDocType(XML_ERROR_DOC);
 	
@@ -1223,17 +1173,13 @@ void CegoDbHandler::sendError(const Chain& msg)
 #endif
 	
     }
-    
     _pN->writeMsg();
-    
 }
 
 void CegoDbHandler::sendBlobInfo(int fileId, int pageId)
 {
-
     if ( _protType == CegoDbHandler::XML )
     {
-
 	_xml.getDocument()->clear();
 	_xml.getDocument()->setDocType(XML_OK_DOC);
 	
@@ -1253,23 +1199,19 @@ void CegoDbHandler::sendBlobInfo(int fileId, int pageId)
 #endif
 	
 	_pN->setMsg(xmlString, xmlString.length());
-
     }
     else
     {
 	_pSer->reset();
 	_pSer->writeChain(Chain(SER_BLOBINFO));
 	_pSer->writeChain(Chain(fileId));
-	_pSer->writeChain(Chain(pageId));	
-
+	_pSer->writeChain(Chain(pageId));
     }
     _pN->writeMsg();
-    
 }
 
-void CegoDbHandler::sendBlobSize(long blobSize)
+void CegoDbHandler::sendBlobSize(unsigned long long blobSize)
 {
-
     if ( _protType == CegoDbHandler::XML )
     {
 
@@ -1299,22 +1241,17 @@ void CegoDbHandler::sendBlobSize(long blobSize)
 	_pSer->writeChain(Chain(SER_BLOBSIZE));
 	_pSer->writeChain(Chain(blobSize));	
     }
-
-    _pN->writeMsg();
-    
+    _pN->writeMsg();   
 }
 
-
-CegoDbHandler::ResultType CegoDbHandler::putBlob(const Chain& tableSet, CegoBlob& blob)
+CegoDbHandler::ResultType CegoDbHandler::putBlob(CegoBlob& blob)
 {
-
     if ( _protType == CegoDbHandler::XML )
     {
-
 	_xml.getDocument()->clear();
 	
 	Element* pRoot = new Element(XML_FRAME_ELEMENT);
-	pRoot->setAttribute(XML_TABLESET_ATTR, tableSet);
+	pRoot->setAttribute(XML_TABLESET_ATTR, _tableSet);
 	pRoot->setAttribute(XML_SIZE_ATTR, blob.getSize());
 	
 	_xml.getDocument()->setRootElement(pRoot);
@@ -1323,16 +1260,14 @@ CegoDbHandler::ResultType CegoDbHandler::putBlob(const Chain& tableSet, CegoBlob
 	Chain request;
 	_xml.getXMLChain(request);
 	
-	_pN->setMsg(request, request.length());
-	
+	_pN->setMsg(request, request.length());	
     }
     else
     {
 	_pSer->reset();
 	_pSer->writeChain(Chain(SER_PUTBLOB));
-	_pSer->writeChain(Chain(tableSet));
+	_pSer->writeChain(Chain(_tableSet));
 	_pSer->writeChain(Chain(blob.getSize()));
-	
     }
 
     _pN->writeMsg();
@@ -1341,7 +1276,6 @@ CegoDbHandler::ResultType CegoDbHandler::putBlob(const Chain& tableSet, CegoBlob
 	
     if ( _protType == CegoDbHandler::XML )
     {
-
 	_xml.getDocument()->clear();
 	_xml.setChain( _pN->getMsg() );
 	_xml.parse();
@@ -1363,7 +1297,6 @@ CegoDbHandler::ResultType CegoDbHandler::putBlob(const Chain& tableSet, CegoBlob
     }
     else
     {
-
 	_pSer->reset();
 	Chain req = _pSer->readChain();
 
@@ -1379,8 +1312,7 @@ CegoDbHandler::ResultType CegoDbHandler::putBlob(const Chain& tableSet, CegoBlob
     blob.reset();
     
     while ( blob.nextChunk(BLOBCHUNKSIZE) )
-    {
-	
+    {	
 	_pN->setMsg((char*)blob.getChunkPtr(), blob.getChunkSize());
 	
 	_pN->writeMsg();
@@ -1391,20 +1323,17 @@ CegoDbHandler::ResultType CegoDbHandler::putBlob(const Chain& tableSet, CegoBlob
 	    return DB_ERROR;
 	}
     }
-    
     return DB_OK;
 }
 
-CegoDbHandler::ResultType CegoDbHandler::getBlob(const Chain& tableSet, CegoBlob& blob)
+CegoDbHandler::ResultType CegoDbHandler::getBlob(CegoBlob& blob)
 {
-
     if ( _protType == CegoDbHandler::XML )
     {
-
 	_xml.getDocument()->clear();
 	
 	Element* pRoot = new Element(XML_FRAME_ELEMENT);
-	pRoot->setAttribute(XML_TABLESET_ATTR, tableSet);
+	pRoot->setAttribute(XML_TABLESET_ATTR, _tableSet);
 	pRoot->setAttribute(XML_FILEID_ATTR, blob.getFileId());
 	pRoot->setAttribute(XML_PAGEID_ATTR, blob.getPageId());
 	
@@ -1418,23 +1347,20 @@ CegoDbHandler::ResultType CegoDbHandler::getBlob(const Chain& tableSet, CegoBlob
     }
     else
     {
-
 	_pSer->reset();
 	_pSer->writeChain(Chain(SER_GETBLOB));
-	_pSer->writeChain(Chain(tableSet));
+	_pSer->writeChain(Chain(_tableSet));
 	_pSer->writeChain(Chain(blob.getFileId()));
-	_pSer->writeChain(Chain(blob.getPageId()));
-	
+	_pSer->writeChain(Chain(blob.getPageId()));	
     }
     _pN->writeMsg();
     
     _pN->readMsg();
 
-    long blobSize;
+    unsigned long long blobSize;
 
     if ( _protType == CegoDbHandler::XML )
     {
-	
 	_xml.getDocument()->clear();
 	_xml.setChain( _pN->getMsg() );
 	_xml.parse();
@@ -1450,12 +1376,11 @@ CegoDbHandler::ResultType CegoDbHandler::getBlob(const Chain& tableSet, CegoBlob
 	pRoot = _xml.getDocument()->getRootElement();
 	if ( pRoot )
 	{
-	    blobSize = pRoot->getAttributeValue(XML_SIZE_ATTR).asLong();
+	    blobSize = pRoot->getAttributeValue(XML_SIZE_ATTR).asUnsignedLongLong();
 	}
     }
     else
     {
-
 	_pSer->reset();
 	Chain req = _pSer->readChain();
 
@@ -1486,16 +1411,14 @@ CegoDbHandler::ResultType CegoDbHandler::getBlob(const Chain& tableSet, CegoBlob
     return DB_OK;
 }
 
-CegoDbHandler::ResultType CegoDbHandler::delBlob(const Chain& tableSet, CegoBlob& blob)
+CegoDbHandler::ResultType CegoDbHandler::delBlob(CegoBlob& blob)
 {
-
     if ( _protType == CegoDbHandler::XML )
     {
-
 	_xml.getDocument()->clear();
 	
 	Element* pRoot = new Element(XML_FRAME_ELEMENT);
-	pRoot->setAttribute(XML_TABLESET_ATTR, tableSet);
+	pRoot->setAttribute(XML_TABLESET_ATTR, _tableSet);
 	pRoot->setAttribute(XML_FILEID_ATTR, blob.getFileId());
 	pRoot->setAttribute(XML_PAGEID_ATTR, blob.getPageId());
 	
@@ -1509,10 +1432,9 @@ CegoDbHandler::ResultType CegoDbHandler::delBlob(const Chain& tableSet, CegoBlob
     }
     else
     {
-
 	_pSer->reset();
 	_pSer->writeChain(Chain(SER_DELBLOB));
-	_pSer->writeChain(Chain(tableSet));
+	_pSer->writeChain(Chain(_tableSet));
 	_pSer->writeChain(Chain(blob.getFileId()));
 	_pSer->writeChain(Chain(blob.getPageId()));
 	
@@ -1522,8 +1444,7 @@ CegoDbHandler::ResultType CegoDbHandler::delBlob(const Chain& tableSet, CegoBlob
     _pN->readMsg();
 
     if ( _protType == CegoDbHandler::XML )
-    {
-	
+    {	
 	_xml.getDocument()->clear();
 	_xml.setChain( _pN->getMsg() );
 	_xml.parse();
@@ -1553,9 +1474,304 @@ CegoDbHandler::ResultType CegoDbHandler::delBlob(const Chain& tableSet, CegoBlob
 	{
 	    return DB_ERROR;
 	}
-
 	return DB_OK;
+    }
+}
 
+void CegoDbHandler::sendClobInfo(int fileId, int pageId)
+{
+    if ( _protType == CegoDbHandler::XML )
+    {
+	_xml.getDocument()->clear();
+	_xml.getDocument()->setDocType(XML_OK_DOC);
+	
+	Element* pRoot = new Element(XML_FRAME_ELEMENT);
+	pRoot->setAttribute(XML_FILEID_ATTR, Chain(fileId));
+	pRoot->setAttribute(XML_PAGEID_ATTR, Chain(pageId));
+	
+	_xml.getDocument()->setRootElement(pRoot);
+	
+	Chain xmlString;
+	_xml.getXMLChain(xmlString);
+	
+#ifdef CGDEBUG
+	_pModule->log(_modId, Logger::DEBUG, Chain("--- XML ---"));
+	_pModule->log(_modId, Logger::DEBUG, xmlString);
+	_pModule->log(_modId, Logger::DEBUG, Chain("--- --- ---"));
+#endif
+	
+	_pN->setMsg(xmlString, xmlString.length());
+    }
+    else
+    {
+	_pSer->reset();
+	_pSer->writeChain(Chain(SER_CLOBINFO));
+	_pSer->writeChain(Chain(fileId));
+	_pSer->writeChain(Chain(pageId));	
+    }
+    _pN->writeMsg();
+}
+
+void CegoDbHandler::sendClobSize(unsigned long long clobSize)
+{
+    if ( _protType == CegoDbHandler::XML )
+    {
+	_xml.getDocument()->clear();
+	_xml.getDocument()->setDocType(XML_OK_DOC);
+	
+	Element* pRoot = new Element(XML_FRAME_ELEMENT);
+	pRoot->setAttribute(XML_SIZE_ATTR, Chain(clobSize));
+	
+	_xml.getDocument()->setRootElement(pRoot);
+	
+	Chain xmlString;
+	_xml.getXMLChain(xmlString);
+	
+#ifdef CGDEBUG
+	_pModule->log(_modId, Logger::DEBUG, Chain("--- XML ---"));
+	_pModule->log(_modId, Logger::DEBUG, xmlString);
+	_pModule->log(_modId, Logger::DEBUG, Chain("--- --- ---"));
+#endif
+	
+	_pN->setMsg(xmlString, xmlString.length());	
+    }
+    else
+    {
+	_pSer->reset();
+	_pSer->writeChain(Chain(SER_CLOBSIZE));
+	_pSer->writeChain(Chain(clobSize));	
+    }
+    _pN->writeMsg();
+}
+
+CegoDbHandler::ResultType CegoDbHandler::putClob(CegoClob& clob)
+{
+    if ( _protType == CegoDbHandler::XML )
+    {
+	_xml.getDocument()->clear();
+	
+	Element* pRoot = new Element(XML_FRAME_ELEMENT);
+	pRoot->setAttribute(XML_TABLESET_ATTR, _tableSet);
+	pRoot->setAttribute(XML_SIZE_ATTR, clob.getSize());
+	
+	_xml.getDocument()->setRootElement(pRoot);
+	_xml.getDocument()->setDocType(XML_PUTCLOB_REQUEST);
+	
+	Chain request;
+	_xml.getXMLChain(request);
+	
+	_pN->setMsg(request, request.length());	
+    }
+    else
+    {
+	_pSer->reset();
+	_pSer->writeChain(Chain(SER_PUTCLOB));
+	_pSer->writeChain(Chain(_tableSet));
+	_pSer->writeChain(Chain(clob.getSize()));
+	
+    }
+
+    _pN->writeMsg();
+	
+    _pN->readMsg();
+	
+    if ( _protType == CegoDbHandler::XML )
+    {
+	_xml.getDocument()->clear();
+	_xml.setChain( _pN->getMsg() );
+	_xml.parse();
+	
+	Chain docType = _xml.getDocument()->getDocType();
+	
+	if ( docType == Chain(XML_ERROR_DOC) )
+	{
+	    return DB_ERROR;
+	}    
+	
+	Element* pRoot;
+	pRoot = _xml.getDocument()->getRootElement();
+	if ( pRoot )
+	{
+	    clob.setFileId(pRoot->getAttributeValue(XML_FILEID_ATTR).asInteger());
+	    clob.setPageId(pRoot->getAttributeValue(XML_PAGEID_ATTR).asInteger());
+	}
+    }
+    else
+    {
+	_pSer->reset();
+	Chain req = _pSer->readChain();
+
+	if ( req == Chain(SER_ERROR) ) 
+	{
+	    return DB_ERROR;
+	}
+
+	clob.setFileId(_pSer->readChain().asInteger());
+	clob.setPageId(_pSer->readChain().asInteger());
+    }
+	
+    clob.reset();
+    
+    while ( clob.nextChunk(CLOBCHUNKSIZE) )
+    {	
+	_pN->setMsg((char*)clob.getChunkPtr(), clob.getChunkSize());
+	
+	_pN->writeMsg();
+	
+	if ( _pN->recvAck() == false )
+	{
+	    _pModule->log(_modId, Logger::LOGERR, Chain("User query abort"));
+	    return DB_ERROR;
+	}
+    }
+    return DB_OK;
+}
+
+CegoDbHandler::ResultType CegoDbHandler::getClob(CegoClob& clob)
+{
+
+    if ( _protType == CegoDbHandler::XML )
+    {
+	_xml.getDocument()->clear();
+	
+	Element* pRoot = new Element(XML_FRAME_ELEMENT);
+	pRoot->setAttribute(XML_TABLESET_ATTR, _tableSet);
+	pRoot->setAttribute(XML_FILEID_ATTR, clob.getFileId());
+	pRoot->setAttribute(XML_PAGEID_ATTR, clob.getPageId());
+	
+	_xml.getDocument()->setRootElement(pRoot);
+	_xml.getDocument()->setDocType(XML_GETCLOB_REQUEST);
+	
+	Chain request;
+	_xml.getXMLChain(request);
+	
+	_pN->setMsg(request, request.length());
+    }
+    else
+    {
+	_pSer->reset();
+	_pSer->writeChain(Chain(SER_GETCLOB));
+	_pSer->writeChain(Chain(_tableSet));
+	_pSer->writeChain(Chain(clob.getFileId()));
+	_pSer->writeChain(Chain(clob.getPageId()));	
+    }
+    _pN->writeMsg();
+    
+    _pN->readMsg();
+
+    unsigned long long clobSize;
+
+    if ( _protType == CegoDbHandler::XML )
+    {
+	_xml.getDocument()->clear();
+	_xml.setChain( _pN->getMsg() );
+	_xml.parse();
+	
+	Chain docType = _xml.getDocument()->getDocType();
+	
+	if ( docType == Chain(XML_ERROR_DOC) )
+	{
+	    return DB_ERROR;
+	}    
+	
+	Element* pRoot;	
+	pRoot = _xml.getDocument()->getRootElement();
+	if ( pRoot )
+	{
+	    clobSize = pRoot->getAttributeValue(XML_SIZE_ATTR).asUnsignedLongLong();
+	}
+    }
+    else
+    {
+	_pSer->reset();
+	Chain req = _pSer->readChain();
+
+	if ( req == Chain(SER_ERROR) ) 
+	{
+	    return DB_ERROR;
+	}
+	clobSize = _pSer->readChain().asInteger();
+    }
+
+    clob.allocate(clobSize);
+    clob.reset();
+    
+    int recvSize = 0;
+    while ( recvSize < clobSize )
+    {
+       	_pN->sendAck();
+	_pN->readMsg();
+	Chain msg(_pN->getMsg(), _pN->getMsgSize() );
+	
+	clob.putChunk((char*)_pN->getMsg(), _pN->getMsgSize() );
+	
+	recvSize += _pN->getMsgSize();
+    }
+    return DB_OK;
+}
+
+CegoDbHandler::ResultType CegoDbHandler::delClob(CegoClob& clob)
+{
+    if ( _protType == CegoDbHandler::XML )
+    {
+	_xml.getDocument()->clear();
+	
+	Element* pRoot = new Element(XML_FRAME_ELEMENT);
+	pRoot->setAttribute(XML_TABLESET_ATTR, _tableSet);
+	pRoot->setAttribute(XML_FILEID_ATTR, clob.getFileId());
+	pRoot->setAttribute(XML_PAGEID_ATTR, clob.getPageId());
+	
+	_xml.getDocument()->setRootElement(pRoot);
+	_xml.getDocument()->setDocType(XML_DELCLOB_REQUEST);
+	
+	Chain request;
+	_xml.getXMLChain(request);
+	
+	_pN->setMsg(request, request.length());
+    }
+    else
+    {
+	_pSer->reset();
+	_pSer->writeChain(Chain(SER_DELCLOB));
+	_pSer->writeChain(Chain(_tableSet));
+	_pSer->writeChain(Chain(clob.getFileId()));
+	_pSer->writeChain(Chain(clob.getPageId()));	
+    }
+    _pN->writeMsg();
+    
+    _pN->readMsg();
+
+    if ( _protType == CegoDbHandler::XML )
+    {
+	_xml.getDocument()->clear();
+	_xml.setChain( _pN->getMsg() );
+	_xml.parse();
+	
+	Chain docType = _xml.getDocument()->getDocType();
+	
+	if ( docType == Chain(XML_OK_DOC) )
+	{
+	    return DB_OK;
+	}	
+	else if ( docType == Chain(XML_ERROR_DOC) )
+	{
+	    return DB_ERROR;
+	}
+	else
+	{
+	    throw Exception(EXLOC, Chain("Invalid document type"));
+	}
+    }
+    else
+    {
+	_pSer->reset();
+	Chain req = _pSer->readChain();
+
+	if ( req == Chain(SER_ERROR) ) 
+	{
+	    return DB_ERROR;
+	}
+	return DB_OK;
     }
 }
 
@@ -1567,7 +1783,6 @@ void CegoDbHandler::sendProcResult(const Chain& msg, const ListT<CegoProcVar>& o
 
     if ( _protType == CegoDbHandler::XML )
     {
-
 	_xml.getDocument()->clear();
 	_xml.getDocument()->setDocType(XML_OK_DOC);
 	
@@ -1615,8 +1830,7 @@ void CegoDbHandler::sendProcResult(const Chain& msg, const ListT<CegoProcVar>& o
 	_pN->setMsg(xmlString, xmlString.length());
     }
     else
-    {
-	
+    {	
 	_pSer->reset();
 	_pSer->writeChain(Chain(SER_PROCRES));
 
@@ -1644,17 +1858,13 @@ void CegoDbHandler::sendProcResult(const Chain& msg, const ListT<CegoProcVar>& o
 	_pModule->log(_modId, Logger::DEBUG, Chain(_pN->getMsg(), _pN->getMsgSize()));
 	_pModule->log(_modId, Logger::DEBUG, Chain("--- --- ---"));
 #endif
-
-
     }
     
     _pN->writeMsg();
-    
 }
 
 void CegoDbHandler::sendProdInfo()
 {
-
     if ( _protType == CegoDbHandler::XML )
     {
 	_xml.getDocument()->clear();
@@ -1665,8 +1875,7 @@ void CegoDbHandler::sendProdInfo()
 	pRoot->setAttribute(XML_DBPRODVERSION_ATTR, XML_DBPRODVERSION_VALUE);
 	
 	_xml.getDocument()->setRootElement(pRoot);
-	
-	
+		
 	Chain xmlString;
 	_xml.getXMLChain(xmlString);
 	
@@ -1677,7 +1886,6 @@ void CegoDbHandler::sendProdInfo()
 #endif
 	
 	_pN->setMsg(xmlString, xmlString.length());
-
     }
     else
     {
@@ -1685,18 +1893,12 @@ void CegoDbHandler::sendProdInfo()
 	_pSer->writeChain(Chain(SER_PRODINFO));
 	_pSer->writeChain(CEGO_PRODUCT);
 	_pSer->writeChain(CEGO_VERSION);
-
     }
-
     _pN->writeMsg();
-    
 }
 
 CegoDbHandler::ResultType CegoDbHandler::getMoreTableData()
 {
-
-    // cout << "GET MORE DATA .." << endl;
-
 #ifdef CGDEBUG
     _pModule->log(_modId, Logger::DEBUG, Chain("Sending ack ..."));
 #endif    
@@ -1909,6 +2111,18 @@ CegoDbHandler::RequestType CegoDbHandler::acceptRequest()
 	{
 	    return CegoDbHandler::DELBLOB;
 	}
+	else if (  docType == Chain(XML_PUTCLOB_REQUEST) ) 
+	{	    
+	    return CegoDbHandler::PUTCLOB;
+	}
+	else if (  docType == Chain(XML_GETCLOB_REQUEST) ) 
+	{
+	    return CegoDbHandler::GETCLOB;
+	}
+	else if (  docType == Chain(XML_DELCLOB_REQUEST) ) 
+	{
+	    return CegoDbHandler::DELCLOB;
+	}
 	else if (  docType == Chain(XML_STARTTRANSACTION_REQUEST) ) 
 	{
 	    return CegoDbHandler::STARTTRANSACTION;
@@ -1953,7 +2167,7 @@ CegoDbHandler::RequestType CegoDbHandler::acceptRequest()
 	}
 	else if ( req == Chain(SER_QUERYABORT) ) 
 	{
-	    _serTid = _pSer->readChain().asLong();
+	    _serTid = _pSer->readChain().asUnsignedLongLong();
 	    return CegoDbHandler::QUERYABORT;
 	}
 	else if ( req == Chain(SER_DBPRODINFO) ) 
@@ -2037,7 +2251,7 @@ CegoDbHandler::RequestType CegoDbHandler::acceptRequest()
 	else if (  req == Chain(SER_PUTBLOB) ) 
 	{
 	    _serTableSet = _pSer->readChain();
-	    _serBlobSize = _pSer->readChain().asLong();
+	    _serBlobSize = _pSer->readChain().asUnsignedLongLong();
 	    return CegoDbHandler::PUTBLOB;
 	}
 	else if (  req == Chain(SER_GETBLOB) ) 
@@ -2051,6 +2265,23 @@ CegoDbHandler::RequestType CegoDbHandler::acceptRequest()
 	{
 	    return CegoDbHandler::DELBLOB;
 	}
+	else if (  req == Chain(SER_PUTCLOB) ) 
+	{
+	    _serTableSet = _pSer->readChain();
+	    _serClobSize = _pSer->readChain().asUnsignedLongLong();
+	    return CegoDbHandler::PUTCLOB;
+	}
+	else if (  req == Chain(SER_GETCLOB) ) 
+	{
+	    _serTableSet = _pSer->readChain();
+	    _serFileId = _pSer->readChain().asInteger();
+	    _serPageId = _pSer->readChain().asInteger();
+	    return CegoDbHandler::GETCLOB;
+	}
+	else if (  req == Chain(SER_DELCLOB) ) 
+	{
+	    return CegoDbHandler::DELCLOB;
+	}
 	else if (  req == Chain(SER_STARTTRANSACTION) ) 
 	{
 	    return CegoDbHandler::STARTTRANSACTION;
@@ -2076,12 +2307,10 @@ CegoDbHandler::RequestType CegoDbHandler::acceptRequest()
 	    return CegoDbHandler::UNKNOWN;
 	}	
     }
-    
 }
 
 Chain CegoDbHandler::getQueryArg()
 {
-
     if ( _protType == CegoDbHandler::XML )
     {	    
 	Element *pRoot = _xml.getDocument()->getRootElement();
@@ -2091,23 +2320,19 @@ Chain CegoDbHandler::getQueryArg()
 	    _serQueryCmd = pRoot->getAttributeValue(XML_CMD_ATTR);
 	}
     }
-    
-    return _serQueryCmd;
-    
+    return _serQueryCmd;    
 }
 
 
-void CegoDbHandler::getPutBlobArg(Chain& tableSet, long& blobSize)
+void CegoDbHandler::getPutBlobArg(Chain& tableSet, unsigned long long& blobSize)
 {
-
     if ( _protType == CegoDbHandler::XML )
     {	    
-
 	Element *pRoot = _xml.getDocument()->getRootElement();
 	if ( pRoot )
 	{
 	    tableSet = pRoot->getAttributeValue(XML_TABLESET_ATTR);
-	    blobSize = pRoot->getAttributeValue(XML_SIZE_ATTR).asLong();
+	    blobSize = pRoot->getAttributeValue(XML_SIZE_ATTR).asUnsignedLongLong();
 	}
     }
     else
@@ -2119,11 +2344,46 @@ void CegoDbHandler::getPutBlobArg(Chain& tableSet, long& blobSize)
 
 void CegoDbHandler::getGetBlobArg(Chain& tableSet, int& fileId, int& pageId)
 {
-
-
     if ( _protType == CegoDbHandler::XML )
     {	    
+	Element *pRoot = _xml.getDocument()->getRootElement();
+	if ( pRoot )
+	{
+	    tableSet = pRoot->getAttributeValue(XML_TABLESET_ATTR);
+	    fileId = pRoot->getAttributeValue(XML_FILEID_ATTR).asInteger();
+	    pageId = pRoot->getAttributeValue(XML_PAGEID_ATTR).asInteger();
+	}
+    }
+    else
+    {
+	tableSet = _serTableSet;
+	fileId = _serFileId;
+	pageId = _serPageId;
+    }
+}
 
+void CegoDbHandler::getPutClobArg(Chain& tableSet, unsigned long long& clobSize)
+{
+    if ( _protType == CegoDbHandler::XML )
+    {	    
+	Element *pRoot = _xml.getDocument()->getRootElement();
+	if ( pRoot )
+	{
+	    tableSet = pRoot->getAttributeValue(XML_TABLESET_ATTR);
+	    clobSize = pRoot->getAttributeValue(XML_SIZE_ATTR).asUnsignedLongLong();
+	}
+    }
+    else
+    {
+	tableSet = _serTableSet;
+	clobSize = _serClobSize;	
+    }
+}
+
+void CegoDbHandler::getGetClobArg(Chain& tableSet, int& fileId, int& pageId)
+{
+    if ( _protType == CegoDbHandler::XML )
+    {	    
 	Element *pRoot = _xml.getDocument()->getRootElement();
 	if ( pRoot )
 	{
@@ -2142,14 +2402,12 @@ void CegoDbHandler::getGetBlobArg(Chain& tableSet, int& fileId, int& pageId)
 
 void CegoDbHandler::collectSchema(const ListT<CegoField>& schema, const Chain& format)
 {
-
 #ifdef CGDEBUG
     _pModule->log(_modId, Logger::DEBUG, Chain("Collecting schema ..."));
 #endif
 
     if ( _protType == CegoDbHandler::XML )
     {	    
-
 	_xml.getDocument()->clear();
 	
 	Element* pRoot = new Element(XML_FRAME_ELEMENT);
@@ -2161,7 +2419,6 @@ void CegoDbHandler::collectSchema(const ListT<CegoField>& schema, const Chain& f
 	CegoField *pF = schema.First();
 	while ( pF ) 
 	{
-	    
 	    Chain tname;
 	    if (pF->getTableAlias().length() > 0)
 	    {
@@ -2185,7 +2442,6 @@ void CegoDbHandler::collectSchema(const ListT<CegoField>& schema, const Chain& f
 	Chain response;
 	_xml.getXMLChain(response);
 	
-	
 #ifdef CGDEBUG
 	_pModule->log(_modId, Logger::DEBUG, Chain("--- XML ---"));
 	_pModule->log(_modId, Logger::DEBUG, response);
@@ -2210,10 +2466,8 @@ void CegoDbHandler::collectSchema(const ListT<CegoField>& schema, const Chain& f
 
 void CegoDbHandler::sendObjInfo(CegoDecodableObject& oe)
 {
-
     if ( _protType == CegoDbHandler::XML )
-    {	    
-
+    {
 	_xml.getDocument()->clear();
 	
 	Element* pRoot = new Element(XML_FRAME_ELEMENT);
@@ -2238,7 +2492,6 @@ void CegoDbHandler::sendObjInfo(CegoDecodableObject& oe)
     }
     else
     {
-
 	_pSer->reset();
 	_pSer->writeChain(Chain(SER_INFO));
 	_pSer->writeObject(oe);
@@ -2251,17 +2504,13 @@ void CegoDbHandler::sendObjInfo(CegoDecodableObject& oe)
 	
 	_pN->writeMsg();
 	_pSer->reset();
-
     }
-    
 }
 
 void CegoDbHandler::collectData(const ListT<CegoField>& schema)
 {
-
     if ( _protType == CegoDbHandler::XML )
     {	    
-
 	Element* pRoot = _xml.getDocument()->getRootElement();
 	if ( pRoot == 0)
 	{
@@ -2299,10 +2548,8 @@ void CegoDbHandler::collectData(const ListT<CegoField>& schema)
 
 void CegoDbHandler::collectData(const ListT<CegoField>& schema, const ListT<CegoFieldValue>& fvl)
 {
-
     if ( _protType == CegoDbHandler::XML )
     {	    
-
 	Element* pRoot = _xml.getDocument()->getRootElement();
 	if ( pRoot == 0)
 	{
@@ -2318,7 +2565,6 @@ void CegoDbHandler::collectData(const ListT<CegoField>& schema, const ListT<Cego
 	
 	while ( pF && pFV )
 	{
-	    
 	    Chain colPos = Chain(NETMNG_COLPREFIX) + Chain(col);
 	    
 	    if ( ! pFV->isNull() )
@@ -2355,7 +2601,6 @@ CegoDbHandler::ResultType CegoDbHandler::sendXMLReq(const Chain& reqType, Elemen
     _xml.getXMLChain(request);
     _xml.getDocument()->clear();
 
-
 #ifdef CGDEBUG
     _pModule->log(_modId, Logger::DEBUG, Chain("--- XML ---"));
     _pModule->log(_modId, Logger::DEBUG, request);
@@ -2394,6 +2639,21 @@ CegoDbHandler::ResultType CegoDbHandler::sendXMLReq(const Chain& reqType, Elemen
     {
 	return DB_OK;
     }
+    else if ( docType == Chain(XML_SACK_DOC) )
+    {
+
+	Element *pRoot = _xml.getDocument()->getRootElement();
+	
+	if ( pRoot )
+	{
+	    _serMsg = pRoot->getAttributeValue(XML_MSG_ATTR);
+	    _serTid = pRoot->getAttributeValue(XML_TID_ATTR).asUnsignedLongLong();
+	    _serDbName = pRoot->getAttributeValue(XML_DBPRODNAME_ATTR);
+	    _serDbVersion = pRoot->getAttributeValue(XML_DBPRODVERSION_ATTR);
+	    _serDateFormat = pRoot->getAttributeValue(XML_DATETIMEFORMAT_ATTR);
+	}
+	return DB_OK;	    
+    }
     else if ( docType == Chain(XML_ERROR_DOC) )
     {
 
@@ -2421,10 +2681,10 @@ CegoDbHandler::ResultType CegoDbHandler::sendSerialReq()
 {
 
 #ifdef CGDEBUG
-    _pModule->log(_modId, Logger::DEBUG, Chain("--- Ser ---"));
+    _pModule->log(_modId, Logger::DEBUG, Chain("--- SER ---"));
     _pModule->log(_modId, Logger::DEBUG, Chain(_pN->getMsg(), _pN->getMsgSize()));
     _pModule->log(_modId, Logger::DEBUG, Chain("--- --- ---"));
-#endif    
+#endif
 
     _pN->writeMsg();
     
@@ -2448,10 +2708,10 @@ CegoDbHandler::ResultType CegoDbHandler::sendSerialReq()
     else if ( req == Chain(SER_SACK))
     {
 	_serMsg = _pSer->readChain();
-	_serTid = _pSer->readChain().asLong();
+	_serTid = _pSer->readChain().asUnsignedLongLong();
 	_serDbName = _pSer->readChain();
 	_serDbVersion = _pSer->readChain();
-	
+	_serDateFormat = _pSer->readChain();
 	return DB_OK;
     }
     else if ( req == Chain(SER_PROCRES))
@@ -2478,10 +2738,8 @@ CegoDbHandler::ResultType CegoDbHandler::sendSerialReq()
 		_outParamList.Insert( CegoProcVar(paramName, CegoProcVar::OUTVAR, fv.getType(), fv.getLength(), fv));
 	    }
 	}
-
 	return DB_OK;
     }
-
     else if ( req == Chain(SER_ERROR) )
     {
 	_serMsg = _pSer->readChain();
@@ -2504,4 +2762,3 @@ CegoDbHandler::ResultType CegoDbHandler::sendSerialReq()
 	return DB_ERROR;
     }    
 }
-
