@@ -6,22 +6,7 @@
 //                                                         
 // Design and Implementation by Bjoern Lemke
 //               
-// (C)opyright 2000-2010 Bjoern Lemke
-//
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
-// 
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; see the file COPYING.  If not, write to
-// the Free Software Foundation, 59 Temple Place - Suite 330,
-// Boston, MA 02111-1307, USA.
+// (C)opyright 2000-2016 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
@@ -34,7 +19,7 @@
 //              requests from a different cego database node.
 //  
 //
-// Status: QG-2.6
+// Status: CLEAN
 //
 ///////////////////////////////////////////////////////////////////////////////
 
@@ -51,6 +36,8 @@
 #include <string.h>
 #include <stdlib.h>
 
+extern Chain __dateTimeFormat;
+
 CegoDbThread::CegoDbThread() : Thread()
 {
 }
@@ -76,7 +63,7 @@ const Chain& CegoDbThread::lastAction() const
     return _lastAction;
 }
 
-long CegoDbThread::allocatedSortArea() const
+unsigned long long CegoDbThread::allocatedSortArea() const
 {
     if ( _pTabMng )
 	return _pTabMng->getAllocatedSortArea();
@@ -137,11 +124,9 @@ void CegoDbThread::checkReloadRequest()
 
 void* CegoDbThread::job(void* arg)
 {
-
     try
     {
-
-	_idx = *(long*)arg;
+	_idx = *(unsigned long long*)arg;
 	
 	_pTabMng = new CegoDistManager(_pDBMng);
 	_pPA = new CegoAction(_pTabMng, _pPool);
@@ -344,7 +329,6 @@ void CegoDbThread::loadObjects(int tabSetId)
 #endif
 	pViewName = viewList.Next();
     }
-
 }
 
 void CegoDbThread::unloadObjects(int tabSetId)
@@ -395,7 +379,6 @@ void CegoDbThread::serveSession(CegoDistDbHandler *pSH)
 	}
 	else
 	{
-
 	    if ( _pDBMng->tableSetExists(pSH->getTableSet()) == false )
 	    {
 		Chain msg = Chain("Unknown tableset ") + pSH->getTableSet();
@@ -406,7 +389,7 @@ void CegoDbThread::serveSession(CegoDistDbHandler *pSH)
 	    _pTabMng->setActiveUser(pSH->getTableSet(), pSH->getUser(), pSH->getPassword());
 
 	    msg = Chain("Access granted");
-	    pSH->sendSessionConfirm(msg, _idx, XML_DBPRODNAME_VALUE, XML_DBPRODVERSION_VALUE);
+	    pSH->sendSessionConfirm(msg, _idx, XML_DBPRODNAME_VALUE, XML_DBPRODVERSION_VALUE, __dateTimeFormat);
 	    
 #ifdef CGDEBUG
 	    _pDBMng->log(_modId, Logger::DEBUG, Chain("Thread ") + Chain(_idx) + Chain(" : Accepted session for tableset ") + pSH->getTableSet());
@@ -420,8 +403,7 @@ void CegoDbThread::serveSession(CegoDistDbHandler *pSH)
 	bool isTerminated = false;
 
 	while ( isTerminated == false && _pPool->isTerminated() == false )
-	{
-	    
+	{	    
 	    CegoDbHandler::RequestType reqType;
 	    reqType = pSH->acceptRequest();
 
@@ -433,7 +415,7 @@ void CegoDbThread::serveSession(CegoDistDbHandler *pSH)
 	    if ( reqType != CegoDbHandler::REQTIMEOUT )
 	    {
 		Chain runState = _pDBMng->getTableSetRunState(pSH->getTableSet());
-		if ( runState != Chain(XML_ONLINE_VALUE) && runState != Chain(XML_BACKUP_VALUE) )
+		if ( runState != Chain(XML_ONLINE_VALUE) && runState != Chain(XML_BACKUP_VALUE) && runState != Chain(XML_CHECKPOINT_VALUE) )
 		{
 		    Chain msg = Chain("Tableset " + pSH->getTableSet() + " not online ( run state is " + runState + ")");
 		    pSH->sendError(msg);
@@ -450,7 +432,6 @@ void CegoDbThread::serveSession(CegoDistDbHandler *pSH)
 		    		    
 		    _pPool->setState(_idx, CegoDbThreadPool::BUSY);
 
-
 		    _pTim->stop();
 		    _pPool->addThreadIdle(_idx, _pTim->getSum());
 		    
@@ -465,15 +446,11 @@ void CegoDbThread::serveSession(CegoDistDbHandler *pSH)
 	    else
 	    {
 
-
 #ifdef CGDEBUG
 		_pDBMng->log(_modId, Logger::DEBUG, Chain("Thread ") + Chain(_idx) + Chain(" : Client request timeout occured, waiting ..."));
 #endif
 	    }
-
 	    checkReloadRequest();
-
-
 	}
     }
 }
@@ -497,7 +474,7 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 	}
 	case CegoDbHandler::QUERYABORT:
 	{
-	    long idx;
+	    unsigned long long idx;
 
 	    idx = pSH->getTid();
 	    _pPool->abortThread(idx);
@@ -537,7 +514,7 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 	    
 	    CegoTableObject oe;
 	    _pTabMng->getObject(tabSetId, tableName, CegoObject::TABLE, oe);
-	    _pTabMng->insertLocalDataTable(oe, fl);
+	    _pTabMng->insertDataTableSynced(oe, fl);
 	    
 	    pSH->sendResponse(Chain("insert ok"));
 	    
@@ -557,7 +534,7 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 	    CegoTableObject oe;
 	    _pTabMng->getObject(tabSetId, tableName, CegoObject::TABLE, oe);
 	    
-	    long delCount = _pTabMng->deleteLocalDataTable(oe, pPred);
+	    unsigned long long delCount = _pTabMng->deleteDataTableSynced(oe, pPred);
 	    	    
 	    pSH->sendResponse(Chain("delete ok"), delCount);
 	    
@@ -579,7 +556,7 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 	    CegoTableObject oe;
 	    _pTabMng->getObject(tabSetId, tableName, CegoObject::TABLE, oe);
 	    
-	    long updCount = _pTabMng->updateLocalDataTable(oe, pPred, updSchema, exprList);
+	    unsigned long long updCount = _pTabMng->updateDataTableSynced(oe, pPred, updSchema, exprList);
 	    
 	    pSH->sendResponse(Chain("update ok"), updCount);
 	    
@@ -600,7 +577,7 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 	    if ( ! idxList.isEmpty() )
 	    {
 		Chain idxName = tableName + Chain(TABMNG_PIDX_SUFFIX);	
-		_pDBMng->addObject(tabSetId, idxName, CegoObject::INDEX);	
+		_pDBMng->addObject(tabSetId, idxName, CegoObject::AVLTREE);	
 	    }
 
 	    pSH->sendResponse(Chain("create ok"));	    
@@ -675,7 +652,7 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 	    
 	    pSH->getDropTableArg(tableSet, objName, objType);	    
 	    int tabSetId = _pDBMng->getTabSetId(tableSet);
-	    _pTabMng->dropLocalObject(tabSetId, objName, objType);
+	    _pTabMng->dropObjectSynced(tabSetId, objName, objType);
 	    pSH->sendResponse(Chain("Drop ok"));	    
 	    break;	    
 	}
@@ -688,7 +665,7 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 	    CegoObject::ObjectType type;	    
 	    pSH->getCreateIndexArg(tableSet, indexName, tableName, idxList, type);	    
 	    int tabSetId = _pDBMng->getTabSetId(tableSet);
-	    _pTabMng->createLocalIndexTable(tabSetId, indexName, tableName, type, idxList);
+	    _pTabMng->createIndexTableSynced(tabSetId, indexName, tableName, type, idxList);
 
 	    _pDBMng->addObject(tabSetId, indexName, type);
 
@@ -712,9 +689,9 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 		{
 		case CegoObject::SYSTEM:
 		case CegoObject::TABLE:
-		case CegoObject::PINDEX:
-		case CegoObject::UINDEX:
-		case CegoObject::INDEX:
+		case CegoObject::PAVLTREE:
+		case CegoObject::UAVLTREE:
+		case CegoObject::AVLTREE:
 		{   
 		    CegoTableObject to;
 		    _pTabMng->getLocalObject(tabSetId, objName, objType, to);
@@ -843,8 +820,10 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 	    ListT<CegoBTreeObject> btreeList;
 	    ListT<CegoKeyObject> keyList;
 	    ListT<CegoCheckObject> checkList;
+	    int numInvalid;
 
-	    _pTabMng->getObjectListByTable(tabSetId, tableName, idxList, btreeList, keyList, checkList);
+	    _pTabMng->setIgnoreInvalid(false);
+	    _pTabMng->getObjectListByTable(tabSetId, tableName, idxList, btreeList, keyList, checkList, numInvalid);
 	    
 	    pSH->sendObjByTableList(idxList, keyList, checkList);
 	    
@@ -929,7 +908,7 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 	    _lastAction = Chain("put blob");
 
 	    Chain tableSet;
-	    long blobSize;
+	    unsigned long long blobSize;
 	    
 	    pSH->getPutBlobArg(tableSet, blobSize);
 
@@ -944,57 +923,88 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 
 	    try
 	    {
-
 		_pTabMng->getNewFilePage(bp, tabSetId, CegoObject::TABLE);
-		bp.initPage(CegoBufferPage::BLOB);
-		
-		fileId = bp.getFileId();
-		pageId = bp.getPageId();
+	    }
+	    catch ( Exception e )
+	    {
+		Chain msg;
+		e.pop(msg);
+		pSH->sendError(msg);
+		break;
+	    }
+
+	    bp.initPage(CegoBufferPage::BLOB);
+	    
+	    fileId = bp.getFileId();
+	    pageId = bp.getPageId();
 		
+	    try
+	    {
 
 #ifdef CGDEBUG
 		_pDBMng->log(_modId, Logger::DEBUG, Chain(" Thread ") + Chain(_idx) + Chain(" : Sending blob info [") + Chain(fileId) + Chain(",") + Chain(pageId) + Chain("]"));
 #endif
 		pSH->sendBlobInfo(fileId, pageId);
+
 		
 		int freeInPage = bp.getChunkLen();
 		char* pagePtr = bp.getChunkEntry();
 		
-		memcpy(pagePtr, &blobSize, sizeof(long));
-		pagePtr += sizeof(long);
-		freeInPage -= sizeof(long);
+		memcpy(pagePtr, &blobSize, sizeof(unsigned long long));
+		pagePtr += sizeof(unsigned long long);
+		freeInPage -= sizeof(unsigned long long);
 		
 		char *bufPtr = 0; 
 		int availFromBuf = 0;
 		
-		long recvSize = 0;
-		while ( recvSize < blobSize )
-		{		    
-		    if ( availFromBuf == 0 )		    
-		    {
-			if ( pN->waitMsg(NETMNG_WAITMSG_TIMEOUT) == false )
-			    throw Exception(EXLOC, "Timeout exceeded for blob chunk");
-			pN->readMsg();
-			pN->sendAck();
-			bufPtr = pN->getMsg();
-			availFromBuf = pN->getMsgSize();
-		    }    
-		    
-		    if ( freeInPage == 0 )
-		    {
-			CegoBufferPage nextPage; 
-			_pTabMng->getNewFilePage(nextPage, tabSetId, CegoObject::TABLE);
-			nextPage.initPage(CegoBufferPage::BLOB);
-			
-			bp.setNextFileId(nextPage.getFileId());
-			bp.setNextPageId(nextPage.getPageId());
+		unsigned long long recvSize = 0;
 
-			_pDBMng->bufferUnfix(bp, true, _pTabMng->getLockHandler());
+		while ( recvSize < blobSize )
+		{	
+		    
+		    try
+		    {
+
+			bool ackRequired = false;
+
+			if ( availFromBuf == 0 )		    
+			{
+			    if ( pN->waitMsg(NETMNG_WAITMSG_TIMEOUT) == false )
+				throw Exception(EXLOC, "Timeout exceeded for blob chunk");
+			    pN->readMsg();
+			    ackRequired = true;
+			    bufPtr = pN->getMsg();
+			    availFromBuf = pN->getMsgSize();
+			}    
 			
-			bp = nextPage;
-			
-			freeInPage = bp.getChunkLen();
-			pagePtr = bp.getChunkEntry();
+			if ( freeInPage == 0 )
+			{
+			    CegoBufferPage nextPage;
+			    _pTabMng->getNewFilePage(nextPage, tabSetId, CegoObject::TABLE);
+			    nextPage.initPage(CegoBufferPage::BLOB);
+			    
+			    bp.setNextFileId(nextPage.getFileId());
+			    bp.setNextPageId(nextPage.getPageId());
+			    
+			    _pDBMng->bufferUnfix(bp, true, _pTabMng->getLockHandler());
+			    
+			    bp = nextPage;
+			    
+			    freeInPage = bp.getChunkLen();
+			    pagePtr = bp.getChunkEntry();
+			}
+
+			if ( ackRequired )
+			{
+			    pN->sendAck();
+			    ackRequired = false;
+			}
+		    
+		    }
+		    catch ( Exception e )
+		    {			
+			pN->sendNack();			    
+			throw e;
 		    }
 		    
 		    if ( freeInPage >= availFromBuf )
@@ -1057,12 +1067,12 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 						     _pTabMng->getLockHandler());
 
 	    	
-		long blobSize;
-		memcpy (&blobSize, bp.getChunkEntry(), sizeof(long));
+		unsigned long long blobSize;
+		memcpy (&blobSize, bp.getChunkEntry(), sizeof(unsigned long long));
 		
 		pSH->sendBlobSize(blobSize);
 		
-		long sentByte = 0;
+		unsigned long long sentByte = 0;
 		
 		while ( bp.isFixed() )
 		{
@@ -1073,7 +1083,7 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 		    {		       
 			chunkSize = bp.getChunkLen();
 			if ( sentByte == 0 )
-			    chunkSize -= sizeof(long);
+			    chunkSize -= sizeof(unsigned long long);
 		    }
 		    else
 		    {
@@ -1082,7 +1092,7 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 		    
 		    if ( sentByte == 0 )
 		    {
-			pN->setMsg(bp.getChunkEntry() + sizeof(long), chunkSize );
+			pN->setMsg(bp.getChunkEntry() + sizeof(unsigned long long), chunkSize );
 		    }
 		    else
 		    {
@@ -1133,6 +1143,250 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 	    break;
 	    
 	}
+
+	case CegoDbHandler::PUTCLOB:
+	{
+
+	    _lastAction = Chain("put clob");
+
+
+	    Chain tableSet;
+	    unsigned long long clobSize;
+	    
+	    pSH->getPutClobArg(tableSet, clobSize);
+
+	    int tabSetId = _pDBMng->getTabSetId(tableSet);
+
+	    NetHandler *pN = pSH->getNetHandler();
+
+	    int fileId = 0;
+	    int pageId = 0;
+
+	    CegoBufferPage bp;
+
+	    try
+	    {
+		_pTabMng->getNewFilePage(bp, tabSetId, CegoObject::TABLE);
+	    }
+	    catch ( Exception e )
+	    {
+		Chain msg;
+		e.pop(msg);
+		pSH->sendError(msg);
+		break;
+	    }
+
+	    bp.initPage(CegoBufferPage::CLOB);
+	    
+	    fileId = bp.getFileId();
+	    pageId = bp.getPageId();
+		
+	    try
+	    {
+
+#ifdef CGDEBUG
+		_pDBMng->log(_modId, Logger::DEBUG, Chain(" Thread ") + Chain(_idx) + Chain(" : Sending clob info [") + Chain(fileId) + Chain(",") + Chain(pageId) + Chain("]"));
+#endif
+		pSH->sendClobInfo(fileId, pageId);
+
+		
+		int freeInPage = bp.getChunkLen();
+		char* pagePtr = bp.getChunkEntry();
+		
+		memcpy(pagePtr, &clobSize, sizeof(unsigned long long));
+		pagePtr += sizeof(unsigned long long);
+		freeInPage -= sizeof(unsigned long long);
+		
+		char *bufPtr = 0; 
+		int availFromBuf = 0;
+		
+		unsigned long long recvSize = 0;
+
+		while ( recvSize < clobSize )
+		{	
+		    
+		    try
+		    {
+
+			bool ackRequired = false;
+
+			if ( availFromBuf == 0 )		    
+			{
+			    if ( pN->waitMsg(NETMNG_WAITMSG_TIMEOUT) == false )
+				throw Exception(EXLOC, "Timeout exceeded for blob chunk");
+			    pN->readMsg();
+			    ackRequired = true;
+			    bufPtr = pN->getMsg();
+			    availFromBuf = pN->getMsgSize();
+			}    
+			
+			if ( freeInPage == 0 )
+			{
+			    CegoBufferPage nextPage;
+			    _pTabMng->getNewFilePage(nextPage, tabSetId, CegoObject::TABLE);
+			    nextPage.initPage(CegoBufferPage::CLOB);
+			    
+			    bp.setNextFileId(nextPage.getFileId());
+			    bp.setNextPageId(nextPage.getPageId());
+			    
+			    _pDBMng->bufferUnfix(bp, true, _pTabMng->getLockHandler());
+			    
+			    bp = nextPage;
+			    
+			    freeInPage = bp.getChunkLen();
+			    pagePtr = bp.getChunkEntry();
+			}
+
+			if ( ackRequired )
+			{
+			    pN->sendAck();
+			    ackRequired = false;
+			}
+		    
+		    }
+		    catch ( Exception e )
+		    {			
+			pN->sendNack();			    
+			throw e;
+		    }
+		    
+		    if ( freeInPage >= availFromBuf )
+		    {
+			memcpy(pagePtr, bufPtr, availFromBuf);
+			recvSize += availFromBuf;
+			pagePtr += availFromBuf;
+			freeInPage -= availFromBuf;
+			availFromBuf = 0;
+		    }
+		    else if ( freeInPage < availFromBuf )
+		    {
+			memcpy(pagePtr, bufPtr, freeInPage);
+			recvSize += freeInPage;
+			bufPtr += freeInPage;
+			availFromBuf -= freeInPage;
+			freeInPage = 0;
+		    }		    
+		}
+		_pDBMng->bufferUnfix(bp, true, _pTabMng->getLockHandler());
+	    }
+	    catch ( Exception e )
+	    {
+		if ( bp.isFixed() )
+		    _pDBMng->bufferUnfix(bp, true, _pTabMng->getLockHandler());
+		if ( fileId || pageId )
+		{
+		    _pTabMng->releaseClob(tabSetId, fileId, pageId);
+		}
+	    }
+
+#ifdef CGDEBUG
+	    _pDBMng->log(_modId, Logger::DEBUG, Chain(" Thread ") + Chain(_idx) + Chain(" : Clob transfer finished"));
+#endif
+
+	    break;
+
+	}
+	case CegoDbHandler::GETCLOB:
+	{
+
+	    _lastAction = Chain("get clob");
+
+	    Chain tableSet;
+	    int fileId;
+	    int pageId;
+	    
+	    pSH->getGetClobArg(tableSet, fileId, pageId);
+
+	    int tabSetId = _pDBMng->getTabSetId(tableSet);
+
+	    NetHandler *pN = pSH->getNetHandler();	    	
+
+	    bool isFixed = false;
+	    CegoBufferPage bp;
+	    try
+	    {
+
+		_pDBMng->bufferFix(bp, tabSetId, fileId, pageId, CegoBufferPool::SYNC,
+						     _pTabMng->getLockHandler());
+
+	    	
+		unsigned long long clobSize;
+		memcpy (&clobSize, bp.getChunkEntry(), sizeof(unsigned long long));
+		
+		pSH->sendClobSize(clobSize);
+		
+		unsigned long long sentByte = 0;
+		
+		while ( bp.isFixed() )
+		{
+		    pN->recvAck();
+		    
+		    int chunkSize;
+		    if (  bp.getChunkLen() < clobSize - sentByte )
+		    {		       
+			chunkSize = bp.getChunkLen();
+			if ( sentByte == 0 )
+			    chunkSize -= sizeof(unsigned long long);
+		    }
+		    else
+		    {
+			chunkSize = clobSize - sentByte;
+		    }
+		    
+		    if ( sentByte == 0 )
+		    {
+			pN->setMsg(bp.getChunkEntry() + sizeof(unsigned long long), chunkSize );
+		    }
+		    else
+		    {
+			pN->setMsg(bp.getChunkEntry(), chunkSize);
+		    }
+		    sentByte += chunkSize;
+
+		    pN->writeMsg();
+		    
+		    fileId = bp.getNextFileId();
+		    pageId = bp.getNextPageId();
+		    
+		    _pDBMng->bufferUnfix(bp, false, _pTabMng->getLockHandler());
+		    
+		    if ( fileId || pageId )			
+		    {
+			_pDBMng->bufferFix(bp, tabSetId, fileId, pageId, CegoBufferPool::SYNC,
+					   _pTabMng->getLockHandler());
+		    }		    
+		}
+	    }
+	    catch ( Exception e )
+	    {
+		if ( bp.isFixed() )
+		    _pDBMng->bufferUnfix(bp, true, _pTabMng->getLockHandler());
+	    }
+
+	    break;
+	    
+	}
+	case CegoDbHandler::DELCLOB:
+	{
+
+	    _lastAction = Chain("del clob");
+
+	    Chain tableSet;
+	    int fileId;
+	    int pageId;
+	    
+	    pSH->getGetClobArg(tableSet, fileId, pageId);
+
+	    int tabSetId = _pDBMng->getTabSetId(tableSet);
+
+	    _pTabMng->releaseClob(tabSetId, fileId, pageId);
+
+	    pSH->sendResponse(Chain("Clob deleted"));
+	    
+	    break;
+	    
+	}
+
 	case CegoDbHandler::STARTTRANSACTION:
 	{
 	    _lastAction = Chain("start transaction");
@@ -1212,4 +1466,3 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 
     return isTerminated;
 }
-
