@@ -8,30 +8,15 @@
 //                                                         
 // Design and Implementation by Bjoern Lemke               
 //     
-// (C)opyright 2000-2013 Bjoern Lemke
-//
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
-// 
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; see the file COPYING.  If not, write to
-// the Free Software Foundation, 59 Temple Place - Suite 330,
-// Boston, MA 02111-1307, USA.
+// (C)opyright 2000-2016 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
-// Class: 
+// Class: NA
 // 
-// Description: 
+// Description: All constant definitions 
 //
-// Status: IMPLEMENTED
+// Status: CLEAN
 //
 ///////////////////////////////////////////////////////////////////////////////
 
@@ -43,7 +28,7 @@
 
 #define CEGO_PRODUCT PACKAGE_NAME
 #define CEGO_VERSION PACKAGE_VERSION
-#define CEGO_COPYRIGHT "Copyright (C) 2000-2013 by Bjoern Lemke. All rights reserved"
+#define CEGO_COPYRIGHT "Copyright (C) 2000-2016 by Bjoern Lemke. All rights reserved"
 
 #define CGEXESHELLVARNAME "CGEXESHELL"
 #define CGSTDEXESHELL "/bin/sh"
@@ -53,14 +38,15 @@
 /*******************************/
 
 /* maximum number of datafiles, that can be managed */
-#define FILMNG_MAXDATAFILE 100
+/* Please note, that id values from 1 to TABMNG_MAXTABSET are reserved for system files */
+#define FILMNG_MAXDATAFILE 5000
 
 /*******************************/
 /* Table Management Parameters */
 /*******************************/
 
 /* maximum number of tablesets, that can be managed */
-#define TABMNG_MAXTABSET 30
+#define TABMNG_MAXTABSET 200
 
 /* maximum size of index entry values */
 #define TABMNG_MAXINDEXVALUE 1000
@@ -71,10 +57,12 @@
 /* suffix defs */
 #define TABMNG_PIDX_SUFFIX "_pidx"
 #define TABMNG_PBTREE_SUFFIX "_pbtree"
-#define SYSSUFFIX ".sys"
-#define TEMPSUFFIX ".temp"
-#define OSPACE "ospace"
-#define GSPACE "gspace"
+#define TABMNG_SYSSUFFIX ".sys"
+#define TABMNG_TEMPSUFFIX ".temp"
+#define TABMNG_APPSUFFIX ".dat"
+#define TABMNG_LOGSUFFIX ".log"
+#define TABMNG_XMLSUFFIX ".xml"
+
 /* how many tables can be joned with one select */
 #define TABMNG_MAXJOINLEVEL 30
 #define TABMNG_MAXTMPSCHEMA 30
@@ -89,11 +77,12 @@
 #define LOGMNG_RECBUFSIZE 65536
 #define LOGMNG_NONEPROG "NONE"
 #define LOGMNG_WBUFSIZE 1024
+#define LOGMNG_MAXNUMLOGFILE 30
+
 
 /************************/
 /* Data Type Parameters */
 /************************/
-
 #define MAX_INT_LEN 10
 #define MAX_LONG_LEN 20
 #define MAX_FLOAT_LEN 40
@@ -109,44 +98,47 @@
 #define MAX_OBJNAME_LEN 50 
 #define MAXSTRINGLEN 10000
 #define BLOB_BLOCKSIZE 1024
+#define CLOB_BLOCKSIZE 1024
+#define STATICFIELDBUF 20 /* statically allocated buffer for field values, should be at least 8 byte */
 
+#define MAX_CAST 20
 /******************************/
 /* Lock Management Parameters */
 /******************************/
 
 /* the amount of semaphores used for locking database data files  */
-#define LCKMNG_NUM_DATAFILE_SEMA 911
+#define LCKMNG_NUM_DATAFILE_SEMA 1021
 
 /* number of semaphores allocated for buffer pool page locking */
-#define LCKMNG_NUM_BUFFERPOOL_SEMA 911
+#define LCKMNG_NUM_BUFFERPOOL_SEMA 1021
 
 /* number of semaphores allocated for record lock operations */
-#define LCKMNG_NUM_RECORD_SEMA 911
+#define LCKMNG_NUM_RECORD_SEMA 1021
 
 /* number of semaphores allocated for rb record lock operations */
-#define LCKMNG_NUM_RBRECORD_SEMA 911
+#define LCKMNG_NUM_RBRECORD_SEMA 1021
 
 /* number of semaphores allocated for rb record lock operations */
-#define LCKMNG_NUM_SYSRECORD_SEMA 911
+#define LCKMNG_NUM_SYSRECORD_SEMA 1021
 
 /* number of semaphores allocated for system page lock operations */
-#define LCKMNG_NUM_SYSPAGE_SEMA 911
+#define LCKMNG_NUM_SYSPAGE_SEMA 1021
 
 /* number of semaphores allocated for data page lock operations */
-#define LCKMNG_NUM_DATAPAGE_SEMA 911
+#define LCKMNG_NUM_DATAPAGE_SEMA 1021
 
 /* number of semaphores allocated for index page lock operations */
-#define LCKMNG_NUM_IDXPAGE_SEMA 911
+#define LCKMNG_NUM_IDXPAGE_SEMA 1021
 
 /* number of semaphores allocated for rb page lock operations */
-#define LCKMNG_NUM_RBPAGE_SEMA 911
+#define LCKMNG_NUM_RBPAGE_SEMA 1021
 
 /* number of record, system- and data page lock operations that can be done in parallel by one thread.
    Note: the number of lock operations may be higher that the number of available semaphores
    Value is valid per file-, bufferpool-, record-, system page- and data page locks */ 
 #define LCKMNG_NUMLOCKS 50
 
-/* the interval in usec a thread goes asleep waiting to achieve a lock.
+/* the interval in msec a thread goes asleep waiting to achieve a lock.
    Setting this value higher decreases CPU load but delays thread synchronization */
 #define LCKMNG_LOCKWAITDELAY 10
 
@@ -157,21 +149,30 @@
 
 /* FileHandler lock timeout in msec */
 #define FH_LOCKTIMEOUT 10000
+
 /* BufferPool lock timeout in msec */
+
 /* this was increased becasue for large bufferpool configurations, timeouts may 
    occur during checkpoint writes */
 #define BP_LOCKTIMEOUT 300000
+
 /* Page lock timeout in msec */
 #define PG_LOCKTIMEOUT 10000
+
 /* Record lock timeout in msec */
 #define RC_LOCKTIMEOUT 10000
+
 /* XMLSpace lock timeout in msec */
 #define XS_LOCKTIMEOUT 10000
+
 /* Transaction manager lock timeout in msec */
 #define TS_LOCKTIMEOUT 10000
+
 /* Database Manager lock timeout in msec */
 #define DBM_LOCKTIMEOUT 10000
-
+#define DBM_MAXLOCKTRIES 30
+/* Database manager lock delay timeout in msec, must be less 1000 ! */
+#define DBM_LOCKDELAY 800
 
 #define RECLOCKGROUP "RECLOCK"
 #define RBRECLOCKGROUP "RBRECLOCK"
@@ -191,14 +192,14 @@
    Increasing this values leads to a better usage of the bufferpool but 
    less performance */
  
-#define BUPMNG_MAXFIXTRIES 10
+#define BUPMNG_MAXFIXTRIES 30
 
 /* For each buffer page, a number of pointer entries ( sizeof int ) is reserved for free pointer management
    The number of pointers is calculated based on the current pagesize. BUPMNG_MINFREERATIO percent of the
    page is reserved for pointers to free blocks. 
    To avoid a minfree exception, the BUPMNG_MINFREERATIO value should be set to the minimum tuple size of the database
    */
- 
+
 #define BUPMNG_MINFREERATIO 10
 
 /* For some processor architecture ( SPARC ) , an adress alignment is required for pointer arithmetic 
@@ -214,8 +215,13 @@
 #define BUPMNG_STATDTFORMAT "%T %Y-%m-%d %Z"
 #define BUPMNG_STATOUTFORMAT "%d.%m.%Y %H:%M:%S"
 
+#define BUPMNG_FIXLOOP 3
+
 /* timeout value for synchronous checkpoint escape command ( in seconds ) */
-#define ESCCMDTIMEOUT 10
+#define ESCCMDTIMEOUT 60
+
+/* timeout value for logfile archiving ( in seconds ) */
+#define LOGARCHTIMEOUT 60
 
 /**********************/
 /* Network Parameters */
@@ -240,7 +246,7 @@
 #define NETMNG_WAITMSG_TIMEOUT 10000
 
 // maximum net msg size
-#define NETMNG_MSG_BUFLEN 500 
+#define NETMNG_MSG_BUFLEN 4096 
 
 /* prefix sign for unique col identification */
 #define NETMNG_COLPREFIX "c"
@@ -302,15 +308,19 @@
 #define XP_MAXCOLBUF 10000
 /* export/import predicate description buffer */
 #define XP_MAXPDBUF 10000
-
+#define XP_ROWINTERVAL 5000
 
 /********************/
 /* field value defs */
 /********************/
 
-#define BLOBCHUNKSIZE 10
-#define BLOBSEP "[],"
-#define DTFORMAT "%d.%m.%Y %H:%M:%S"
+#define BLOBCHUNKSIZE 1024
+#define CLOBCHUNKSIZE 1024
+#define LOBSEP "[],"
+
+#define DEFAULTDATEFORMAT "%d.%m.%Y"
+#define DEFAULTDATETIMEFORMAT "%d.%m.%Y %H:%M:%S"
+
 
 /*********************/
 /* system table defs */
@@ -333,8 +343,6 @@
 #define SYSTAB_BUINFO_ATTR "buinfo"
 #define SYSTAB_BUMSG_ATTR "msg"
 
-
-
 /*****************/
 /* grouping defs */
 /*****************/
@@ -379,15 +387,27 @@
 #define INFO_CLSN "CLSN"
 #define INFO_NLSN "NLSN"
 #define INFO_ARCHMODE "ArchMode"
+#define INFO_AUTOCORRECT "AutoCorrect"
+#define INFO_TABLECACHE "TableCache"
+#define INFO_QUERYCACHE "QueryCache"
+
+#define INFO_TABLECACHEMAXENTRY "TC MaxEntryNum"
+#define INFO_TABLECACHEMAXSIZE "TC MaxEntrySize"
+#define INFO_TABLECACHEUSEDSIZE "TC UsedSize"
+#define INFO_QUERYCACHEMAXENTRY "QC MaxEntryNum"
+#define INFO_QUERYCACHEMAXSIZE "QC MaxEntrySize"
+#define INFO_QUERYCACHEUSEDSIZE "QC UsedSize"
 
 #define INFO_LOGFILE "LogFile"
 #define INFO_LOGSIZE "LogSize"
 
+#define INFO_MAXCACHEIDLEN 300
+
 /*****************/
 /* role defs */
 /*****************/
 #define ROLE_ADMIN "admin"
-
+#define ROLE_JDBC "jdbc"
 
 /*********************/
 /* thread management */
@@ -402,6 +422,5 @@
 #define XML_VERSION_ATTR "version"
 #define XML_VERSION_VALUE "1.0"
 
-
 #endif
 
