@@ -6,35 +6,23 @@
 //      
 // Design and Implementation by Bjoern Lemke
 //     
-// (C)opyright 2000-2012 Bjoern Lemke
-//
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
-// 
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; see the file COPYING.  If not, write to
-// the Free Software Foundation, 59 Temple Place - Suite 330,
-// Boston, MA 02111-1307, USA.
+// (C)opyright 2000-2016 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
 // Class: CegoDistCursor
 // 
-// Description: 
+// Description: Table cursor for distributed table access
 //
-// Status: QG-2.6
+// Status: CLEAN
 //
 ///////////////////////////////////////////////////////////////////////////////
 
+// LFC INCLUDES
 #include <lfcbase/Net.h>
+// #include <lfcbase/Sleeper.h>
 
+// CEGO INCLUDES
 #include "CegoDistCursor.h"
 #include "CegoDistManager.h"
 #include "CegoSelect.h"
@@ -42,67 +30,69 @@
 #include "CegoJoinObject.h"
 #include "CegoXMLdef.h"
 
-
 CegoDistCursor::CegoDistCursor()
 {
 }
 
 CegoDistCursor::CegoDistCursor(CegoDistManager* pGTM, CegoContentObject *pCO)
 {
-
+    
     _isLocal = true;
     _moreTuple = false;
     
     _pSelect = 0;
     _pSH = 0;
     _pGTM = pGTM;
-    _tableName = pCO->getTabName();
+
+    _cursorObjectUsed = false;
     
+    _tableName = pCO->getTabName();
     _tableAlias = pCO->getName();
     _tabSetId = pCO->getTabSetId();
-
-    _pDBMng = _pGTM->getDBMng(); 
     
-    _pCO = pCO;
+    _pDBMng = _pGTM->getDBMng(); 
 
+    _pCache = _pDBMng->getTableCache(_tabSetId);
+    _isCached = false;
+    _pCacheArray = 0;
+    _pCacheList = 0;
+    _pCO = pCO;
+    
     _pTCLeft = 0;
     _pTCRight = 0;
     _pTC = 0;
     _pC = 0;
 
     _idxMatch = CegoAttrCond::INAPP;
+
+    checkType();
     
     _modId = _pDBMng->getModId("CegoDistCursor");
 
-
 }
 
 CegoDistCursor::~CegoDistCursor()
 {
+    
+    finishCaching();
+    
     if ( _pTC )
     {
 #ifdef CGDEBUG
 	_pDBMng->log(_modId, Logger::DEBUG, Chain("Releasing global table cursor for ") + _tableName);
 #endif
 
-	try {
-	    
-	    _pDBMng->unuseObject(_tabSetId, _tableName, CegoObject::TABLE, CegoDatabaseManager::SHARED);
-	}
-	catch ( Exception e )
-	{
-	    Chain msg;
-	    e.pop(msg);
-	    _pDBMng->log(_modId, Logger::LOGERR, Chain("Ignoring error : ") + msg);
-	}
 	delete _pTC;
     }
     if ( _pSelect )
     {
+
 	try
 	{
+#ifdef CGDEBUG
+	_pDBMng->log(_modId, Logger::DEBUG, Chain("Cleaning up select ..."));
+#endif
 	    _pSelect->cleanUp();
-	    _pDBMng->unuseObject(_tabSetId, _tableName, CegoObject::VIEW, CegoDatabaseManager::SHARED);
 	}
 	catch ( Exception e )
 	{
@@ -123,16 +113,12 @@ CegoDistCursor::~CegoDistCursor()
 	_pDBMng->log(_modId, Logger::DEBUG, Chain("Releasing global table cursor for ") + pCOLeft->getTabName());
 #endif
 	
-	if ( pCOLeft->getType() == CegoObject::VIEW || pCOLeft->getType() == CegoObject::TABLE )
-	    _pDBMng->unuseObject(pCOLeft->getTabSetId(), pCOLeft->getTabName(), pCOLeft->getType(), CegoDatabaseManager::SHARED);
 	delete _pTCLeft;
 	
 #ifdef CGDEBUG
 	_pDBMng->log(_modId, Logger::DEBUG, Chain("Releasing global table cursor for ") + pCORight->getTabName());
 #endif
 	
-	if ( pCORight->getType() == CegoObject::VIEW || pCORight->getType() == CegoObject::TABLE )
-	    _pDBMng->unuseObject(pCORight->getTabSetId(), pCORight->getTabName(), pCORight->getType(), CegoDatabaseManager::SHARED);
 	delete _pTCRight;
 		
     }
@@ -145,24 +131,101 @@ CegoDistCursor::~CegoDistCursor()
 	delete _pC;
     }
 
+    unuseCursorObject();
+    
     if ( _pSH )
     {
 	_pDBMng->releaseSession(_pSH);
     }
+#ifdef CGDEBUG
+    _pDBMng->log(_modId, Logger::DEBUG, Chain("Destructor for DistCursor finished"));
+#endif
+
+    if ( _pCacheList )
+    {
+	delete _pCacheList;
+    }
+
+    if ( _pCacheArray && _pCache )
+    {
+	// FOR DEBUGGING : sleep for a while
+	// cout << "Sleeping for 10 sec before destructing cursor  ..." << endl;
+	// Sleeper s;
+	// s.secSleep(10);
+
+	_pCache->releaseEntry(_tabSetId, _tableName);
+	_pCacheArray=0;
+	_isCached = true;
+    }
+}
+
+void CegoDistCursor::useCursorObject()
+{
+    if ( _cursorObjectUsed == false )
+    {
+	if ( _pCO->getType() == CegoObject::VIEW )
+	{
+	    _pDBMng->useObject(_tabSetId, _tableName, CegoObject::VIEW, CegoDatabaseManager::SHARED, _pGTM->getThreadId());	
+	}
+	else if ( _pCO->getType() == CegoObject::TABLE )
+	{
+	    _pDBMng->useObject(_tabSetId, _tableName, CegoObject::TABLE, CegoDatabaseManager::SHARED, _pGTM->getThreadId());	
+	}
+	else if ( _pCO->getType() == CegoObject::JOIN )
+	{
+	    CegoJoinObject *pJCO = (CegoJoinObject*)_pCO;
+	    CegoContentObject *pCOLeft = pJCO->getLeftObject();
+	    CegoContentObject *pCORight = pJCO->getRightObject();
+
+	    if ( pCOLeft->getType() == CegoObject::VIEW || pCOLeft->getType() == CegoObject::TABLE )
+		_pDBMng->useObject(pCOLeft->getTabSetId(), pCOLeft->getTabName(), pCOLeft->getType(), CegoDatabaseManager::SHARED, _pGTM->getThreadId());	
+	    if ( pCORight->getType() == CegoObject::VIEW || pCORight->getType() == CegoObject::TABLE )
+		_pDBMng->useObject(pCORight->getTabSetId(), pCORight->getTabName(), pCORight->getType(), CegoDatabaseManager::SHARED, _pGTM->getThreadId());	
+	}
+
+	_cursorObjectUsed = true;
+    }
+}
+
+void CegoDistCursor::unuseCursorObject()
+{
+    if ( _cursorObjectUsed == true )
+    {
+	if ( _pCO->getType() == CegoObject::VIEW )
+	{
+	    _pDBMng->unuseObject(_tabSetId, _tableName, CegoObject::VIEW);	
+	}
+	else if ( _pCO->getType() == CegoObject::TABLE )
+	{
+	    _pDBMng->unuseObject(_tabSetId, _tableName, CegoObject::TABLE);	
+	}
+	else if ( _pCO->getType() == CegoObject::JOIN )
+	{	
+	    CegoJoinObject *pJCO = (CegoJoinObject*)_pCO;
+	    CegoContentObject *pCOLeft = pJCO->getLeftObject();
+	    CegoContentObject *pCORight = pJCO->getRightObject();
+
+	    if ( pCOLeft->getType() == CegoObject::VIEW || pCOLeft->getType() == CegoObject::TABLE )
+		_pDBMng->unuseObject(pCOLeft->getTabSetId(), pCOLeft->getTabName(), pCOLeft->getType());	
+	    if ( pCORight->getType() == CegoObject::VIEW || pCORight->getType() == CegoObject::TABLE )
+		_pDBMng->unuseObject(pCORight->getTabSetId(), pCORight->getTabName(), pCORight->getType());	
+	}
+
+	_cursorObjectUsed = false;
+    }
 }
 
 void CegoDistCursor::checkType()
 {
-
+    useCursorObject();
+    
     if ( _pCO->getType() == CegoObject::VIEW )
-
     {
+
 #ifdef CGDEBUG
 	_pDBMng->log(_modId, Logger::DEBUG, Chain("View ") + _tableName + Chain(" detected as the cursor source"));
 #endif
 
-	_pDBMng->useObject(_tabSetId, _tableName, CegoObject::VIEW, CegoDatabaseManager::SHARED, _pGTM->getThreadId());
-
 	CegoView *pView;
 
 	try
@@ -171,7 +234,7 @@ void CegoDistCursor::checkType()
 	}
 	catch ( Exception e )
 	{
-	    _pDBMng->unuseObject(_tabSetId, _tableName, CegoObject::VIEW, CegoDatabaseManager::SHARED);
+	    _pDBMng->unuseObject(_tabSetId, _tableName, CegoObject::VIEW);
 	    throw e;
 	}
 	_pSelect = pView->getSelect();
@@ -198,7 +261,8 @@ void CegoDistCursor::checkType()
 #endif
 
 	    _pTC = new CegoTableCursor(_pGTM, _tabSetId, _tableName, false);	
-	    _pDBMng->useObject(_tabSetId, _tableName, CegoObject::TABLE, CegoDatabaseManager::SHARED, _pGTM->getThreadId());
+
+	    _cacheSchema = oe.getSchema();
 	    _isLocal = true;
 	}
 	else
@@ -231,24 +295,16 @@ void CegoDistCursor::checkType()
     {	
 
 	CegoJoinObject *pJCO = (CegoJoinObject*)_pCO;
-
 	CegoContentObject *pCOLeft = pJCO->getLeftObject();
 	CegoContentObject *pCORight = pJCO->getRightObject();
 	
 	_pTCLeft = new CegoDistCursor(_pGTM, pCOLeft);
-	_pTCLeft->checkType();
-
-	if ( pCOLeft->getType() == CegoObject::VIEW || pCOLeft->getType() == CegoObject::TABLE )
-	    _pDBMng->useObject(pCOLeft->getTabSetId(), pCOLeft->getTabName(), pCOLeft->getType(), CegoDatabaseManager::SHARED, _pGTM->getThreadId());
-	
 	_pTCRight = new CegoDistCursor(_pGTM, pCORight);
-	_pTCRight->checkType();
-
-	if ( pCORight->getType() == CegoObject::VIEW || pCORight->getType() == CegoObject::TABLE )
-	    _pDBMng->useObject(pCORight->getTabSetId(), pCORight->getTabName(), pCORight->getType(), CegoDatabaseManager::SHARED, _pGTM->getThreadId());
 	
     }
 
+    // we have to release object here to avoid dead locks
+    unuseCursorObject();    
 }
 
 void CegoDistCursor::distSetup()
@@ -270,8 +326,30 @@ void CegoDistCursor::distSetup()
 	_pSelect->checkValidRef();
     }
     else if ( _pCO->getType() == CegoObject::TABLE )
-    {
-	// nothing to do
+    {	
+	CegoAttrCond attrCond; // empty cond
+	_pTC->setup(attrCond);
+	
+	if ( _pCache )
+	{
+
+	    _useCache = true;
+	    
+	    finishCaching();
+
+	    _pCacheArray = _pCache->claimEntry( _tabSetId, _tableName, _cacheRows, _cacheCols);
+
+	    if ( _pCacheArray )
+	    {
+		_isCached = true;
+	    }
+	    else
+	    {
+		_isCached = false;
+		_cacheEntrySize = 0;
+		_pCacheList = new ListT< ListT< CegoFieldValue > >;
+	    }
+	}	
     }
     else if ( _pCO->getType() == CegoObject::SYSTEM )
     {
@@ -279,16 +357,13 @@ void CegoDistCursor::distSetup()
     }
     else if ( _pCO->getType() == CegoObject::JOIN )
     {
-	CegoAttrCond attrCond;
+	CegoAttrCond attrCond;	
 	joinSetup(attrCond);
     }
 }
 
 void CegoDistCursor::distSetup( const CegoAttrCond& attrCond)
 {
-
-    // cout << "Attr Comp Size = " << attrCond.numComp() << endl;
-
     if ( attrCond.numComp() == 0 )
 	return distSetup();
 
@@ -302,6 +377,7 @@ void CegoDistCursor::distSetup( const CegoAttrCond& attrCond)
 #ifdef CGDEBUG
 	_pDBMng->log(_modId, Logger::DEBUG, Chain("Setting view condition ") + attrCond.toChain());
 #endif
+
 	_pSelect->setViewCond(attrCond);
 	_pSelect->setTabSetId(_tabSetId);
 	_pSelect->prepare();
@@ -317,15 +393,43 @@ void CegoDistCursor::distSetup( const CegoAttrCond& attrCond)
     {
 	if ( _isLocal == true )
 	{
+
+	    if ( _pCache )
+	    {
+		// check if a previous cursor trace has to be finished to complete cache 
+		finishCaching();
+	    }
+	    
+	    _useCache = false;
+		
 	    _idxMatch = _pTC->setup(attrCond);
 	    
 	    if ( _idxMatch != CegoAttrCond::FULL )
 	    {	       
 		_doEval=true;
+
+		// if index match was inappropriate, all tuples are retrieved from table cursor
+		// In this case, we can use cache
+		if ( _pCache && _idxMatch == CegoAttrCond::INAPP )
+		{
+		    _useCache = true;
+		    _pCacheArray = _pCache->claimEntry( _tabSetId, _tableName, _cacheRows, _cacheCols);
+		    
+		    if ( _pCacheArray )
+		    {
+			_isCached = true;
+		    }
+		    else
+		    {
+			_isCached = false;
+			_cacheEntrySize = 0;
+			_pCacheList = new ListT< ListT< CegoFieldValue > >;
+		    }
+		}
 	    }
-
+	    
 	    _cursorCond = attrCond;
-
+	   
 	}
     }
     else if ( _pCO->getType() == CegoObject::SYSTEM )
@@ -340,23 +444,35 @@ void CegoDistCursor::distSetup( const CegoAttrCond& attrCond)
 	_pDBMng->log(_modId, Logger::DEBUG, Chain("Setting join condition ") + attrCond.toChain());	
 #endif
 
+	_cursorCond = attrCond;
+	_doEval = true;
 	joinSetup(attrCond);
     }
 }
 
-bool CegoDistCursor::nextTuple(ListT<CegoField>* flArray, int pos, int size)
+bool CegoDistCursor::nextTuple(ListT<CegoField>** pFLA, int pos, int size)
 {
-    while ( getTuple(flArray, pos, size) ) 
-    {
+    useCursorObject();
+    
+    while ( getTuple(pFLA, pos, size) )
+    {	
 	if ( _doEval == true )
 	{
-	    if ( evalCondition(flArray, pos, size) )
+	    if ( evalCondition(pFLA, pos, size) )
 	    {
+#ifdef CGDEBUG
+		_pDBMng->log(_modId, Logger::DEBUG, Chain("Returning tuple with eval condition"));
+#endif
+
 		return true;
 	    }
 	}
 	else
 	{
+#ifdef CGDEBUG
+	    _pDBMng->log(_modId, Logger::DEBUG, Chain("Returning tuple without eval condition"));
+#endif
+
 	    return true;
 	}
     }
@@ -364,30 +480,22 @@ bool CegoDistCursor::nextTuple(ListT<CegoField>* flArray, int pos, int size)
     return false;
 }
 
-bool CegoDistCursor::nextTuple(ListT<CegoField>& fl)
+bool CegoDistCursor::evalCondition(ListT<CegoField>** pFLA, int pos, int size)
 {
-    return nextTuple(&fl, 0, 1);
-}
-
-bool CegoDistCursor::evalCondition(ListT<CegoField>* flArray, int pos, int size)
-{
-
     CegoAttrComp *pAC = _cursorCond.getAttrCompSet().First();
-
+    
     while ( pAC )
     {
 
 	CegoField *pF = 0;
-	
+	    
 	int i=0;
-	int lpos = pos;
 	while ( pF == 0 && i < size)
 	{	   
 	    if ( _pCO->getType() == CegoObject::VIEW || _pCO->getType() == CegoObject::TABLE )
-		pF = flArray[pos].Find( CegoField(_tableAlias, pAC->getAttrName()));
+		pF = pFLA[pos+i]->Find( CegoField(_tableAlias, pAC->getAttrName()));
 	    else
-		pF = flArray[pos].Find( CegoField(Chain(), pAC->getAttrName()));
-	    lpos++;
+		pF = pFLA[pos+i]->Find( CegoField(Chain(), pAC->getAttrName()) );
 	    i++;
 	}
 	
@@ -401,7 +509,7 @@ bool CegoDistCursor::evalCondition(ListT<CegoField>* flArray, int pos, int size)
 		    return false;	    
 	    }
 	    else if ( pAC->getCompMode() == CegoAttrComp::ISLIKE )
-	    {		
+	    {
 		if ( pAC->getMatcher()->match(pF->getValue().valAsChain()) == false)		
 		    return false;
 	       
@@ -467,25 +575,22 @@ bool CegoDistCursor::evalCondition(ListT<CegoField>* flArray, int pos, int size)
     return true;
 }
 
-bool CegoDistCursor::getTuple(ListT<CegoField>* flArray, int offset, int size)
+bool CegoDistCursor::getTuple(ListT<CegoField>** pFLA, int offset, int size)
 {
-
     if ( _pCO->getType() == CegoObject::VIEW )
     {
 	try 
-	{
-
-	    _moreTuple = _pSelect->nextTuple(flArray[offset]);
-
+	{	    
+	    _moreTuple = _pSelect->nextTuple(*pFLA[offset]);
+	    
 	    if ( _moreTuple )
 	    {
-
-		CegoField *pF = flArray[offset].First();
+		CegoField *pF = pFLA[offset]->First();
 		while ( pF ) 
 		{
 		    pF->setTableName(_tableName);
 		    pF->setTableAlias(_tableAlias);
-		    pF = flArray[offset].Next();
+		    pF = pFLA[offset]->Next();
 		}
 	    }
 	    return _moreTuple;
@@ -500,30 +605,178 @@ bool CegoDistCursor::getTuple(ListT<CegoField>* flArray, int offset, int size)
     {
 	if ( _isLocal )
 	{
-	    
-	    CegoDataPointer dp;
-	    if ( _isFirst )
+	    if ( _useCache )
 	    {
-		_moreTuple = _pTC->getFirst(flArray[offset], dp);
-		_isFirst = false;
+		if ( _isCached )
+		{		    
+		    CegoFieldValue** pCacheRow = 0;
+		    if ( _isFirst ) 
+		    {
+			_isFirst = false;
+			_cachePos = 0;
+			if ( _cacheRows > 0 )
+			{
+			    pCacheRow = _pCacheArray[_cachePos];
+			    _cachePos++;			
+			}			  
+		    }
+		    else
+		    {
+			if ( _cachePos < _cacheRows )
+			{			    
+			    pCacheRow = _pCacheArray[_cachePos];
+			    _cachePos++;
+			}
+		    }
+		    	
+		    if ( pCacheRow  )
+		    {
+			CegoField *pF = pFLA[offset]->First();
+			
+			while ( pF )
+			{
+			    CegoField *pSF = _cacheSchema.First();
+			    
+			    int pos = 0;
+			    while ( pF && pSF && ( *pF != *pSF ) )
+			    {				
+				pSF = _cacheSchema.Next();
+				pos++;
+			    }
+			    if ( *pF == *pSF )
+			    {
+				pF->setValue(*pCacheRow[pos]);
+			    }
+			    pF = pFLA[offset]->Next();
+			}
+
+			_moreTuple = true;
+		    }
+		    else
+		    {
+			
+			if ( _pCacheArray && _pCache )
+			{
+			    // FOR DEBUGGING : sleep for a while
+			    // cout << "Sleeping for 10 sec before end of cursor  ..." << endl;
+			    // Sleeper s;
+			    // s.secSleep(10);
+			    
+			    _pCache->releaseEntry( _tabSetId, _tableName);
+			    _pCacheArray = 0;
+			    _isCached = true;
+			}
+
+			_moreTuple = false;
+		    }
+
+		    return _moreTuple;
+		}
+		else // still not cached, so we have to retrieve and cache it
+		{
+
+		    CegoDataPointer dp;
+		    if ( _isFirst )
+		    {
+			_moreTuple = _pTC->getFirst(_cacheSchema, dp);			
+			_isFirst = false;
+		    }
+		    else
+		    {
+			_moreTuple = _pTC->getNext(_cacheSchema, dp);	    
+		    }
+		    
+		    if ( _moreTuple )
+		    {
+			// put tuple into cache
+
+			if ( _pCacheList )
+			{
+			    ListT<CegoFieldValue> staticFieldList;
+			    CegoField* pF = _cacheSchema.First();
+			    while ( pF )
+			    {			
+				staticFieldList.Insert(pF->getValue().getLocalCopy());			       
+				_cacheEntrySize += pF->getValue().size();
+				pF = _cacheSchema.Next();
+			    }
+
+			    if ( _pCache->getMaxSize() > _cacheEntrySize )
+			    {
+				_pCacheList->Insert(staticFieldList);
+			    }
+			    else
+			    {
+				delete _pCacheList;
+				_pCacheList = 0;
+			    }
+			}
+
+			// propagate  tuple to flArray
+			CegoField *pF = pFLA[offset]->First();			
+			while ( pF )
+			{
+			    CegoField *pSF = _cacheSchema.First();
+			    while ( pF && pSF && ( *pF != *pSF ) )
+			    {
+				pSF = _cacheSchema.Next();
+			    }
+			    // cout << "Comparing " << pF->getTableAlias() << "." << pF->getAttrName()  << " and " << pSF->getTableAlias() << "." << pSF->getAttrName() << endl;
+			    if ( *pF == *pSF )
+			    {
+				// cout << "MATCH" << endl;
+				pF->setValue(pSF->getValue());
+			    }
+			    pF = pFLA[offset]->Next();
+			}
+		    }
+		    else
+		    {
+
+			if ( _pCacheList )
+			{
+			    // put table cache entry into cache
+			    // cout << "PUT TCE " << _tableName << " into cache" << endl;
+			    _pCache->addEntry( _tabSetId, _tableName, _pCacheList );
+			    // cout << "PUT DONE" << endl;
+			    _isCached = true;
+			    delete _pCacheList;
+			    _pCacheList = 0;
+			}
+		    }
+
+		    return _moreTuple;
+
+		}
+	    }
+	    else // retrieve tuple via native table cursor
+	    {
+		// cout << "Retrieve native .." << endl;
+		CegoDataPointer dp;
+		if ( _isFirst )
+		{
+		    _moreTuple = _pTC->getFirst(*pFLA[offset], dp);
+		    _isFirst = false;
+	        }
+		else
+		{
+		    _moreTuple = _pTC->getNext(*pFLA[offset], dp);	    
+		}
+		
+		return _moreTuple;
 	    }
-	    else
-		_moreTuple = _pTC->getNext(flArray[offset], dp);	    
-	    return _moreTuple;
-	    
 	}
 	else
 	{
-	    flArray[offset] = _schema;
+	    *pFLA[offset] = _schema;
 	    _moreTuple = false;
 
-	    if ( _pSH->receiveTableData(flArray[offset]) == CegoDbHandler::DB_DATA )
+	    if ( _pSH->receiveTableData(*pFLA[offset]) == CegoDbHandler::DB_DATA )
 	    {
 		_moreTuple = true;
 	    }
 
-	    return _moreTuple;
-	    
+	    return _moreTuple;	    
 	}
     }
     else if ( _pCO->getType() == CegoObject::SYSTEM )
@@ -531,13 +784,13 @@ bool CegoDistCursor::getTuple(ListT<CegoField>* flArray, int offset, int size)
 
 	if ( _pC )
 	{
-	    
+
 	    CegoDataPointer dp;
 
 	    int len;
 	    char *pc;
 	    if ( _isFirst )
-	    {	       
+	    {
 		pc = (char*)_pC->getFirst(len, dp);		
 		_isFirst=false;
 	    }
@@ -548,12 +801,18 @@ bool CegoDistCursor::getTuple(ListT<CegoField>* flArray, int offset, int size)
 
 	    if ( pc && len > 0 )
 	    {
-		int tid;
-		int tastep;	    
+
+		unsigned long long tid;
+		unsigned long long tastep;
 		CegoTupleState ts;
+
+		int toff = CegoQueryHelper::decodeTupleHeader(tid, tastep, ts, pc);
+
+		char* tp = pc + toff;
+		int tlen = len - toff;
+
+		CegoQueryHelper::decodeFVL(*pFLA[offset], tp, tlen);
 		
-		CegoQueryHelper qh;
-		qh.decodeFVL(flArray[offset], pc, len, tid, tastep, ts);			    
 		return true;   
 	    }
 	    else
@@ -577,7 +836,7 @@ bool CegoDistCursor::getTuple(ListT<CegoField>* flArray, int offset, int size)
 	    
 	    if ( pName )
 	    {
-		CegoField *pF = flArray[offset].Find(CegoField(_tableAlias, Chain(SYSTAB_NAME_ATTR)));
+		CegoField *pF = pFLA[offset]->Find(CegoField(_tableAlias, Chain(SYSTAB_NAME_ATTR)));
 		if ( pF )		    
 		{
 		    pF->setValue( CegoFieldValue(VARCHAR_TYPE, *pName));
@@ -593,16 +852,16 @@ bool CegoDistCursor::getTuple(ListT<CegoField>* flArray, int offset, int size)
 		    if ( _tableName == Chain(SYSTAB_TABLE_ID)  )
 			pageCount = _pGTM->getPageCount(_tabSetId, *pName, CegoObject::TABLE);
 		    else if ( _tableName == Chain(SYSTAB_INDEX_ID)  )
-			pageCount = _pGTM->getPageCount(_tabSetId, *pName, CegoObject::INDEX);
+			pageCount = _pGTM->getPageCount(_tabSetId, *pName, CegoObject::AVLTREE);
 		    else if ( _tableName == Chain(SYSTAB_BTREE_ID)  )
 			pageCount = _pGTM->getPageCount(_tabSetId, *pName, CegoObject::BTREE);
 
-		    CegoField *pF = flArray[offset].Find(CegoField(_tableAlias, Chain(SYSTAB_SIZE_ATTR)));
+		    CegoField *pF = pFLA[offset]->Find(CegoField(_tableAlias, Chain(SYSTAB_SIZE_ATTR)));
 		    if ( pF )		    
 		    {
 			pF->setValue( CegoFieldValue(INT_TYPE, pageCount));
 		    }
-		    pF = flArray[offset].Find(CegoField(_tableAlias, Chain(SYSTAB_STATUS_ATTR)));
+		    pF = pFLA[offset]->Find(CegoField(_tableAlias, Chain(SYSTAB_STATUS_ATTR)));
 		    if ( pF )
 		    {
 			if ( pageCount > 0 )
@@ -618,7 +877,7 @@ bool CegoDistCursor::getTuple(ListT<CegoField>* flArray, int offset, int size)
 		    if ( _pGTM->checkCompView(_tabSetId, *pName) )
 			status = Chain("compiled");
 		    
-		    CegoField *pF = flArray[offset].Find(CegoField(_tableAlias, Chain(SYSTAB_STATUS_ATTR)));
+		    CegoField *pF = pFLA[offset]->Find(CegoField(_tableAlias, Chain(SYSTAB_STATUS_ATTR)));
 		    if ( pF )		    
 		    {
 			pF->setValue( CegoFieldValue(VARCHAR_TYPE, status));
@@ -630,7 +889,7 @@ bool CegoDistCursor::getTuple(ListT<CegoField>* flArray, int offset, int size)
 		    if ( _pGTM->checkCompProcedure(_tabSetId, *pName) )
 			status = Chain("compiled");
 		    
-		    CegoField *pF = flArray[offset].Find(CegoField(_tableAlias, Chain(SYSTAB_STATUS_ATTR)));
+		    CegoField *pF = pFLA[offset]->Find(CegoField(_tableAlias, Chain(SYSTAB_STATUS_ATTR)));
 		    if ( pF )		    
 		    {
 			pF->setValue( CegoFieldValue(VARCHAR_TYPE, status));
@@ -647,8 +906,7 @@ bool CegoDistCursor::getTuple(ListT<CegoField>* flArray, int offset, int size)
 	return _moreTuple;
     }
     else if ( _pCO->getType() == CegoObject::JOIN )
-    {
-
+    {	
 	if ( _isFirst )
 	{
 	    _moreLeft = true;
@@ -659,52 +917,41 @@ bool CegoDistCursor::getTuple(ListT<CegoField>* flArray, int offset, int size)
 	
 	if ( pJO->getJoinType() == CegoJoinObject::INNER )
 	{
+	    
 	    bool isEvaluated = false;
 	    
 	    while ( isEvaluated == false && _moreLeft )
 	    {
 		if ( _isFirst )
-		{		
-		    _pTCLeft->distSetup(_cursorCond);
-		    
-		    _moreLeft = _pTCLeft->nextTuple(flArray, offset, size-1);
-		    
+		{
+		    _pTCLeft->distSetup(_outerCond);
+		    _moreLeft = _pTCLeft->nextTuple(pFLA, offset, size-1);
 		    if ( _moreLeft )
 		    {			
-			_innerCond.setup(flArray, offset, size-1);
+			_innerCond.setup(pFLA, offset);
 
 			_pTCRight->reset();
-
+			
 			if ( _isAttrCondValid )
 			    _pTCRight->distSetup(_innerCond);
 			else
 			    _pTCRight->distSetup();
 
-			_moreRight = _pTCRight->nextTuple(flArray, offset+size-1, 1);
+			_moreRight = _pTCRight->nextTuple(pFLA, offset+size-1, 1);
 		    }
 
-		    /*
-		    if ( _moreLeft )
-			cout << "MoreLeft true" << endl;
-		    if ( _moreRight )
-			cout << "MoreRight true" << endl;
-			
-
-		    cout << "First tuple finished.." << endl;
-		    */
-
 		    _isFirst = false;
 		}
 		else if ( _moreRight )
 		{
 
-		    _moreRight = _pTCRight->nextTuple(flArray, offset+size-1, 1);
+		    _moreRight = _pTCRight->nextTuple(pFLA, offset+size-1, 1);
 		    if ( _moreRight == false )
 		    {
-			_moreLeft = _pTCLeft->nextTuple(flArray, offset, size-1);
+			_moreLeft = _pTCLeft->nextTuple(pFLA, offset, size-1);
 			if ( _moreLeft )
 			{
-			    _innerCond.setup(flArray, offset, size-1);
+			    _innerCond.setup(pFLA, offset);
 
 			    _pTCRight->reset();
 
@@ -713,16 +960,16 @@ bool CegoDistCursor::getTuple(ListT<CegoField>* flArray, int offset, int size)
 			    else
 				_pTCRight->distSetup();
 
-			    _moreRight = _pTCRight->nextTuple(flArray, offset+size-1, 1);
+			    _moreRight = _pTCRight->nextTuple(pFLA, offset+size-1, 1);
 			}
 		    }
 		}
 		else if ( _moreLeft )
 		{
-		    _moreLeft = _pTCLeft->nextTuple(flArray, offset, size-1);
+		    _moreLeft = _pTCLeft->nextTuple(pFLA, offset, size-1);
 		    if ( _moreLeft )
 		    {
-			_innerCond.setup(flArray, offset, size-1);
+			_innerCond.setup(pFLA, offset);
 
 			_pTCRight->reset();
 
@@ -731,27 +978,23 @@ bool CegoDistCursor::getTuple(ListT<CegoField>* flArray, int offset, int size)
 			else
 			    _pTCRight->distSetup();
 
-			_moreRight = _pTCRight->nextTuple(flArray, offset+size-1, 1);
+			_moreRight = _pTCRight->nextTuple(pFLA, offset+size-1, 1);
 		    }
 		}
 
 		if ( _moreLeft && _moreRight )
 		{
 		    if ( _evalPredicate )
-		    {
-			
-			CegoQueryHelper queryHelper;
+		    {		       
 			
 			pJO->getPredDesc()->clearAttrCache();
 			
-			isEvaluated = queryHelper.evalPredicate(0,
-								0,
-								0,
-								flArray,
-								offset,
-								size,
-								pJO->getPredDesc(),
-								0);
+			isEvaluated = CegoQueryHelper::evalPredicate(0,
+								     0,
+								     pFLA,
+								     offset,
+								     pJO->getPredDesc(),
+								     0);
 		    }
 		    else
 		    {			
@@ -772,13 +1015,13 @@ bool CegoDistCursor::getTuple(ListT<CegoField>* flArray, int offset, int size)
 		
 		if ( _isFirst )
 		{
-		    _pTCLeft->distSetup(_cursorCond);
+		    _pTCLeft->distSetup(_outerCond);
 		    
-		    _moreLeft = _pTCLeft->nextTuple(flArray, offset, size-1);
+		    _moreLeft = _pTCLeft->nextTuple(pFLA, offset, size-1);
 		    
 		    if ( _moreLeft )
 		    {
-			_innerCond.setup(flArray, offset, size-1);
+			_innerCond.setup(pFLA, offset);
 			
 			_pTCRight->reset();
 			
@@ -791,20 +1034,20 @@ bool CegoDistCursor::getTuple(ListT<CegoField>* flArray, int offset, int size)
 			    _pTCRight->distSetup();
 			}
 			
-			_moreRight = _pTCRight->nextTuple(flArray, offset+size-1, 1);
+			_moreRight = _pTCRight->nextTuple(pFLA, offset+size-1, 1);
 		    }
 		    _isFirst = false;
 		}
 		else if ( _moreRight )
 		{
-		    _moreRight = _pTCRight->nextTuple(flArray, offset+size-1, 1);
+		    _moreRight = _pTCRight->nextTuple(pFLA, offset+size-1, 1);
 		    
 		    if ( _moreRight == false )
 		    {
-			_moreLeft = _pTCLeft->nextTuple(flArray, offset, size-1);
+			_moreLeft = _pTCLeft->nextTuple(pFLA, offset, size-1);
 			if ( _moreLeft )
 			{
-			    _innerCond.setup(flArray, offset, size-1);
+			    _innerCond.setup(pFLA, offset);
 			    
 			    _pTCRight->reset();
 			    
@@ -817,16 +1060,16 @@ bool CegoDistCursor::getTuple(ListT<CegoField>* flArray, int offset, int size)
 				_pTCRight->distSetup();
 			    }
 			    
-			    _moreRight = _pTCRight->nextTuple(flArray, offset+size-1, 1);
+			    _moreRight = _pTCRight->nextTuple(pFLA, offset+size-1, 1);
 			}
 		    }		
 		}
 		else if ( _moreLeft )
 		{
-		    _moreLeft = _pTCLeft->nextTuple(flArray, offset, size-1);
+		    _moreLeft = _pTCLeft->nextTuple(pFLA, offset, size-1);
 		    if ( _moreLeft )
 		    {
-			_innerCond.setup(flArray, offset, size-1);
+			_innerCond.setup(pFLA, offset);
 			
 			_pTCRight->reset();
 			
@@ -839,23 +1082,19 @@ bool CegoDistCursor::getTuple(ListT<CegoField>* flArray, int offset, int size)
 			    _pTCRight->distSetup();
 			}
 			
-			_moreRight = _pTCRight->nextTuple(flArray, offset+size-1, 1);
+			_moreRight = _pTCRight->nextTuple(pFLA, offset+size-1, 1);
 		    }
 		}
 		
 		if ( _evalPredicate && _moreLeft && _moreRight )
-		{
-		    
-		    CegoQueryHelper queryHelper;
+		{		    
 		    pJO->getPredDesc()->clearAttrCache();
-		    isEvaluated = queryHelper.evalPredicate(0,
-							    0,
-							    0,
-							    flArray,
-							    offset,
-							    size,
-							    pJO->getPredDesc(),
-							    0);		   
+		    isEvaluated = CegoQueryHelper::evalPredicate(0,
+								 0,
+								 pFLA,
+								 offset,
+								 pJO->getPredDesc(),
+								 0);		   
 		}
 		else
 		{
@@ -868,12 +1107,12 @@ bool CegoDistCursor::getTuple(ListT<CegoField>* flArray, int offset, int size)
 		if ( _moreRight == false )
 		{
 		    // make right fields null
-		    CegoField *pF = flArray[offset+size-1].First();
+		    CegoField *pF = pFLA[offset+size-1]->First();
 		    while ( pF )
 		    {
 			CegoFieldValue nullValue;
 			pF->setValue(nullValue);
-			pF = flArray[offset+size-1].Next();
+			pF = pFLA[offset+size-1]->Next();
 		    }
 		}
 		return true;
@@ -892,11 +1131,11 @@ bool CegoDistCursor::getTuple(ListT<CegoField>* flArray, int offset, int size)
 		{		
 		    _pTCRight->distSetup(_cursorCond);
 		    
-		    _moreRight = _pTCRight->nextTuple(flArray, offset+size-1, 1);
+		    _moreRight = _pTCRight->nextTuple(pFLA, offset+size-1, 1);
 		    
 		    if ( _moreRight )
 		    {
-			_innerCond.setup(flArray, offset+size-1, 1);
+			_innerCond.setup(pFLA, offset+size-1);
 
 			_pTCLeft->reset();
 			
@@ -905,21 +1144,21 @@ bool CegoDistCursor::getTuple(ListT<CegoField>* flArray, int offset, int size)
 			else
 			    _pTCLeft->distSetup();
 
-			_moreLeft = _pTCLeft->nextTuple(flArray, offset, size-1);
+			_moreLeft = _pTCLeft->nextTuple(pFLA, offset, size-1);
 
 		    }
 		    _isFirst = false;
 		}
 		else if ( _moreLeft )
 		{
-		    _moreLeft = _pTCLeft->nextTuple(flArray, offset, size-1);
+		    _moreLeft = _pTCLeft->nextTuple(pFLA, offset, size-1);
 		    
 		    if ( _moreLeft == false )
 		    {
-			_moreRight = _pTCRight->nextTuple(flArray, offset+size-1, 1);
+			_moreRight = _pTCRight->nextTuple(pFLA, offset+size-1, 1);
 			if ( _moreRight )
 			{
-			    _innerCond.setup(flArray, offset+size-1, 1);
+			    _innerCond.setup(pFLA, offset+size-1);
 			    
 			    _pTCLeft->reset();
 			    
@@ -928,16 +1167,16 @@ bool CegoDistCursor::getTuple(ListT<CegoField>* flArray, int offset, int size)
 			    else
 				_pTCLeft->distSetup();
 			    
-			    _moreLeft = _pTCLeft->nextTuple(flArray, offset, size-1);
+			    _moreLeft = _pTCLeft->nextTuple(pFLA, offset, size-1);
 			}
 		    }		
 		}
 		else if ( _moreRight )
 		{
-		    _moreRight = _pTCRight->nextTuple(flArray, offset+size-1, 1);
+		    _moreRight = _pTCRight->nextTuple(pFLA, offset+size-1, 1);
 		    if ( _moreRight )
 		    {
-			_innerCond.setup(flArray, offset+size-1, 1);
+			_innerCond.setup(pFLA, offset+size-1);
 			
 			_pTCLeft->reset();
 			
@@ -946,23 +1185,19 @@ bool CegoDistCursor::getTuple(ListT<CegoField>* flArray, int offset, int size)
 			else
 			    _pTCLeft->distSetup();
 			
-			_moreLeft = _pTCLeft->nextTuple(flArray, offset, size-1);
+			_moreLeft = _pTCLeft->nextTuple(pFLA, offset, size-1);
 		    }
 		}
 
 		if ( _evalPredicate && _moreLeft && _moreRight )
-		{
-		    
-		    CegoQueryHelper queryHelper;
+		{		    
 		    pJO->getPredDesc()->clearAttrCache();
-		    isEvaluated = queryHelper.evalPredicate(0,
-							    0,
-							    0,
-							    flArray,
-							    offset,
-							    size,
-							    pJO->getPredDesc(),
-							    0);		   
+		    isEvaluated = CegoQueryHelper::evalPredicate(0,
+								 0,
+								 pFLA,
+								 offset,
+								 pJO->getPredDesc(),
+								 0);		   
 		}
 		else
 		{
@@ -977,12 +1212,12 @@ bool CegoDistCursor::getTuple(ListT<CegoField>* flArray, int offset, int size)
 		    for ( int i=0; i<size-1; i++)
 		    {
 			// make right fields null
-			CegoField *pF = flArray[offset+i].First();
+			CegoField *pF = pFLA[offset+i]->First();
 			while ( pF )
 			{
 			    CegoFieldValue nullValue;
 			    pF->setValue(nullValue);
-			    pF = flArray[offset+i].Next();
+			    pF = pFLA[offset+i]->Next();
 			}
 		    }
 		}
@@ -995,6 +1230,47 @@ bool CegoDistCursor::getTuple(ListT<CegoField>* flArray, int offset, int size)
     throw Exception(EXLOC, Chain("Cannot resolve object type"));
 }
 
+void CegoDistCursor::finishCaching()
+{
+    
+    if ( _useCache && _pCache && _pCacheList && _isCached == false && _pTC && _isFirst == false )
+    {	
+	CegoDataPointer dp;
+	
+	while ( _pTC->getNext(_cacheSchema, dp) && _pCacheList )
+	{
+
+	    ListT<CegoFieldValue> staticFieldList;
+	    CegoField* pF = _cacheSchema.First();
+	    while ( pF )
+	    {			
+		staticFieldList.Insert(pF->getValue().getLocalCopy());			       
+		_cacheEntrySize += pF->getValue().size();
+		pF = _cacheSchema.Next();
+	    }
+	    
+	    if ( _pCache->getMaxSize() > _cacheEntrySize )
+	    {
+		_pCacheList->Insert(staticFieldList);
+	    }
+	    else
+	    {
+		delete _pCacheList;
+		_pCacheList = 0;
+	    }
+	}
+       	    
+	if ( _pCacheList )
+	{
+	    // put table cache entry into cache
+	    _pCache->addEntry( _tabSetId, _tableName, _pCacheList );
+	    _isCached = true;
+	    delete _pCacheList;
+	    _pCacheList = 0;
+	}
+    }
+}
+
 void CegoDistCursor::reset()
 {
     _isFirst=true;
@@ -1010,6 +1286,26 @@ void CegoDistCursor::reset()
 	{
 	    if ( _pTC )
 		_pTC->abort();
+
+	    // we have to check, if cache was created
+	    if ( _isCached == false && _pCacheList )
+	    {
+		delete _pCacheList;
+		_pCacheList = 0;
+	    }
+	    
+	    if ( _pCacheArray && _pCache )
+	    {
+
+		// FOR DEBUGGING : sleep for a while
+		// cout << "Sleeping for 10 sec before cursor reset  ..." << endl;
+		// Sleeper s;
+		// s.secSleep(10);
+
+		_pCache->releaseEntry( _tabSetId, _tableName);
+		_pCacheArray = 0;
+		_isCached = 0;
+	    }
 	}
 	else
 	{
@@ -1029,6 +1325,8 @@ void CegoDistCursor::reset()
 	if ( _pC )
 	    _pC->reset();
     }
+
+    unuseCursorObject();
 }
 
 void CegoDistCursor::sysSetup()
@@ -1049,7 +1347,7 @@ void CegoDistCursor::sysSetup()
     }
     else  if ( _tableName == Chain(SYSTAB_INDEX_ID) )
     {
-	_pGTM->getDistObjectList(tableSet, CegoObject::INDEX, _sysObjList);
+	_pGTM->getDistObjectList(tableSet, CegoObject::AVLTREE, _sysObjList);
     }
     else  if ( _tableName == Chain(SYSTAB_BTREE_ID) )
     {
@@ -1067,7 +1365,7 @@ void CegoDistCursor::sysSetup()
 
 void CegoDistCursor::joinSetup(const CegoAttrCond& attrCond)
 {
-
+    
     CegoJoinObject *pJO = (CegoJoinObject*)_pCO;
         
     ListT<CegoField> outerSchema;
@@ -1080,7 +1378,7 @@ void CegoDistCursor::joinSetup(const CegoAttrCond& attrCond)
 	outerSchema = pJO->getLeftObject()->getSchema();
 	innerSchema = pJO->getRightObject()->getSchema(); 
 
-	_cursorCond = attrCond.getFilterCond(outerSchema);
+	_outerCond = attrCond.getFilterCond(outerSchema);
 	addInnerCond = attrCond.getFilterCond(innerSchema);
 
     }
@@ -1089,14 +1387,13 @@ void CegoDistCursor::joinSetup(const CegoAttrCond& attrCond)
 	innerSchema = pJO->getLeftObject()->getSchema();
 	outerSchema = pJO->getRightObject()->getSchema();
 
-	_cursorCond = attrCond.getFilterCond(outerSchema);
+	_outerCond = attrCond.getFilterCond(outerSchema);
 	addInnerCond = attrCond.getFilterCond(innerSchema);
     }
     
     CegoAttrCond ac;
-    CegoQueryHelper qh;
     
-    CegoQueryHelper::AttrCondMatch m  = qh.checkAttrCond(ac, pJO->getPredDesc(), innerSchema, &outerSchema, 1, 0);
+    CegoQueryHelper::AttrCondMatch m  = CegoQueryHelper::checkAttrCond(ac, pJO->getPredDesc(), innerSchema, &outerSchema, 1, 0);
     
     if ( m == CegoQueryHelper::COMPLETE )
     {
@@ -1110,22 +1407,11 @@ void CegoDistCursor::joinSetup(const CegoAttrCond& attrCond)
     }
 
     _isAttrCondValid = true;
-
-    if (  pJO->getJoinType() == CegoJoinObject::INNER || pJO->getJoinType() == CegoJoinObject::LEFTOUTER )
-    {
-	_pTCLeft->distSetup(_cursorCond);
-	_pTCRight->distSetup(_innerCond);
-    }
-    if ( pJO->getJoinType() == CegoJoinObject::RIGHTOUTER )
-    {
-	_pTCRight->distSetup(_cursorCond);
-	_pTCLeft->distSetup(_innerCond);
-    }
+    
 }
 
 Element* CegoDistCursor::getPlan()
 {
-    
     Element *pCursorPlan = new Element(XML_JOIN_ELEMENT);
 
     pCursorPlan->setAttribute(XML_TABLENAME_ATTR, _tableName);       
@@ -1134,7 +1420,18 @@ Element* CegoDistCursor::getPlan()
     if ( _pCO->getType() == CegoObject::VIEW )
     {
 	pCursorPlan->setAttribute(XML_TABLETYPE_ATTR, XML_VIEW_VALUE);
+
+#ifdef CGDEBUG
+	_pDBMng->log(_modId, Logger::DEBUG, Chain("Getting view plan for ") + _tableAlias);
+#endif
+
 	pCursorPlan->addContent( _pSelect->getPlan() );
+
+
+#ifdef CGDEBUG
+	_pDBMng->log(_modId, Logger::DEBUG, Chain("View plan for ") + _tableAlias + Chain(" retrieved"));
+#endif
+
     }
     else if ( _pCO->getType() == CegoObject::TABLE )
     {
@@ -1147,13 +1444,15 @@ Element* CegoDistCursor::getPlan()
 	}
 	else if( _idxMatch == CegoAttrCond::PART )
 	{
-	    pCursorPlan->setAttribute(XML_JOINSTRAT_ATTR, Chain("index support on ") + _cursorCond.toChain()); 
+	    pCursorPlan->setAttribute(XML_JOINSTRAT_ATTR, Chain("index support on ") + _cursorCond.toChain() + Chain(" using index ") + _pTC->getIndexName()); 
 	}
 	else
 	{
-	    pCursorPlan->setAttribute(XML_JOINSTRAT_ATTR, Chain("full table scan"));
-	}
-	
+	    if ( _cursorCond.numComp() > 0 )
+		pCursorPlan->setAttribute(XML_JOINSTRAT_ATTR, Chain("full table scan using condition ") + _cursorCond.toChain());
+	    else
+		pCursorPlan->setAttribute(XML_JOINSTRAT_ATTR, Chain("full table scan with no condition "));
+	}	
     }
     else if ( _pCO->getType() == CegoObject::JOIN )
     {	
@@ -1166,11 +1465,28 @@ Element* CegoDistCursor::getPlan()
 	    pCursorPlan->setAttribute(XML_TABLETYPE_ATTR, XML_LEFTOUTERJOIN_VALUE);
 	else if ( pJCO->getJoinType() == CegoJoinObject::RIGHTOUTER)	    
 	    pCursorPlan->setAttribute(XML_TABLETYPE_ATTR, XML_RIGHTOUTERJOIN_VALUE);
-
+	
+	if ( _pTCLeft )
+	{
+	    _pTCLeft->distSetup(_outerCond);
+	}
+	if ( _pTCRight )
+	{
+	    
+	    if ( _isAttrCondValid )
+	    {
+		_pTCRight->distSetup(_innerCond);
+	    }
+	    else
+	    {
+		_pTCRight->distSetup();
+	    }	    
+	}
+	
 	if ( _pTCLeft )
 	    pCursorPlan->addContent( _pTCLeft->getPlan() );
 	if ( _pTCRight )
-	    pCursorPlan->addContent( _pTCRight->getPlan() );	    
+	    pCursorPlan->addContent( _pTCRight->getPlan() );
     }
     else
     {
@@ -1179,5 +1495,4 @@ Element* CegoDistCursor::getPlan()
     }
 
     return pCursorPlan;
-
 }
