@@ -6,30 +6,15 @@
 //      
 // Design and Implementation by Bjoern Lemke
 //     
-// (C)opyright 2000-2010 Bjoern Lemke
-//
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
-// 
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; see the file COPYING.  If not, write to
-// the Free Software Foundation, 59 Temple Place - Suite 330,
-// Boston, MA 02111-1307, USA.
+// (C)opyright 2000-2016 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
 // Class: CegoJDBCInfo
 // 
-// Description: 
+// Description: JDBC request handling
 //
-// Status: QG-2.6
+// Status: CLEAN
 //
 ///////////////////////////////////////////////////////////////////////////////
 
@@ -49,6 +34,13 @@ CegoJDBCInfo::~CegoJDBCInfo()
 {
 }
 
+void CegoJDBCInfo::checkAccess() const
+{
+    if ( _pTabMng->getDBMng()->verifyJDBC(_pTabMng->getUser()) == false )
+    {
+	throw Exception(EXLOC, Chain("User not authorized for JDBC information"));	
+    }
+}
 
 void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 {
@@ -60,6 +52,7 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 
 	if ( *pReq == Chain("catalog") )
 	{
+
 	    ListT<CegoField> schema;
 	    schema.Insert( CegoField("JDBC", "JDBC", "TABLE_CAT", VARCHAR_TYPE, MAX_OBJNAME_LEN));
 	    
@@ -78,6 +71,8 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 	else if ( *pReq == Chain("schema") )
 	{
 	    
+	    checkAccess();
+
 	    ListT<Chain> tsList;
 	    _pTabMng->getDBMng()->getTableSetList(tsList);
 	    
@@ -125,6 +120,7 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 	}
 	else if ( *pReq == Chain("tab") )
 	{
+	    checkAccess();
 
 	    Chain *pTS = argList.Next();
 
@@ -204,6 +200,8 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 	else if ( *pReq == Chain("proc") )
 	{
 
+	    checkAccess();
+
 	    Chain *pTS = argList.Next();
 
 	    if ( pTS )
@@ -253,6 +251,8 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 	}
 	else if ( *pReq == Chain("pkey") )
 	{
+
+	    checkAccess();
 	    
 	    Chain *pTS = argList.Next();
 	    Chain *pTable = argList.Next();
@@ -264,8 +264,9 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 		ListT<CegoBTreeObject> btreeList;
 		ListT<CegoKeyObject> keyList;
 		ListT<CegoCheckObject> checkList;
-		
-		_pTabMng->getDistObjectByTableList(*pTS, *pTable, idxList, btreeList, keyList, checkList);
+		int numInvalid;
+
+		_pTabMng->getDistObjectByTableList(*pTS, *pTable, idxList, btreeList, keyList, checkList, numInvalid);
 		
 		ListT<CegoField> schema;
 		schema.Insert( CegoField("JDBC", "JDBC", "TABLE_CAT", VARCHAR_TYPE, MAX_OBJNAME_LEN));
@@ -285,10 +286,10 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 		while ( pIdx )
 		{
 		    
-		    if ( pIdx->getType() == CegoObject::PINDEX )
+		    if ( pIdx->getType() == CegoObject::PAVLTREE )
 		    {
 			CegoTableObject oe;
-			_pTabMng->getDistObject(*pTS, pIdx->getName(), CegoObject::INDEX, oe);
+			_pTabMng->getDistObject(*pTS, pIdx->getName(), CegoObject::AVLTREE, oe);
 			
 			int i=1;
 			CegoField *pF = oe.getSchema().First();
@@ -352,6 +353,8 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 	else if ( *pReq == Chain("ikey") )
 	{
 
+	    checkAccess();
+
 	    Chain *pTS = argList.Next();
 	    Chain *pTable = argList.Next();
 
@@ -362,8 +365,9 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 		ListT<CegoBTreeObject> btreeList;
 		ListT<CegoKeyObject> keyList;
 		ListT<CegoCheckObject> checkList;
-		    
-		_pTabMng->getDistObjectByTableList(*pTS, *pTable, idxList, btreeList, keyList, checkList);
+		int numInvalid;
+
+		_pTabMng->getDistObjectByTableList(*pTS, *pTable, idxList, btreeList, keyList, checkList, numInvalid);
 
 		ListT<CegoField> schema;
  		schema.Insert( CegoField("JDBC", "JDBC", "PKTABLE_CAT", VARCHAR_TYPE, MAX_OBJNAME_LEN));
@@ -446,6 +450,8 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 	else if ( *pReq == Chain("ekey") )
 	{
 
+	    checkAccess();
+
 	    Chain *pTS = argList.Next();
 	    Chain *pTable = argList.Next();
 
@@ -456,8 +462,9 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 		ListT<CegoBTreeObject> btreeList;
 		ListT<CegoKeyObject> keyList;
 		ListT<CegoCheckObject> checkList;
-		    
-		_pTabMng->getDistObjectByTableList(*pTS, *pTable, idxList, btreeList, keyList, checkList);
+		int numInvalid;
+
+		_pTabMng->getDistObjectByTableList(*pTS, *pTable, idxList, btreeList, keyList, checkList, numInvalid);
 
 		ListT<CegoField> schema;
 
@@ -541,6 +548,8 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 	else if ( *pReq == Chain("idx") || *pReq == Chain("uidx") )
 	{
 
+	    checkAccess();
+
 	    Chain *pTS = argList.Next();
 	    Chain *pTable = argList.Next();
 
@@ -551,8 +560,9 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 		ListT<CegoBTreeObject> btreeList;
 		ListT<CegoKeyObject> keyList;
 		ListT<CegoCheckObject> checkList;
-		    
-		_pTabMng->getDistObjectByTableList(*pTS, *pTable, idxList, btreeList, keyList, checkList);
+		int numInvalid;
+
+		_pTabMng->getDistObjectByTableList(*pTS, *pTable, idxList, btreeList, keyList, checkList, numInvalid);
 
 		ListT<CegoField> schema;
 
@@ -585,7 +595,7 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 		{
 		    
 		    CegoTableObject io;
-		    _pTabMng->getDistObject(*pTS, pIdx->getName(), CegoObject::INDEX, io);
+		    _pTabMng->getDistObject(*pTS, pIdx->getName(), CegoObject::AVLTREE, io);
 		    
 		    int i=1;
 		    CegoField *pIF = io.getSchema().First();
@@ -596,7 +606,7 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 			f1.Insert( CegoFieldValue(VARCHAR_TYPE, *pTS));
 			f1.Insert( CegoFieldValue(VARCHAR_TYPE, io.getTabName()));
 			
-			if ( pIdx->getType() == CegoObject::INDEX )
+			if ( pIdx->getType() == CegoObject::AVLTREE )
 			    f1.Insert( CegoFieldValue(BOOL_TYPE, Chain("true"))); // index is non-unique
 			else
 			    f1.Insert( CegoFieldValue(BOOL_TYPE, Chain("false"))); // pindex and uindex are unique
@@ -675,10 +685,11 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 	else if ( *pReq == Chain("colinfo") )
 	{
 
+	    checkAccess();
+
 	    Chain *pTS = argList.Next();
 	    Chain *pTable = argList.Next();
 
-
 	    if ( pTS && pTable )
 	    {
 
