@@ -6,49 +6,34 @@
 //                                                         
 // Design and Implementation by Bjoern Lemke               
 //                                                         
-// (C)opyright 2000-2010 Bjoern Lemke                        
-//
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
-// 
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; see the file COPYING.  If not, write to
-// the Free Software Foundation, 59 Temple Place - Suite 330,
-// Boston, MA 02111-1307, USA.
+// (C)opyright 2000-2016 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
 // Class: CegoLogManager
-// 
-// Description: Cego database logging
 //
-// Status: QG-2.6
+// Description: Cego database log management
+//
+// Status: CLEAN
 //
 ///////////////////////////////////////////////////////////////////////////////
 
-// base includes
+// LFC INCLUDES
 #include <lfcbase/Exception.h>
 #include <lfcbase/File.h>
-#include <lfcbase/Host.h>
 #include <lfcbase/Net.h>
 #include <lfcbase/Datetime.h>
 
-// cego includes
+// CEGO INCLUDES
 #include "CegoLogManager.h"
 #include "CegoXMLdef.h"
 #include "CegoDefs.h"
 
+// POSIX INCLUDES
 #include <string.h>
 #include <stdlib.h>
 
-CegoLogManager::CegoLogManager(const Chain& xmlDef, const Chain& logFile) : CegoFileHandler(logFile), CegoXMLSpace(xmlDef)
+CegoLogManager::CegoLogManager(const Chain& xmlDef, const Chain& logFile, const Chain& progName) : CegoFileHandler(logFile, progName), CegoXMLSpace(xmlDef)
 {
     for (int i=0;i< TABMNG_MAXTABSET;i++)
     {
@@ -73,17 +58,17 @@ CegoLogManager::~CegoLogManager()
     }
 }
 
-void CegoLogManager::setLSN(int tabSetId, long lsn)
+void CegoLogManager::setLSN(int tabSetId, unsigned long long lsn)
 {
     _lsn[tabSetId] = lsn;
 }
 
-long CegoLogManager::getLSN(int tabSetId)
+unsigned long long CegoLogManager::getLSN(int tabSetId)
 {
     return _lsn[tabSetId];
 }
 
-Chain CegoLogManager::getArchiveLogName(const Chain& tableSet, long lsn)
+Chain CegoLogManager::getArchiveLogName(const Chain& tableSet, unsigned long long lsn)
 {
     Chain lsnStr = Chain("000000000000") + Chain(lsn);
     Chain lsnFix = lsnStr.subChain(lsnStr.length() - 12, lsnStr.length());
@@ -113,7 +98,6 @@ void CegoLogManager::setLogFile(int tabSetId, const Chain& logFile, bool readOnl
 
     _logPos[tabSetId] = sizeof(int);
     _logActive[tabSetId] = false;
-
 }
 
 void CegoLogManager::allocateLogConnection(int tabSetId, const Chain& tableSet, const Chain& logHost, int logPort)
@@ -168,7 +152,6 @@ void CegoLogManager::releaseLogConnection(int tabSetId)
     }
 }
 
-
 void CegoLogManager::initLog(int tabSetId, int size)
 {
     
@@ -196,9 +179,7 @@ void CegoLogManager::initLog(int tabSetId, int size)
 
     _pLog[tabSetId]->close();
     delete _pLog[tabSetId];
-    _pLog[tabSetId]=0;
-
-    
+    _pLog[tabSetId]=0;    
 }
 
 int CegoLogManager::getLogOffset(int tabSetId)
@@ -215,6 +196,13 @@ void CegoLogManager::resetLog(int tabSetId)
 
 void CegoLogManager::stopLog(int tabSetId)
 {
+    if (_pLog[tabSetId])
+    {
+	_pLog[tabSetId]->close();
+	delete _pLog[tabSetId];
+	_pLog[tabSetId] = 0;
+    }
+
     _logActive[tabSetId] = false;
 }
 
@@ -247,6 +235,8 @@ CegoLogManager::LogResult CegoLogManager::logAction(int tabSetId, CegoLogRecord&
     char* pBuf = (char*)malloc(len);
         
     logRec.encode(pBuf);
+
+    bool isFull = false;
     
     if ( _pLogHandler[tabSetId] )
     {
@@ -268,8 +258,8 @@ CegoLogManager::LogResult CegoLogManager::logAction(int tabSetId, CegoLogRecord&
 
 	if ( _logOffset[tabSetId] + len > _logSize[tabSetId] )
 	{
-	    free (pBuf);
-	    return LOG_FULL;
+	    // is log is full, we anyway write the log entry to complete log for checkpoint
+	    isFull=true;
 	}
         
 	_pLog[tabSetId]->writeByte((char*)&len, sizeof(int));       
@@ -284,10 +274,13 @@ CegoLogManager::LogResult CegoLogManager::logAction(int tabSetId, CegoLogRecord&
     _lsn[tabSetId]++;
 
     free(pBuf);
-    return LOG_SUCCESS;
+    if ( isFull )
+	return LOG_FULL;
+    else
+	return LOG_SUCCESS;
 }
 
-long CegoLogManager::getMinLSN(int tabSetId)
+unsigned long long CegoLogManager::getMinLSN(int tabSetId)
 {
 
     if (_pLog[tabSetId] == 0)
@@ -301,7 +294,7 @@ long CegoLogManager::getMinLSN(int tabSetId)
     
     _logPos[tabSetId] = sizeof(int);
 
-    long minlsn=0;
+    unsigned long long minlsn=0;
 
     if ( _logPos[tabSetId] < _logOffset[tabSetId] )
     {
@@ -330,7 +323,7 @@ long CegoLogManager::getMinLSN(int tabSetId)
 }
 
 
-long CegoLogManager::getMaxLSN(int tabSetId)
+unsigned long long CegoLogManager::getMaxLSN(int tabSetId)
 {
 
     if (_pLog[tabSetId] == 0)
@@ -348,7 +341,7 @@ long CegoLogManager::getMaxLSN(int tabSetId)
         
     _logPos[tabSetId] = sizeof(int);
 
-    long maxlsn=0;
+    unsigned long long maxlsn=0;
 
     while (_logPos[tabSetId] < _logOffset[tabSetId])
     {
@@ -389,8 +382,7 @@ void CegoLogManager::seekToStart(int tabSetId)
     _logPos[tabSetId] = sizeof(int);
 }
 
-
-bool CegoLogManager::logSeek(int tabSetId, long lsn)
+bool CegoLogManager::logSeek(int tabSetId, unsigned long long lsn)
 {
     if (_pLog[tabSetId] == 0)
     {
@@ -403,7 +395,7 @@ bool CegoLogManager::logSeek(int tabSetId, long lsn)
     
     _logPos[tabSetId] = sizeof(int);
 
-    long loglsn=0;
+    unsigned long long loglsn=0;
 
     while (_logPos[tabSetId] < _logOffset[tabSetId])
     {
@@ -450,7 +442,6 @@ bool CegoLogManager::logSeek(int tabSetId, long lsn)
     {
 	Chain msg = "LSN=" + Chain(lsn) + " too high, log delta is missing";
 	throw Exception(EXLOC, msg);	
-	
     }
 }
 
@@ -479,13 +470,10 @@ bool CegoLogManager::logRead(int tabSetId, CegoLogRecord& logRec)
 	return true;
     }
     return false;
-
 }
 
-
 bool CegoLogManager::switchLogFile(int tabSetId)
 {    
-
     if ( _pLogHandler[tabSetId] == 0 )
     {
 
@@ -528,8 +516,7 @@ bool CegoLogManager::switchLogFile(int tabSetId)
 		
 		
 		setLogFileStatus(tableSet, *pNextLogFile, XML_ACTIVE_VALUE);
-		
-		
+				
 		log(_modId, Logger::NOTICE, Chain("Logfile switch to logfile ") + *pNextLogFile + Chain(" for tableSet ") +  tableSet);
 		
 		setLogFile(tabSetId, *pNextLogFile, false);
@@ -544,8 +531,7 @@ bool CegoLogManager::switchLogFile(int tabSetId)
 	}
 	doc2Xml();
 	resetLog(tabSetId);
-	startLog(tabSetId);
-	
+	startLog(tabSetId);	
     }
 
     CegoLogRecord lr;
@@ -557,7 +543,6 @@ bool CegoLogManager::switchLogFile(int tabSetId)
 
 void CegoLogManager::setActiveLogFile(const Chain& tableSet)
 {
-
     int tabSetId = getTabSetId(tableSet);
 
     ListT<Chain> lfList;
@@ -570,8 +555,7 @@ void CegoLogManager::setActiveLogFile(const Chain& tableSet)
     Chain *pStatus = statusList.First();
     
     while ( pLogFile && pStatus ) 
-    {
-	
+    {	
 	if ( *pStatus == Chain(XML_ACTIVE_VALUE))
 	{
 	    log(_modId, Logger::NOTICE, Chain("Setting active logfile to ") + Chain(*pLogFile) + Chain(" ..."));	
