@@ -6,36 +6,24 @@
 //
 // Design and Implementation by Bjoern Lemke               
 //                                                         
-// (C)opyright 2000-2010 Bjoern Lemke                        
-//
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
-// 
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; see the file COPYING.  If not, write to
-// the Free Software Foundation, 59 Temple Place - Suite 330,
-// Boston, MA 02111-1307, USA.
+// (C)opyright 2000-2016 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
-// Class: Main
+// Class: main
 // 
 // Description: Main module for the cego database program. The program can be invoked in different modes.
 //
-// Status: QG-2.6
+// Status: CLEAN
 //
 ///////////////////////////////////////////////////////////////////////////////
 
+// POSIX INCLUDES
 #include <locale.h>
 #include <stdlib.h>
+#include <unistd.h>
 
+// LFC INCLUDES
 #include <lfcbase/Sleeper.h>
 #include <lfcbase/Process.h>
 #include <lfcbase/GetLongOpt.h>
@@ -43,16 +31,17 @@
 #include <lfcbase/Timer.h>
 #include <lfcbase/Tokenizer.h>
 #include <lfcbase/Exception.h>
-#include <lfcbase/Host.h>
 #include <lfcbase/AESCrypt.h>
 #include <lfcbase/SigHandler.h>
-
+#include <lfcbase/Host.h>
+#include <lfcbase/ThreadLock.h>
 #include <lfcxml/XMLSuite.h>
 
+
+// CEGO INCLUDES
 #include "CegoDbThreadPool.h"
 #include "CegoLogThreadPool.h"
 #include "CegoAdminThreadPool.h"
-
 #include "CegoDatabaseManager.h"
 #include "CegoDistManager.h"
 #include "CegoAction.h"
@@ -68,51 +57,54 @@
 #define USAGE "Usage:\n\n\
   Info:               cego [ --version ] [ --help ]\n\
 \n\
-  Initialization:     cego --mode=init --dbxml=<db xml> --dbname=<dbname>\n\
-                           [ --hostname=<hostname> ] [ --pgsize=<page size> ]\n\
-                           [ --dbport=<data port> ] [ --admport=<admin port> ] [ --logport=<log port ]\n\
-                           [ --pidfile=<pid file> ] [ --loglevel=<log level> ]\n\
+  Initialization:     cego --mode=init --dbxml=<filename> --dbname=<id>\n\
+                           [ --hostname=<id> ] [ --pgsize=<int> ]\n\
+                           [ --dbport=<int> ] [ --admport=<int> ] [ --logport=<int> ]\n\
+                           [ --pidfile=<filename> ] [ --csmode={ID|STR|NONE} ] [ --qescmode={ON|OFF} ] [ --loglevel=<log level> ]\n\
 \n\
-  User setup:         cego --mode=adduser --user=<user>/<password>\n\
-                           --role=<role> --dbxml=<db xml>\n\
+  User setup:         cego --mode=adduser --user=<string>/<string>\n\
+                           --role=<string> --dbxml=<filename>\n\
 \n\
-  Role setup:         cego --mode=addrole --role=<role> --dbxml=<db xml>\n\
+  Role setup:         cego --mode=addrole --role=<string> --dbxml=<filename>\n\
 \n\
-  Permission setup:   cego --mode=addperm --role=<role> --permid=<permid>\n\
-                           --tableset=<tableset> --filter=<object filter>\n\
-                           --perm=<permission> --dbxml=<db xml>\n\
+  Permission setup:   cego --mode=addperm --role=<string> --permid=<string>\n\
+                           --tableset=<string> --filter=<string>\n\
+                           --perm=<string> --dbxml=<db xml>\n\
 \n\
-  Define Mode:        cego --mode=define --tsdef=<db attr string> --tableset=<tableset>\n\
-                           --dbxml=<db xml>\n\
+  Define Mode:        cego --mode=define --tsdef=<attr string> --tableset=<string>\n\
+                           --dbxml=<filename>\n\
 \n\
-  Creation Mode:      cego --mode=create --tableset=<tableset> --dbxml=<db xml>\n\
+  Creation Mode:      cego --mode=create --tableset=<string> --dbxml=<filename>\n\
 \n\
-  Export Mode:        cego --mode=<xmlexport|binexport|plainexport> --expfile=<exportfile>\n\
-                           --tableset=<tableset> --dbxml=<db xml> --user=<user/pwd>\n\
+  Export Mode:        cego --mode={xmlexport|binexport|plainexport} --expfile=<filename>\n\
+                           --tableset=<string> --dbxml=<filename> --user=<string>/<string>\n\
 \n\
-  Import Mode:        cego --mode=<xmlimport|binimport|plainimport> --impfile=<importfile>\n\
-                            --tableset=<tableset> --dbxml=<db xml> --user=<user/pwd> [ --noLogging ]\n\
+  Import Mode:        cego --mode={xmlimport|binimport|plainimport} --impfile=<filename>\n\
+                            --tableset=<id> --dbxml=<filename> --user=<string>/<string> [ --noLogging ]\n\
 \n\
-  Batch Mode:         cego --mode=batch --batchfile=<batchfile> --tableset=<tableset>\n\
-                           --dbxml=<db xml> --user=<user/pwd> [ --poolsize=<poolsize> ] [ --ignore ]\n\
+  Batch Mode:         cego --mode=batch --batchfile=<string> --tableset=<string> --dbxml=<filename> --user=<string>/<string>\n\
+                           [ --poolsize=[<numseg>x]<segsize> ] [ --ignore ]\n\
 \n\
-  Daemon Mode:        cego --mode=daemon --dbxml=<db xml> [--tableset=<tableset>,<tableset>,... ]\n\
-                           [ --cleanup ] [ --forceload ] [ --nolockstat ]\n\
-                           [ --numdbthread=<numdb> ] [ --numadminthread=<numadm> ] [ --numlogthread=<numlog> ]\n\
-                           [ --poolsize=<poolsize> ]  [ --pidfile=<pidfile> ] [ --protocol={serial|xml}]\n\
-                           [ --master=<host:port:user:passwd> ]\n\
+  Daemon Mode:        cego --mode=[daemon|server] --dbxml=<filename> [--tableset=<string>,<string>,... ]\n\
+                           [ --cleanup ] [ --forceload ] [ --lockstat ]\n\
+                           [ --numdbthread=<int> ] [ --numadminthread=<int> ] [ --numlogthread=<int> ]\n\
+                           [ --poolsize=[<numseg>x]<segsize> ] [ --pidfile=<filename> ] [ --protocol={serial|xml}]\n\
+                           [ --master=<host>:<port>:<user>:<passwd> ]\n\
 \n\
-  General Options:    [ --logfile=<logfile> ] [ --lockfile=<lockfile> ]\n"
+  General Options:    [ --logfile=<filename> ] [--syslog=yes|no ] [ --lockfile=<filename> ]\n"
 
 // DEFINES
+#define SYSLOGPROGNAME "cego"
 #define DEFAULTLOGFILE "cego.log"
-#ifdef HAVE_MINGW32
+#define DEFAULTSYSLOG "yes"
+#ifdef HAVE_MINGW
 #define DEFAULTLOCKFILE "c:/windows/temp/cego.lock"
 #else
 #define DEFAULTLOCKFILE "/tmp/cego.lock"
 #endif
-#define DEFAULTPROTOCOL "xml"
-#define DEFAULTPOOLSIZE 1000
+#define DEFAULTPROTOCOL "serial"
+#define DEFAULTPOOLSIZE "3x1000"
+
 #define DEFAULTDBTHREADNUM 10
 #define DEFAULTADMINTHREADNUM 5
 #define DEFAULTLOGTHREADNUM 3
@@ -122,38 +114,51 @@
 #define DEFAULTROLE XML_ROLE_ALL_VALUE
 #define DEFAULTTMPSIZE 100
 #define DEFAULTSYSSIZE 100
+#define DEFAULTAPPSIZE 3000
 #define DEFAULTLOGNUM 3
 #define DEFAULTLOGSIZE 1000000
 #define DEFAULTORDERSIZE 10000000
 #define DEFAULTTSROOT "./"
-#define DEFAULTPRIMARY "localhost"
-#define DEFAULTSECONDARY "localhost"
-#define DEFAULTTSTICKET "tsticket.xml"
-#define DEFAULTAPPFILENAME "./data01.dbf"
-#define DEFAULTAPPFILESIZE 1000
 
 #define DEFAULTHOST "localhost"
 #define DEFAULTPIDFILE "./pid"
-#define DEFAULTPAGESIZE 32768
+#define DEFAULTPAGESIZE 8192
 #define DEFAULTDBPORT 2200
 #define DEFAULTADMPORT 2000
 #define DEFAULTLOGPORT 3000
+#define DEFAULTCSMODE "ID"
+#define DEFAULTQESCMODE "ON"
+
 #define DEFAULTLOGLEVEL "NOTICE"
 
 #define MAXCMDLEN 10000
 
 #define LANGUAGE_ENV "LANG"
-#define LOCALE_CATEGORY LC_TIME
+#define LOCALE_CATEGORY LC_ALL
 
+extern unsigned _funcSeed;
 extern char __lfcVersionString[];
 extern char __lfcxmlVersionString[];
 extern char __caseSensitiveFlag;
 extern char __quoteEscapeFlag;
-extern Chain __dateFormatString;
+extern char __allowDuplicateNull;
+
+extern char __currencySymbol;
+extern char __decimalPoint;
+
+// how to format date on default
+extern Chain __dateTimeFormat;
+// valid date time scan formats
+
+// since the date format list is accessed by several threads, we have to provide a mutex lock for it
+extern ListT<Chain> __dateFormatList;
+extern ThreadLock __dateFormatLock;
+
 extern bool __lockStatOn;
 
 enum XPortFormat { BINARY, XML, PLAIN };
 
+void readParameter(CegoDatabaseManager *pDBMng);
 int init(GetLongOpt& longOpt);
 int addUser(GetLongOpt& longOpt);
 int addRole(GetLongOpt& longOpt);
@@ -165,10 +170,10 @@ int exportTableSet(GetLongOpt& longOpt, XPortFormat format);
 int runBatch(GetLongOpt& longOpt);
 int runDaemon(GetLongOpt& longOpt);
 bool processBatchFile(CegoDatabaseManager *pDBMng, CegoAction *pAction, const Chain& batchFileName, bool ignoreError, bool consoleOut, Chain& errorMsg);
+bool getPoolInfo(const Chain& poolSize, int& numSeg, int& segSize);
 
 int main(int argc, char **argv)
 {	
-
     GetLongOpt longOpt(argc, argv);
     
     longOpt.addOpt("version");
@@ -185,11 +190,13 @@ int main(int argc, char **argv)
     longOpt.addOpt("admport", Chain(DEFAULTADMPORT));
     longOpt.addOpt("logport", Chain(DEFAULTLOGPORT));
     longOpt.addOpt("loglevel", Chain(DEFAULTLOGLEVEL));
+    longOpt.addOpt("csmode", Chain(DEFAULTCSMODE));
+    longOpt.addOpt("qescmode", Chain(DEFAULTQESCMODE));
 
     longOpt.addOpt("tableset");
     longOpt.addOpt("cleanup");
     longOpt.addOpt("forceload");
-    longOpt.addOpt("nolockstat");
+    longOpt.addOpt("lockstat");
     longOpt.addOpt("role", DEFAULTROLE);
     longOpt.addOpt("filter");
     longOpt.addOpt("perm");
@@ -201,12 +208,12 @@ int main(int argc, char **argv)
     longOpt.addOpt("batchfile");
     longOpt.addOpt("poolsize", Chain(DEFAULTPOOLSIZE));
     longOpt.addOpt("ignore");
-    longOpt.addOpt("daemon");
     longOpt.addOpt("numdbthread", Chain(DEFAULTDBTHREADNUM));
     longOpt.addOpt("numadminthread", Chain(DEFAULTADMINTHREADNUM));
     longOpt.addOpt("numlogthread", Chain(DEFAULTLOGTHREADNUM));
     longOpt.addOpt("master");
     longOpt.addOpt("logfile", DEFAULTLOGFILE);
+    longOpt.addOpt("syslog", DEFAULTSYSLOG);
     longOpt.addOpt("lockfile", DEFAULTLOCKFILE);
     longOpt.addOpt("pidfile");
     longOpt.addOpt("protocol", DEFAULTPROTOCOL);
@@ -228,16 +235,24 @@ int main(int argc, char **argv)
     char* lang = 0;
     if ( ( lang = getenv(LANGUAGE_ENV) ) != 0)
     {
-	if ( setlocale(LOCALE_CATEGORY, lang) == 0)
+	// if ( setlocale(LOCALE_CATEGORY, lang) == 0)
+	if ( setlocale(LC_ALL, lang) == 0)
 	{
 	    Chain msg = Chain("Cannot set locale ") + Chain(lang);
 	    cerr << msg << endl;
 	    exit(1);
 	}
     }
+
+    const struct lconv * const currentlocale = localeconv();
+
+    __currencySymbol = *currentlocale->currency_symbol;
+    __decimalPoint = *currentlocale->decimal_point;        
+
+    // cout << "Decimal Point = " << __decimalPoint << endl;
+    
     // for random generation in CegoFunction
-    int seed = 5 * (time(NULL) % 100000); 	
-    srand( seed  ); 
+    _funcSeed = 5 * (time(NULL) % 100000); 	
 
     if ( longOpt.isSet( Chain("help") ) )
     {	
@@ -247,15 +262,49 @@ int main(int argc, char **argv)
 
     if ( longOpt.isSet( Chain("version") ) )
     {
-	cout << CEGO_PRODUCT << " (" << sizeof(long) * 8 << " bit), Version " << CEGO_VERSION 
-	     << " [ lfc : " << __lfcVersionString  
-	     << ", lfcxml : " <<  __lfcxmlVersionString << " ]" << endl;
+	cout << CEGO_PRODUCT << " Server Program (" << sizeof(void*) * 8 << " bit), Version " << CEGO_VERSION 
+	     << " [ lfcbase: " << __lfcVersionString  
+	     << ", lfcxml: " <<  __lfcxmlVersionString << " ]" << endl;
 	cout << CEGO_COPYRIGHT << endl;
 	exit(0);
     }
 
-    if ( longOpt.isSet("nolockstat") )
-	__lockStatOn = false;
+    if ( longOpt.isSet("lockstat") )
+	__lockStatOn = true;
+
+    __dateFormatLock.init(LCKMNG_LOCKWAITDELAY, __lockStatOn);
+
+    // parameter validation
+
+    Chain poolSize = longOpt.getOptValue("poolsize");
+    int numSeg, segSize;
+    if ( getPoolInfo(poolSize, numSeg, segSize) == false )
+    {
+	cerr << "Invalid value for poolsize : " <<  poolSize << endl;
+	exit(1);
+    }
+
+    Chain numDbThread = longOpt.getOptValue("numdbthread");
+    if ( numDbThread.asInteger() <= 0 )
+    {
+	cerr << "Invalid value for numdbthread : " << numDbThread << endl;
+	exit(1);
+    }
+
+    Chain numAdminThread = longOpt.getOptValue("numadminthread");
+    if ( numAdminThread.asInteger() <= 0 )
+    {
+	cerr << "Invalid value for numadminthread : " << numAdminThread << endl;
+	exit(1);
+    }
+
+    Chain numLogThread = longOpt.getOptValue("numlogthread");
+    if ( numLogThread.asInteger() <= 0 )
+    {
+	cerr << "Invalid value for numlogthread : " << numLogThread << endl;
+	exit(1);
+    }
+
     
     Chain mode = longOpt.getOptValue("mode");
     
@@ -315,29 +364,86 @@ int main(int argc, char **argv)
     }
     else if ( mode == Chain("daemon") )
     {
+
+#ifdef HAVE_MINGW
+	cerr << "Daemon mode for windows not supported" << endl;
+	exit(1);	
+#else	 
+	pid_t pid;
+	
+	pid = fork();  
+	if ( pid == -1 )
+	{
+	    cerr << "Cannot fork process" << endl;
+	    exit(1);
+	}
+	else if ( pid != 0 )  
+	{
+	    // parent process exit
+	    exit (0);  
+	}
+
+	if ( setsid() == -1 )  
+	{
+	    cerr << "Cannot set process group" << endl;
+	    exit(1);
+	}
+      
+	exitCode = runDaemon(longOpt);
+#endif
+    }
+    else if ( mode == Chain("server") )
+    {
+	// don't run as background daemon
 	exitCode = runDaemon(longOpt);
     }
     else
     {
 	cerr << USAGE << endl;
-	exit(0);	
-    }
-    
+	exit(1);	
+    }    
     exit(exitCode);
 }
 
+void readParameter(CegoDatabaseManager *pDBMng)
+{
+    __caseSensitiveFlag = pDBMng->getCaseSensitiveFlag();
+
+    if ( pDBMng->allowDuplicateNull() )
+	__allowDuplicateNull = 1;
+    else
+	__allowDuplicateNull = 0;
+
+    if ( pDBMng->isQuoteEscapeMode() )	       
+	__quoteEscapeFlag = 1;
+    else
+	__quoteEscapeFlag = 0;
+    
+    __dateTimeFormat = pDBMng->getDateTimeFormat();
+    if ( __dateTimeFormat == Chain("") )
+	__dateTimeFormat = Chain(DEFAULTDATETIMEFORMAT);
+    
+    __dateFormatList = pDBMng->getDateFormatList();	
+    if ( __dateFormatList.Size() == 0 )
+    {
+	__dateFormatList.Insert(DEFAULTDATETIMEFORMAT);
+	__dateFormatList.Insert(DEFAULTDATEFORMAT);
+    }
+
+}
 
 int init(GetLongOpt& longOpt)
 {
-    
+
     int exitCode = 0;
 
     CegoDatabaseManager* pDBMng = 0;
 
     try {
 	
-	Chain logFile = longOpt.getOptValue("logfile");	
-	Chain lockFile = longOpt.getOptValue("lockfile");	
+	Chain logFile = longOpt.getOptValue("logfile");
+	Chain lockFile = longOpt.getOptValue("lockfile");
+	Chain syslog = longOpt.getOptValue("syslog");	
 	Chain dbXML = longOpt.getOptValue("dbxml");
 	Chain dbName = longOpt.getOptValue("dbname");
 
@@ -347,6 +453,19 @@ int init(GetLongOpt& longOpt)
 	Chain dbport = longOpt.getOptValue("dbport");
 	Chain admport = longOpt.getOptValue("admport");
 	Chain logport = longOpt.getOptValue("logport");
+
+	Chain csmode = longOpt.getOptValue("csmode");
+	if ( csmode != Chain(XML_ID_VALUE) && csmode != Chain(XML_STR_VALUE) && csmode != Chain(XML_NONE_VALUE))
+	{
+	    throw Exception(EXLOC, "Invalid value for csmode ( must be ID, STR or NONE)");
+	}
+
+	Chain qescmode = longOpt.getOptValue("qescmode");
+	if ( qescmode != Chain(XML_ON_VALUE) && qescmode != Chain(XML_OFF_VALUE) )
+	{
+	    throw Exception(EXLOC, "Invalid value for qescmode ( must be either ON or OFF)");
+	}
+
 	Chain logLevel = longOpt.getOptValue("loglevel");
 
 
@@ -368,9 +487,15 @@ int init(GetLongOpt& longOpt)
 	    throw Exception(EXLOC, "Invalid log level");
 	}
 
-	pDBMng = new CegoDatabaseManager(dbXML, lockFile, logFile);
-	
-	pDBMng->initXml(dbName, pgsize.asInteger(), hostname, dbport.asInteger(), admport.asInteger(), logport.asInteger(), pidfile, logLevel);
+	Chain progName = SYSLOGPROGNAME;
+	if ( syslog == Chain("no") )
+	    progName=Chain("");
+
+	pDBMng = new CegoDatabaseManager(dbXML, lockFile, logFile, progName);
+
+
+	pDBMng->initXml(dbName, pgsize.asInteger(), hostname, dbport.asInteger(), admport.asInteger(), logport.asInteger(), pidfile, logLevel,
+			csmode, qescmode == Chain(XML_ON_VALUE) );
 	
 	delete pDBMng;
     }
@@ -388,7 +513,6 @@ int init(GetLongOpt& longOpt)
     return exitCode;
 }
 
-
 int addUser(GetLongOpt& longOpt)
 {
     
@@ -398,6 +522,7 @@ int addUser(GetLongOpt& longOpt)
 
     try {
 	
+	Chain syslog = longOpt.getOptValue("syslog");	
 	Chain logFile = longOpt.getOptValue("logfile");	
 	Chain lockFile = longOpt.getOptValue("lockfile");	
 	Chain dbXML = longOpt.getOptValue("dbxml");
@@ -424,8 +549,12 @@ int addUser(GetLongOpt& longOpt)
 	{
 	    throw Exception(EXLOC, "No password set");
 	}
+
+	Chain progName = SYSLOGPROGNAME;
+	if ( syslog == Chain("no") )
+	    progName=Chain("");
 	
-	pDBMng = new CegoDatabaseManager(dbXML, lockFile, logFile);
+	pDBMng = new CegoDatabaseManager(dbXML, lockFile, logFile, progName);
 	
 	pDBMng->xml2Doc();
 	pDBMng->configureLogger();
@@ -460,6 +589,7 @@ int addRole(GetLongOpt& longOpt)
 
     try {
 
+	Chain syslog = longOpt.getOptValue("syslog");
 	Chain logFile = longOpt.getOptValue("logfile");
 	Chain lockFile = longOpt.getOptValue("lockfile");
 	Chain dbXML = longOpt.getOptValue("dbxml");
@@ -473,8 +603,13 @@ int addRole(GetLongOpt& longOpt)
 	{
 	    throw Exception(EXLOC, "No dbxml set");
 	}
+
+	Chain progName = SYSLOGPROGNAME;
+	if ( syslog == Chain("no") )
+	    progName=Chain("");
+	
+	pDBMng = new CegoDatabaseManager(dbXML, lockFile, logFile, progName);
 	
-	pDBMng = new CegoDatabaseManager(dbXML, lockFile, logFile);
 	
 	pDBMng->xml2Doc();
 	pDBMng->configureLogger();
@@ -508,6 +643,7 @@ int addPerm(GetLongOpt& longOpt)
 
     try {
 
+	Chain syslog = longOpt.getOptValue("syslog");
 	Chain logFile = longOpt.getOptValue("logfile");
 	Chain lockFile = longOpt.getOptValue("lockfile");
 	Chain dbXML = longOpt.getOptValue("dbxml");
@@ -523,8 +659,12 @@ int addPerm(GetLongOpt& longOpt)
 	{
 	    throw Exception(EXLOC, "No dbxml set");
 	}
+
+	Chain progName = SYSLOGPROGNAME;
+	if ( syslog == Chain("no") )
+	    progName=Chain("");
 	
-	pDBMng = new CegoDatabaseManager(dbXML, lockFile, logFile);
+	pDBMng = new CegoDatabaseManager(dbXML, lockFile, logFile, progName);       
 	
 	pDBMng->xml2Doc();
 	pDBMng->configureLogger();
@@ -550,7 +690,6 @@ int addPerm(GetLongOpt& longOpt)
     return exitCode;
 }
 
-
 int defineTableSet(GetLongOpt& longOpt)
 {
 
@@ -560,6 +699,7 @@ int defineTableSet(GetLongOpt& longOpt)
 
     try {
 	
+	Chain syslog = longOpt.getOptValue("syslog");
 	Chain logFile = longOpt.getOptValue("logfile");
 	Chain lockFile = longOpt.getOptValue("lockfile"); 
 	Chain dbXML = longOpt.getOptValue("dbxml");
@@ -578,22 +718,25 @@ int defineTableSet(GetLongOpt& longOpt)
 	{
 	    throw Exception(EXLOC, "No tsdef set");
 	}
-	       
-	pDBMng = new CegoDatabaseManager(dbXML, lockFile, logFile);
+
+	Chain progName = SYSLOGPROGNAME;
+	if ( syslog == Chain("no") )
+	    progName=Chain("");
 	
+	pDBMng = new CegoDatabaseManager(dbXML, lockFile, logFile, progName);
+	       
 	pDBMng->xml2Doc();
 	pDBMng->configureLogger();
 	
 	Chain tsRoot(DEFAULTTSROOT);
-	Chain tsTicket(DEFAULTTSTICKET);
 	
 	int sysSize = DEFAULTSYSSIZE;
 	int tmpSize = DEFAULTTMPSIZE;
+	int appSize = DEFAULTAPPSIZE;
+
 	int logFileSize = DEFAULTLOGSIZE;
 	int logFileNum = DEFAULTLOGNUM;
 	
-	Chain appFileName = DEFAULTAPPFILENAME;
-	int appFileSize = DEFAULTAPPFILESIZE;
 	long orderSize = DEFAULTORDERSIZE;
 
 	Tokenizer atok(tsDef, Chain(","));
@@ -606,48 +749,47 @@ int defineTableSet(GetLongOpt& longOpt)
 	    Chain value;
 	    vtok.nextToken(attr);
 	    vtok.nextToken(value);
+
 	    if ( attr == Chain("syssize"))
 		sysSize = value.asInteger();
 	    else if ( attr == Chain("tmpsize"))
 		tmpSize = value.asInteger();
+	    else if ( attr == Chain("appsize"))
+		appSize = value.asInteger();
 	    else if ( attr == Chain("logfilesize"))
 		logFileSize = value.asInteger();
 	    else if ( attr == Chain("logfilenum"))
 		logFileNum = value.asInteger();
-	    else if ( attr == Chain("tsticket"))
-		tsTicket = value;
 	    else if ( attr == Chain("tsroot"))
 		tsRoot = value;
-	    else if ( attr == Chain("appfile"))
-		appFileName = value;
-	    else if ( attr == Chain("appsize"))
-		appFileSize = value.asInteger();
-	    else if ( attr == Chain("ordersize"))
+	    else if ( attr == Chain("sortareasize"))
 		orderSize = value.asLong();
-
 	}
 	
-	Host h;
 	int sysFid = pDBMng->nextTSID();
+	
 	int tmpFid = pDBMng->nextFID();
 	
+	Chain dbHost;
+	pDBMng->getDBHost(dbHost);
+	
 	pDBMng->addTableSetDef(tableSet, 
-			       tsRoot, 
-			       tsTicket,
-			       h.getName(), 
-			       h.getName(), 
-			       h.getName(), 
+			       tsRoot,
+			       dbHost, 
+			       dbHost, 
+			       dbHost, 
 			       sysFid, 
 			       tmpFid, 
 			       sysSize, 
 			       tmpSize,
+			       appSize,
 			       logFileSize, 
 			       logFileNum,
 			       orderSize); 
 	
 	
-	int appFid = pDBMng->nextFID();
-	pDBMng->addDataFile(tableSet, Chain(XML_APPFILE_VALUE), appFid, appFileName, appFileSize);
+	// int appFid = pDBMng->nextFID();
+	// pDBMng->addDataFile(tableSet, Chain(XML_APPFILE_VALUE), appFid, appFileName, appFileSize);
 	
 	pDBMng->doc2Xml();
 	
@@ -679,11 +821,16 @@ int createTableSet(GetLongOpt& longOpt)
     try {
 	
 	Chain logFile = longOpt.getOptValue("logfile");
+	Chain syslog = longOpt.getOptValue("syslog");
+
 	Chain lockFile = longOpt.getOptValue("lockfile");	
 	Chain dbXML = longOpt.getOptValue("dbxml");
 	Chain tableSet = longOpt.getOptValue("tableset");
+
 	Chain poolSize = longOpt.getOptValue("poolsize");
-	
+	int numSeg, segSize;
+	getPoolInfo(poolSize, numSeg, segSize);
+
 	if ( dbXML.length() == 0  || dbXML == Chain("") )
 	{
 	    throw Exception(EXLOC, "No dbxml set");
@@ -692,23 +839,31 @@ int createTableSet(GetLongOpt& longOpt)
 	{
 	    throw Exception(EXLOC, "No tableSet set");
 	}
-		
-	pDBMng = new CegoDatabaseManager(dbXML, lockFile, logFile);
-		
-	pDBMng->initPool(poolSize.asInteger());
+
+	Chain progName = SYSLOGPROGNAME;
+	if ( syslog == Chain("no") )
+	    progName=Chain("");
+
+	pDBMng = new CegoDatabaseManager(dbXML, lockFile, logFile, progName);
+
+	pDBMng->xml2Doc();
+	pDBMng->initDoc();
 	pDBMng->configureLogger();
+
+	pDBMng->initPool(numSeg, segSize);
+
 	unsigned long modId = pDBMng->getModId("CegoMain");
 	
 	pDBMng->log(modId, Logger::NOTICE, Chain("Creating tableset ") + tableSet + Chain(" ..."));
 	
 	CegoLockHandler* pLockHandler = new CegoLockHandler(pDBMng);
-	
+
 	pLockHandler->initLocks();	    
 	
-	CegoTableManager tabMng(pDBMng);
-	
+	CegoDistManager tabMng(pDBMng);
+
 	tabMng.createTableSet(tableSet);
-	
+
 	pDBMng->log(modId, Logger::NOTICE, Chain("Tableset ") + tableSet + Chain(" created"));
 	
 	pLockHandler->deleteLocks();
@@ -730,7 +885,6 @@ int createTableSet(GetLongOpt& longOpt)
     return exitCode;
 }
 
-
 int importTableSet(GetLongOpt& longOpt, XPortFormat format)
 {
 	
@@ -742,11 +896,12 @@ int importTableSet(GetLongOpt& longOpt, XPortFormat format)
     try 
     {
 	
+	Chain syslog = longOpt.getOptValue("syslog");
 	Chain logFile = longOpt.getOptValue("logfile");
 	Chain lockFile = longOpt.getOptValue("lockfile");
 	Chain dbXML = longOpt.getOptValue("dbxml");
 	Chain tableSet = longOpt.getOptValue("tableset");
-	Chain poolSize = longOpt.getOptValue("poolsize");
+	Chain poolSize = longOpt.getOptValue("poolize");
 	Chain impFileName = longOpt.getOptValue("impfile");
 
 	bool doLogging = true;
@@ -775,11 +930,22 @@ int importTableSet(GetLongOpt& longOpt, XPortFormat format)
 	{
 	    throw Exception(EXLOC, "No impfile set");
 	}
-	
-	pDBMng = new CegoDatabaseManager(dbXML, lockFile, logFile);
 
-	pDBMng->initPool(poolSize.asInteger());
+	Chain progName = SYSLOGPROGNAME;
+	if ( syslog == Chain("no") )
+	    progName=Chain("");
+	
+	pDBMng = new CegoDatabaseManager(dbXML, lockFile, logFile, progName);
+
+	pDBMng->xml2Doc();
+	pDBMng->initDoc();
 	pDBMng->configureLogger();
+	
+	int numSeg, segSize;
+	getPoolInfo(poolSize, numSeg, segSize);
+
+	pDBMng->initPool(numSeg, segSize);
+
 	unsigned long modId = pDBMng->getModId("CegoMain");
 	
 	CegoLockHandler* pLockHandler = 0;
@@ -789,22 +955,8 @@ int importTableSet(GetLongOpt& longOpt, XPortFormat format)
 	try
 	{
 
-	    if ( pDBMng->isCaseSensitiveMode() )
-		__caseSensitiveFlag = 1;
-	    else
-		__caseSensitiveFlag = 0;
-	    
-	    if ( pDBMng->isQuoteEscapeMode() )	       
-		__quoteEscapeFlag = 1;
-	    else
-		__quoteEscapeFlag = 0;
-
-
-	    Chain dtFormat = pDBMng->getDateFormatString();
-	    
-	    if ( dtFormat != Chain("") )
-		__dateFormatString=dtFormat;
-	    
+	    readParameter(pDBMng);
+	    	    
 	    pLockHandler = new CegoLockHandler(pDBMng);
 	    
 	    pLockHandler->initLocks();
@@ -826,8 +978,10 @@ int importTableSet(GetLongOpt& longOpt, XPortFormat format)
 	    pDBMng->log(modId, Logger::DEBUG, Chain("Starting tableset ") + tableSet + Chain(" ..."));
 #endif	    
 	    
-	    Host h;
-	    pTabMng->startDistTableSet(tableSet, h.getName(), false);
+	    Chain dbHost;
+	    pDBMng->getDBHost(dbHost);
+
+	    pTabMng->startDistTableSet(tableSet, dbHost, false);
 	    
 	    CegoXPorter xp(pTabMng);
 	    
@@ -915,11 +1069,17 @@ int exportTableSet(GetLongOpt& longOpt, XPortFormat format)
     try 
     {
 
+	Chain syslog = longOpt.getOptValue("syslog");
 	Chain logFile = longOpt.getOptValue("logfile");
 	Chain lockFile = longOpt.getOptValue("lockfile");
 	Chain dbXML = longOpt.getOptValue("dbxml");
 	Chain tableSet = longOpt.getOptValue("tableset");
+
 	Chain poolSize = longOpt.getOptValue("poolsize");
+	int numSeg, segSize;
+	getPoolInfo(poolSize, numSeg, segSize);
+
+
 	Chain expFileName = longOpt.getOptValue("expfile");  
 	Chain userName = longOpt.getOptValue("user");
 
@@ -944,11 +1104,19 @@ int exportTableSet(GetLongOpt& longOpt, XPortFormat format)
 	{
 	    throw Exception(EXLOC, "No expfile set");
 	}
+
+	Chain progName = SYSLOGPROGNAME;
+	if ( syslog == Chain("no") )
+	    progName=Chain("");
 	
-	pDBMng = new CegoDatabaseManager(dbXML, lockFile, logFile);
-	
-	pDBMng->initPool(poolSize.asInteger());	   
+	pDBMng = new CegoDatabaseManager(dbXML, lockFile, logFile, progName);
+
+	pDBMng->xml2Doc();
+	pDBMng->initDoc();
 	pDBMng->configureLogger();
+
+	pDBMng->initPool(numSeg, segSize);
+
 	unsigned long modId = pDBMng->getModId("CegoMain");
 		
 	CegoLockHandler* pLockHandler = 0;
@@ -958,21 +1126,8 @@ int exportTableSet(GetLongOpt& longOpt, XPortFormat format)
 	try
 	{
 
-	    if ( pDBMng->isCaseSensitiveMode() )
-		__caseSensitiveFlag = 1;
-	    else
-		__caseSensitiveFlag = 0;
+	    readParameter(pDBMng);
 	    
-	    if ( pDBMng->isQuoteEscapeMode() )	       
-		__quoteEscapeFlag = 1;
-	    else
-		__quoteEscapeFlag = 0;
-
-	    Chain dtFormat = pDBMng->getDateFormatString();
-	    
-	    if ( dtFormat != Chain("") )
-		__dateFormatString=dtFormat;
-	    	    
 	    pLockHandler = new CegoLockHandler(pDBMng);
 
 	    pLockHandler->initLocks();
@@ -992,8 +1147,11 @@ int exportTableSet(GetLongOpt& longOpt, XPortFormat format)
 #ifdef CGDEBUG	
 	    pDBMng->log(modId, Logger::DEBUG, Chain("Starting tableset ") + tableSet + Chain(" ..."));    	
 #endif    
-	    Host h;
-	    pTabMng->startDistTableSet(tableSet, h.getName(), false);
+
+	    Chain dbHost;
+	    pDBMng->getDBHost(dbHost);
+
+	    pTabMng->startDistTableSet(tableSet, dbHost, false);
 	    
 	    CegoXPorter xp(pTabMng);
 	    
@@ -1083,11 +1241,16 @@ int runBatch(GetLongOpt& longOpt)
     try 
     {
 
+	Chain syslog = longOpt.getOptValue("syslog");
 	Chain logFile = longOpt.getOptValue("logfile");
 	Chain lockFile = longOpt.getOptValue("lockfile");
 	Chain dbXML = longOpt.getOptValue("dbxml");
 	Chain tableSet = longOpt.getOptValue("tableset");
+
 	Chain poolSize = longOpt.getOptValue("poolsize");
+	int numSeg, segSize;
+	getPoolInfo(poolSize, numSeg, segSize);
+
 	Chain batchFileName = longOpt.getOptValue("batchfile");  
 	bool ignoreError = longOpt.isSet("ignore");
 	Chain userName = longOpt.getOptValue("user");
@@ -1100,7 +1263,6 @@ int runBatch(GetLongOpt& longOpt)
 	userTok.nextToken(user);
 	userTok.nextToken(password);
 	
-
 	if ( dbXML.length() == 0  || dbXML == Chain("") )
 	{
 	    throw Exception(EXLOC, "No dbxml set");
@@ -1109,11 +1271,20 @@ int runBatch(GetLongOpt& longOpt)
 	{
 	    throw Exception(EXLOC, "No tableSet set");
 	}
-	    	    	
-	pDBMng = new CegoDatabaseManager(dbXML, lockFile, logFile);
+
+	Chain progName = SYSLOGPROGNAME;
+	if ( syslog == Chain("no") )
+	    progName=Chain("");
 	
-	pDBMng->initPool(poolSize.asInteger());
+	pDBMng = new CegoDatabaseManager(dbXML, lockFile, logFile, progName);
+
+
+	pDBMng->xml2Doc();
+	pDBMng->initDoc();
 	pDBMng->configureLogger();
+		
+	pDBMng->initPool(numSeg, segSize);
+
 	unsigned long modId = pDBMng->getModId("CegoMain");
 		
 	CegoLockHandler* pLockHandler = 0;
@@ -1124,22 +1295,8 @@ int runBatch(GetLongOpt& longOpt)
 	try {
 	    
 	    pLockHandler = new CegoLockHandler(pDBMng);
-	    
-	    
-	    if ( pDBMng->isCaseSensitiveMode() )
-		__caseSensitiveFlag = 1;
-	    else
-		__caseSensitiveFlag = 0;
-	    
-	    if ( pDBMng->isQuoteEscapeMode() )	       
-		__quoteEscapeFlag = 1;
-	    else
-		__quoteEscapeFlag = 0;
-	    
-	    Chain dtFormat = pDBMng->getDateFormatString();
-	    if ( dtFormat != Chain("") )
-		__dateFormatString=dtFormat;
-	    
+
+	    readParameter(pDBMng);
 	    
 	    pLockHandler->initLocks();
 	    
@@ -1162,8 +1319,10 @@ int runBatch(GetLongOpt& longOpt)
 	    pDBMng->log(modId, Logger::DEBUG, Chain("Starting tableset ") + tableSet + Chain(" ..."));
 #endif	    
 
-	    Host h;
-	    pTabMng->startDistTableSet(tableSet, h.getName(), false);
+	    Chain dbHost;
+	    pDBMng->getDBHost(dbHost);
+
+	    pTabMng->startDistTableSet(tableSet, dbHost, false);
 	    	    
 #ifdef CGDEBUG	
 	    pDBMng->log(modId, Logger::DEBUG, Chain("Tableset ") + tableSet + Chain(" completed"));
@@ -1224,11 +1383,8 @@ int runBatch(GetLongOpt& longOpt)
     {
 	cerr << "ERROR: " << errorMsg << endl;
 	
-    }	
-
-    
+    }
     return exitCode;
-
 }
 
 int runDaemon(GetLongOpt& longOpt)
@@ -1241,6 +1397,7 @@ int runDaemon(GetLongOpt& longOpt)
 
     try {
     
+	Chain syslog = longOpt.getOptValue("syslog");
 	Chain logFile = longOpt.getOptValue("logfile");
 	Chain lockFile = longOpt.getOptValue("lockfile");
 	Chain pidFile = longOpt.getOptValue("pidfile");
@@ -1248,6 +1405,9 @@ int runDaemon(GetLongOpt& longOpt)
 	Chain tableSetList = longOpt.getOptValue("tableset");
 
 	Chain poolSize = longOpt.getOptValue("poolsize");
+	int numSeg, segSize;
+	getPoolInfo(poolSize, numSeg, segSize);
+
 	Chain numDbThread = longOpt.getOptValue("numdbthread");
 	Chain numAdminThread = longOpt.getOptValue("numadminthread");
 	Chain numLogThread = longOpt.getOptValue("numlogthread");
@@ -1256,7 +1416,7 @@ int runDaemon(GetLongOpt& longOpt)
 	bool cleanIt = longOpt.isSet("cleanup");
 	bool forceload = longOpt.isSet("forceload");
 
-	
+
 	CegoMediatorThread *pMedThread = 0;
     	
 	if ( dbXML.length() == 0  || dbXML == Chain("") )
@@ -1278,8 +1438,12 @@ int runDaemon(GetLongOpt& longOpt)
 	    throw Exception(EXLOC, "Invalid protocol");
 	}
 
+	Chain progName = SYSLOGPROGNAME;
+	if ( syslog == Chain("no") )
+	    progName=Chain("");
 	
-	pDBMng = new CegoDatabaseManager(dbXML, lockFile, logFile, protType);
+	pDBMng = new CegoDatabaseManager(dbXML, lockFile, logFile, progName, protType);
+		
 	pMedThread = new CegoMediatorThread(pDBMng);
 	    
 	modId = pDBMng->getModId("CegoMain");
@@ -1307,43 +1471,36 @@ int runDaemon(GetLongOpt& longOpt)
 
 	    CegoBeatConnection beat;
 	    beat = CegoBeatConnection(host, port.asInteger(), user, aescrypt.encrypt(passwd), pDBMng);
-	    
+
+	    pDBMng->xml2Doc();
 	    pDBMng->configureLogger();
 	    pMedThread->getDbSpec(dbXML, beat.getHostName(), beat.getPortNo(), beat.getUser(), beat.getPasswd());
 	    pDBMng->setXmlDef(dbXML);
 
-	    pDBMng->xml2Doc();	   
+	    pDBMng->xml2Doc();
+	    
+	    // setting dbHost to gethostbyname
 	    Host h;
 	    pDBMng->setDBHost(h.getName());
 	    pDBMng->doc2Xml();	   
 	}
 
-	pDBMng->initPool(poolSize.asInteger());
-	pDBMng->configureLogger();	     	
+	pDBMng->xml2Doc();
+	pDBMng->initDoc();
+	pDBMng->configureLogger();
 	
+	pDBMng->initPool(numSeg, segSize);
 
+	
 #ifdef CGDEBUG	
 	pDBMng->log(modId, Logger::DEBUG, "Initializing page and record locks ...");
 #endif	
 
+	readParameter(pDBMng);
+	
 	CegoLockHandler* pLockHandler = new CegoLockHandler(pDBMng);
-	
-	if ( pDBMng->isCaseSensitiveMode() )       
-	    __caseSensitiveFlag = 1;
-	else
-	    __caseSensitiveFlag = 0;
-	
-	if ( pDBMng->isQuoteEscapeMode() )	       
-	    __quoteEscapeFlag = 1;
-	else
-	    __quoteEscapeFlag = 0;
-	
-	Chain dtFormat = pDBMng->getDateFormatString();
-	if ( dtFormat != Chain("") )
-	    __dateFormatString=dtFormat;
-	
 	pLockHandler->initLocks();
-	
+
 #ifdef CGDEBUG	
 	pDBMng->log(modId, Logger::DEBUG, "Creating db threadpool ...");
 #endif
@@ -1376,8 +1533,8 @@ int runDaemon(GetLongOpt& longOpt)
 		
 	CegoBeatThread *pBeatThread = new CegoBeatThread(pDBMng);
 	
+
 	pMedThread->start(0);
-	
 	pDbPool->start(0);
 	pLogPool->start(0);
 	pAdminPool->start(0);
@@ -1387,11 +1544,8 @@ int runDaemon(GetLongOpt& longOpt)
 	// is pidfile set in command line ?
 	if ( pidFile.length() == 0  || pidFile == Chain("") )
 	{
-
 	    // if not, get pid file from xml
-	    pDBMng->getPidFile(pidFile);
-	    
-
+	    pDBMng->getPidFile(pidFile);   
 	}
 
 	if ( pidFile != Chain("") )
@@ -1407,12 +1561,12 @@ int runDaemon(GetLongOpt& longOpt)
 	{
 	    throw Exception(EXLOC, "No pidfilename set");
 	}
-	
-	Host h;
-	
+		
 	// correct online entries 
 	ListT<Chain> corList;
-	pDBMng->getActiveTableSet(h.getName(), corList);
+	Chain dbHost;
+	pDBMng->getDBHost(dbHost);
+	pDBMng->getActiveTableSet(dbHost, corList);
 	
 	if ( corList.Size() > 0 )
 	{
@@ -1514,8 +1668,8 @@ int runDaemon(GetLongOpt& longOpt)
 	    }
 	    catch ( Exception e )
 	    {
-		delete pTabMng;
-		throw Exception(EXLOC, Chain("Cannot start tableset  ") + tableSet, e);
+		delete pTabMng;		
+		throw e;
 	    }
 	    delete pTabMng;
 	}
@@ -1536,7 +1690,7 @@ int runDaemon(GetLongOpt& longOpt)
 	    ListT<Chain> actList;
 
 	    // we get all active tableset, except the tablesets which actually are recovered
-	    pDBMng->getActiveTableSet(h.getName(), actList, false);
+	    pDBMng->getActiveTableSet(dbHost, actList, false);
 	    
 	    if ( actList.Size() > 0 )
 	    {
@@ -1572,16 +1726,14 @@ int runDaemon(GetLongOpt& longOpt)
 			    exep += Chain("\n\t") + module + Chain("(") +  Chain(line) + Chain(") : ") + msg;
 			}
 			
-			pDBMng->log(modId, Logger::LOGERR, Chain("Main Thread : ") + exep);
+			pDBMng->log(modId, Logger::LOGALERT, Chain("Main Thread : ") + exep);
 		    }
 #ifdef CGDEBUG	
 		    pDBMng->log(modId, Logger::DEBUG, Chain("Tableset ") + *pAct + Chain(" checked"));
 #endif
 
-		    
 		    pAct = actList.Next();
-		}
-		
+		}		
 	    }
 
 	    pDBMng->beat();
@@ -1596,7 +1748,7 @@ int runDaemon(GetLongOpt& longOpt)
 	
 	ListT<Chain> atsList;
 	
-	pDBMng->getActiveTableSet(h.getName(), atsList);
+	pDBMng->getActiveTableSet(dbHost, atsList);
 	
 	if ( atsList.Size() > 0 )
 	{
@@ -1702,7 +1854,6 @@ int runDaemon(GetLongOpt& longOpt)
     return exitCode;
 }
 
-
 bool processBatchFile(CegoDatabaseManager *pDBMng, CegoAction *pAction, const Chain& batchFileName, bool ignoreError, bool consoleOut, Chain& errorMsg)
 {
     File batchFile(batchFileName);
@@ -1725,7 +1876,7 @@ bool processBatchFile(CegoDatabaseManager *pDBMng, CegoAction *pAction, const Ch
     {
 	lineNo++;
 	line = line.cutTrailing(" \t");
-	
+
 	bool isCommentLine = false;
 	
 	if ( line.length() >= 2 )
@@ -1751,17 +1902,21 @@ bool processBatchFile(CegoDatabaseManager *pDBMng, CegoAction *pAction, const Ch
 		cmd = cmd.cutTrailing(" \t");
 	    }
 	    
+	    
 	    if ( cmd.length() > 0 )
 	    {
-		
+	
 		if (cmd.subChain(cmd.length()-1, cmd.length()) == Chain(";") 
 		    && disableDelimiter==false)
 		{
-		    
+
+		    if ( consoleOut == false )			
+		    {
+			pDBMng->log(modId, Logger::NOTICE, Chain("Processing batch command <<<") + cmd + Chain(">>>"));
+		    }
 #ifdef CGDEBUG	
-		    pDBMng->log(modId, Logger::DEBUG, Chain("Processing command <<<") + cmd + Chain(">>>"));	
-#endif		
-    
+		    pDBMng->log(modId, Logger::DEBUG, Chain("Processing batch command <<<") + cmd + Chain(">>>"));
+#endif
 		    pAction->cleanUp();
 		    pAction->setCommandChain(cmd);
 		    
@@ -1789,7 +1944,7 @@ bool processBatchFile(CegoDatabaseManager *pDBMng, CegoAction *pAction, const Ch
 			    exep += Chain("\n\t") + module + Chain("(") +  Chain(line) + Chain(") : ") + msg;
 			}
 			
-			pDBMng->log(modId, Logger::LOGERR, Chain("Batch Thread : ") + exep);
+			pDBMng->log(modId, Logger::LOGERR, Chain("Batch Error : ") + exep);
 
 			errorMsg=e.getBaseMsg();			
 			
@@ -1823,7 +1978,7 @@ bool processBatchFile(CegoDatabaseManager *pDBMng, CegoAction *pAction, const Ch
     }
     if ( cmd.length() > 1 && consoleOut)
     {
-	cout << "incomplete command ..<" << cmd << ">" << endl;
+	cout << "Incomplete command <<<" << cmd << ">>>" << endl;
     }
     
     batchFile.close();
@@ -1831,3 +1986,37 @@ bool processBatchFile(CegoDatabaseManager *pDBMng, CegoAction *pAction, const Ch
     return true;
     
 }
+
+bool getPoolInfo(const Chain& poolSize, int& numSeg, int& segSize)
+{
+    Tokenizer tok(poolSize, "xX");
+    Chain t1, t2;
+    if ( tok.nextToken(t1) )
+    {
+	if ( tok.nextToken(t2) )
+	{
+	    // multisegment spec
+	    if ( t1.asInteger() <= 0 || t2.asInteger() <= 0 )
+	    {
+		return false;
+	    }
+	    numSeg=t1.asInteger();
+	    segSize=t2.asInteger();
+	    return true;	    
+	}
+	else
+	{
+	    if ( t1.asInteger() <= 0 )
+	    {
+		return false;
+	    }
+	    else
+	    {
+		numSeg=1;
+		segSize=t1.asInteger();
+		return true;
+	    }
+	}
+    }
+    return false;    
+}
