@@ -6,30 +6,15 @@
 //                                                         
 // Design and Implementation by Bjoern Lemke               
 //                                                         
-// (C)opyright 2000-2010 Bjoern Lemke                        
-//
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
-// 
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; see the file COPYING.  If not, write to
-// the Free Software Foundation, 59 Temple Place - Suite 330,
-// Boston, MA 02111-1307, USA.
+// (C)opyright 2000-2016 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
 // Class: CegoProcNoopStmt
-// 
-// Description:
 //
-// Status: QG-2.6
+// Description: Stored procedure no-operation statement
+//
+// Status: CLEAN
 //
 ///////////////////////////////////////////////////////////////////////////////
 
@@ -40,6 +25,7 @@
 
 // cego includes
 #include "CegoProcNoopStmt.h"
+#include "CegoDatabaseFormater.h"
 
 CegoProcNoopStmt::CegoProcNoopStmt(CegoProcBlock *pBlock) : CegoProcStmt(pBlock)
 {
@@ -60,3 +46,8 @@ Chain CegoProcNoopStmt::toChain(const Chain& indent) const
     s = indent + Chain("noop");
     return s;
 }
+
+Chain CegoProcNoopStmt::dbFormat(CegoDatabaseFormater *pForm) const
+{
+    return pForm->formatProcNoopStmt();
+}
