@@ -6,30 +6,15 @@
 //     
 // Design and Implementation by Bjoern Lemke
 //     
-// (C)opyright 2000-2012 Bjoern Lemke
-//
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
-// 
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; see the file COPYING.  If not, write to
-// the Free Software Foundation, 59 Temple Place - Suite 330,
-// Boston, MA 02111-1307, USA.
+// (C)opyright 2000-2016 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
 // Class: CegoQueryHelper
-// 
-// Description: 
 //
-// Status: QG-2.6
+// Description: The CegoQueryHelper class provides several utility methods with are used for tuple retrieval and evalution
+//
+// Status: CLEAN
 //
 ///////////////////////////////////////////////////////////////////////////////
 
@@ -54,12 +39,10 @@ CegoQueryHelper::~CegoQueryHelper()
 {
 }
 
-bool CegoQueryHelper::evalPredicate(ListT<CegoField>* pParentJoinBuf,
+bool CegoQueryHelper::evalPredicate(ListT<CegoField>** pParentJoinBuf,
 				    int parentJoinBufPos,
-				    int parentJoinBufSize,
-				    ListT<CegoField>* joinBuf,
+				    ListT<CegoField>** pJoinBuf,
 				    int pos,
-				    int size, 
 				    CegoPredDesc* pP, 
 				    CegoProcBlock *pBlock)
 {
@@ -75,35 +58,28 @@ bool CegoQueryHelper::evalPredicate(ListT<CegoField>* pParentJoinBuf,
 	case CegoCondDesc::AND:
 	    return ( evalPredicate(pParentJoinBuf, 
 				   parentJoinBufPos,
-				   parentJoinBufSize,
-				   joinBuf,
+				   pJoinBuf,
 				   pos,
-				   size, 
 				   pC->Left(), 
-				   pBlock) 
+				   pBlock)
 		     && evalPredicate(pParentJoinBuf,
 				      parentJoinBufPos,
-				      parentJoinBufSize,
-				      joinBuf,
+				      pJoinBuf,
 				      pos,
-				      size, 
 				      pC->Right(),
 				      pBlock));
 	case CegoCondDesc::OR:
+	    	    
 	    return ( evalPredicate(pParentJoinBuf,
 				   parentJoinBufPos,
-				   parentJoinBufSize, 
-				   joinBuf,
+				   pJoinBuf,
 				   pos,
-				   size, 
 				   pC->Left(),
 				   pBlock) 
 		     || evalPredicate(pParentJoinBuf,
 				      parentJoinBufPos,
-				      parentJoinBufSize,
-				      joinBuf,
+				      pJoinBuf,
 				      pos,
-				      size, 
 				      pC->Right(),
 				      pBlock));
 	default:
@@ -116,10 +92,8 @@ bool CegoQueryHelper::evalPredicate(ListT<CegoField>* pParentJoinBuf,
 	{
 	    bool np = evalPredicate(pParentJoinBuf,
 				    parentJoinBufPos,
-				    parentJoinBufSize,
-				    joinBuf,
+				    pJoinBuf,
 				    pos,
-				    size,
 				    pP->getNotPred(),
 				    pBlock);
 	    if ( np )
@@ -131,25 +105,46 @@ bool CegoQueryHelper::evalPredicate(ListT<CegoField>* pParentJoinBuf,
 	else
 	{
 
-	    ListT<CegoField>* flArray;
-	    int flSize;
-	    bool isAllocated = false;
+	    ListT<CegoField>** flArray;
+	    ListT<CegoField>* flStaticArray[TABMNG_MAXJOINLEVEL];
 
 	    if ( pParentJoinBuf )
 	    {
-		flSize = parentJoinBufSize + size;
-		flArray = new ListT<CegoField>[flSize];
-		isAllocated = true;
+		flArray = flStaticArray;
 		
-		for ( int i=0; i<parentJoinBufSize; i++)
-		    flArray[i] = pParentJoinBuf[parentJoinBufPos + i];
-		for ( int i=0; i<size; i++)
-		    flArray[parentJoinBufSize + i] = joinBuf[pos + i];		
+		int flpos=0;
+		int i=parentJoinBufPos;
+		while ( pParentJoinBuf[i] )
+		{
+		    if ( flpos > TABMNG_MAXJOINLEVEL )
+		    {
+			Chain msg = Chain("Join buffer exceeded");
+			throw Exception(EXLOC, msg);
+		    }			
+			    
+		    flArray[flpos] = pParentJoinBuf[i];
+		    i++;
+		    flpos++;
+		}
+		i=pos;
+		while ( pJoinBuf[i] )
+		{
+		    if ( flpos > TABMNG_MAXJOINLEVEL )
+		    {
+			Chain msg = Chain("Join buffer exceeded");
+			throw Exception(EXLOC, msg);
+		    }			
+
+		    flArray[flpos] = pJoinBuf[i];
+		    i++;
+		    flpos++;
+		}
+		// terminate array
+		flArray[flpos]=0;
 	    }
 	    else
 	    {
-		flArray = joinBuf + pos;
-		flSize = size;
+		flArray = pJoinBuf + pos;
 	    }
 	    
 	    bool retVal = false;
@@ -159,23 +154,22 @@ bool CegoQueryHelper::evalPredicate(ListT<CegoField>* pParentJoinBuf,
 		
 	    case CegoPredDesc::EXPRCOMP:
 	    {
-		pP->getExpr1()->setFieldListArray(flArray, flSize);
-		pP->getExpr2()->setFieldListArray(flArray, flSize);
+		pP->getExpr1()->setFieldListArray(flArray);
+		pP->getExpr2()->setFieldListArray(flArray);
 		
 		pP->getExpr1()->setBlock(pBlock);
 		pP->getExpr2()->setBlock(pBlock);
 
 		// cout << "Eval : " << pP->getExpr1()->evalFieldValue() << " comp " << pP->getExpr2()->evalFieldValue() << endl;
-		
 		retVal = evalFields(pP->getComparison(), pP->getExpr1()->evalFieldValue(), pP->getExpr2()->evalFieldValue());
 
 		break;
 	    }    
 	    case CegoPredDesc::BETWEEN:
 	    {
-		pP->getExpr1()->setFieldListArray(flArray, flSize);
-		pP->getExpr2()->setFieldListArray(flArray, flSize);
-		pP->getExpr3()->setFieldListArray(flArray, flSize);
+		pP->getExpr1()->setFieldListArray(flArray);
+		pP->getExpr2()->setFieldListArray(flArray);
+		pP->getExpr3()->setFieldListArray(flArray);
 		    
 		pP->getExpr1()->setBlock(pBlock);
 		pP->getExpr2()->setBlock(pBlock);
@@ -189,7 +183,7 @@ bool CegoQueryHelper::evalPredicate(ListT<CegoField>* pParentJoinBuf,
 	    }    		
 	    case CegoPredDesc::NULLCOMP:
 	    {	  
-		pP->getExpr1()->setFieldListArray(flArray, flSize);
+		pP->getExpr1()->setFieldListArray(flArray);
 		pP->getExpr1()->setBlock(pBlock);
 		
 		if ( pP->getExpr1()->evalFieldValue().isNull() )
@@ -205,7 +199,7 @@ bool CegoQueryHelper::evalPredicate(ListT<CegoField>* pParentJoinBuf,
 	    case CegoPredDesc::NOTNULLCOMP:
 	    {	
 		
-		pP->getExpr1()->setFieldListArray(flArray, flSize);
+		pP->getExpr1()->setFieldListArray(flArray);
 		pP->getExpr1()->setBlock(pBlock);
 		
 		if ( pP->getExpr1()->evalFieldValue().isNull() )
@@ -223,8 +217,8 @@ bool CegoQueryHelper::evalPredicate(ListT<CegoField>* pParentJoinBuf,
 		
 		CegoSelect* pSelect = pP->getSelectQuery();
 
-		pSelect->reset(true);		
-		pSelect->setParentJoinBuf(flArray, flSize);
+		pSelect->reset(false);		
+		pSelect->setParentJoinBuf(flArray);
 				
 		ListT<CegoField> fl;
 		
@@ -236,39 +230,39 @@ bool CegoQueryHelper::evalPredicate(ListT<CegoField>* pParentJoinBuf,
 		{
 		    retVal = false;
 		}
-		// pSelect->reset();
+		
+		pSelect->reset(false);
 		break;
 	    }
 	    case CegoPredDesc::ISLIKE:
 	    case CegoPredDesc::ISNOTLIKE:
 	    {
 		
-		pP->getExpr1()->setFieldListArray(flArray, flSize);
+		pP->getExpr1()->setFieldListArray(flArray);
 		pP->getExpr1()->setBlock(pBlock);
+
 		
 		if ( pP->getMode() == CegoPredDesc::ISLIKE )
 		    retVal = pP->match(pP->getExpr1()->evalFieldValue());
 		else
 		    retVal =  ! pP->match(pP->getExpr1()->evalFieldValue());
-		
+
 		break;
 	    }
 	    case CegoPredDesc::INSUB:
 	    case CegoPredDesc::NOTINSUB:
 	    {
 
-		pP->getExpr1()->setFieldListArray(flArray, flSize);
+		pP->getExpr1()->setFieldListArray(flArray);
 		pP->getExpr1()->setBlock(pBlock);
 		
 		CegoSelect* pSelect = pP->getSelectQuery();
 
-		pSelect->reset(true);
+		pSelect->reset(false);
 				
-		pSelect->setParentJoinBuf(flArray, flSize);
+		pSelect->setParentJoinBuf(flArray);
 		pSelect->setProcBlock(pBlock);
-		
-		pSelect->setCaching(true);
-			       
+					       
 		ListT<CegoField> fl;
 				
 		if ( pP->getMode() == CegoPredDesc::INSUB )
@@ -298,14 +292,14 @@ bool CegoQueryHelper::evalPredicate(ListT<CegoField>* pParentJoinBuf,
 		{
 		    while ( pSelect->nextTuple(fl));
 		}
-
+		
+		pSelect->reset(false);
+		
 		break;    
 	    }
 	    default:
 		break;
 	    }
-	    if ( isAllocated == true )
-		delete[] flArray;
 	    return retVal;
 	}
     }
@@ -326,7 +320,6 @@ CegoQueryHelper::AttrCondMatch CegoQueryHelper::checkAttrCond(CegoAttrCond& ac,
     return m;
 }
 
-
 CegoQueryHelper::AttrCondMatch CegoQueryHelper::evalAttrCond(CegoAttrCond& ac, 
 							     CegoPredDesc* pP, 
 							     const ListT<CegoField>& schema,
@@ -341,8 +334,6 @@ CegoQueryHelper::AttrCondMatch CegoQueryHelper::evalAttrCond(CegoAttrCond& ac,
     }
     else if (pP->getMode() == CegoPredDesc::EXPRCOMP)
     {
-
-
 	CegoAttrDesc* pAttrDesc1;
 	CegoAttrDesc* pAttrDesc2;
 	
@@ -373,7 +364,8 @@ CegoQueryHelper::AttrCondMatch CegoQueryHelper::evalAttrCond(CegoAttrCond& ac,
 				 pAttrDesc2->getTableName() == Chain())
 				&& (Chain)pAttrDesc2->getAttrName() == (Chain)pFB->getAttrName())
 			    {
-				ac.add(CegoAttrComp(pF->getAttrName(), 
+				ac.add(CegoAttrComp(pF->getTableAlias(), 
+						    pF->getAttrName(), 
 						    pP->getComparison(),
 						    *pAttrDesc2));
 			    }
@@ -397,8 +389,21 @@ CegoQueryHelper::AttrCondMatch CegoQueryHelper::evalAttrCond(CegoAttrCond& ac,
 				 pAttrDesc1->getTableName() == Chain())
 				&& (Chain)pAttrDesc1->getAttrName() == (Chain)pFB->getAttrName())
 			    {
-				ac.add(CegoAttrComp(pF->getAttrName(), 
-						    pP->getComparison(),
+
+				// we have to map comparison 
+				CegoComparison comp = pP->getComparison();
+				if ( comp == LESS_THAN )
+				    comp = MORE_THAN;
+				else if ( comp == MORE_THAN )
+				    comp = LESS_THAN;
+				else if ( comp == LESS_EQUAL_THAN )
+				    comp = MORE_EQUAL_THAN;
+				else if ( comp == MORE_EQUAL_THAN )
+				    comp = LESS_EQUAL_THAN;
+
+				ac.add(CegoAttrComp(pF->getTableAlias(), 
+						    pF->getAttrName(), 
+						    comp,
 						    *pAttrDesc1));
 			    }
 			    pFB = flArray[i].Next();
@@ -420,12 +425,15 @@ CegoQueryHelper::AttrCondMatch CegoQueryHelper::evalAttrCond(CegoAttrCond& ac,
 		    && pAttrDesc1->getAttrName() == (Chain)pF->getAttrName())
 		{
 		    pP->getExpr2()->setBlock(pBlock);
-		    ac.add(CegoAttrComp(pF->getAttrName(),
+		    ac.add(CegoAttrComp(pF->getTableAlias(), 
+					pF->getAttrName(),
 					pP->getComparison(),
 					pP->getExpr2()->evalFieldValue()));
+		    return COMPLETE;
 		}
 		pF = schema.Next();
 	    }
+	    return PARTIAL;
 	}
 	else if ( pAttrDesc2 && attrRefList1.Size() == 0 )    
 	{
@@ -439,12 +447,15 @@ CegoQueryHelper::AttrCondMatch CegoQueryHelper::evalAttrCond(CegoAttrCond& ac,
 		    && pAttrDesc2->getAttrName() == (Chain)pF->getAttrName())
 		{
 		    pP->getExpr1()->setBlock(pBlock);
-		    ac.add(CegoAttrComp(pF->getAttrName(),
+		    ac.add(CegoAttrComp(pF->getTableAlias(), 
+					pF->getAttrName(),
 					pP->getComparison(),
 					pP->getExpr1()->evalFieldValue()));
+		    return COMPLETE;
 		}
 		pF = schema.Next();
 	    }
+	    return PARTIAL;
 	}
 	else
 	{
@@ -476,12 +487,15 @@ CegoQueryHelper::AttrCondMatch CegoQueryHelper::evalAttrCond(CegoAttrCond& ac,
 		    pP->getExpr2()->setBlock(pBlock);
 		    pP->getExpr3()->setBlock(pBlock);
 
-		    ac.add(CegoAttrComp(pF->getAttrName(),
+		    ac.add(CegoAttrComp(pF->getTableAlias(), 
+					pF->getAttrName(),
 					pP->getExpr2()->evalFieldValue(),
 					pP->getExpr3()->evalFieldValue()));
+		    return COMPLETE;
 		}
 		pF = schema.Next();
-	    }	    
+	    }
+	    return PARTIAL;
 	}
 	else
 	{
@@ -494,12 +508,26 @@ CegoQueryHelper::AttrCondMatch CegoQueryHelper::evalAttrCond(CegoAttrCond& ac,
 	pAttrDesc = pP->getExpr1()->checkAttr();
 	if ( pAttrDesc  )
 	{
-	    ac.add(CegoAttrComp(pAttrDesc->getAttrName(), pP->getPattern(), false));
+	    CegoField* pF = schema.First(); 
+	    while (pF)
+	    {		
+		if ((pAttrDesc->getTableName() == (Chain)pF->getTableName() ||
+		     pAttrDesc->getTableName() == (Chain)pF->getTableAlias() ||
+		     pAttrDesc->getTableName() == Chain())
+		    && pAttrDesc->getAttrName() == (Chain)pF->getAttrName())
+		{
+		    ac.add(CegoAttrComp(pAttrDesc->getTableName(), pAttrDesc->getAttrName(), pP->getPattern(), false));
+		    return COMPLETE;
+		}
+		pF = schema.Next();
+	    }
+	    return PARTIAL;
 	}
 	else
 	{
-	    return PARTIAL;	    
+	    return PARTIAL;
 	}
+
     }
     else if (pP->getMode() == CegoPredDesc::ISNOTLIKE)
     {
@@ -507,13 +535,97 @@ CegoQueryHelper::AttrCondMatch CegoQueryHelper::evalAttrCond(CegoAttrCond& ac,
 	pAttrDesc = pP->getExpr1()->checkAttr();
 	if ( pAttrDesc  )
 	{
-	    ac.add(CegoAttrComp(pAttrDesc->getAttrName(), pP->getPattern(), true));
+	    CegoField* pF = schema.First(); 
+	    while (pF)
+	    {		
+		if ((pAttrDesc->getTableName() == (Chain)pF->getTableName() ||
+		     pAttrDesc->getTableName() == (Chain)pF->getTableAlias() ||
+		     pAttrDesc->getTableName() == Chain())
+		    && pAttrDesc->getAttrName() == (Chain)pF->getAttrName())
+		{
+		    ac.add(CegoAttrComp(pAttrDesc->getTableName(), pAttrDesc->getAttrName(), pP->getPattern(), true));
+		    return COMPLETE;
+		}
+		pF = schema.Next();
+	    }
+	    return PARTIAL;
 	}
 	else
 	{
 	    return PARTIAL;
 	}
     }
+    else if (pP->getMode() == CegoPredDesc::NULLCOMP)
+    {
+	CegoAttrDesc* pAttrDesc;	
+	pAttrDesc = pP->getExpr1()->checkAttr();
+	if ( pAttrDesc  )
+	{
+	    CegoField* pF = schema.First(); 
+	    while (pF)
+	    {		
+		if ((pAttrDesc->getTableName() == (Chain)pF->getTableName() ||
+		     pAttrDesc->getTableName() == (Chain)pF->getTableAlias() ||
+		     pAttrDesc->getTableName() == Chain())
+		    && pAttrDesc->getAttrName() == (Chain)pF->getAttrName())
+		{		    
+		    CegoFieldValue nullValue;
+		    ac.add(CegoAttrComp(pAttrDesc->getTableName(), pAttrDesc->getAttrName(), EQUAL, nullValue));
+		    return COMPLETE;
+		}
+		pF = schema.Next();
+	    }
+	    return PARTIAL;
+	}
+	else
+	{
+	    return PARTIAL;	    
+	}
+    }
+    else if (pP->getMode() == CegoPredDesc::NOTNULLCOMP)
+    {
+	CegoAttrDesc* pAttrDesc;	
+	pAttrDesc = pP->getExpr1()->checkAttr();
+	if ( pAttrDesc  )
+	{
+
+	    CegoField* pF = schema.First(); 
+	    while (pF)
+	    {		
+		if ((pAttrDesc->getTableName() == (Chain)pF->getTableName() ||
+		     pAttrDesc->getTableName() == (Chain)pF->getTableAlias() ||
+		     pAttrDesc->getTableName() == Chain())
+		    && pAttrDesc->getAttrName() == (Chain)pF->getAttrName())
+		{		    
+		    CegoFieldValue nullValue;
+		    ac.add(CegoAttrComp(pAttrDesc->getTableName(), pAttrDesc->getAttrName(), NOT_EQUAL, nullValue));
+		    return COMPLETE;
+		}
+		pF = schema.Next();
+	    }
+	    return PARTIAL;
+	}
+	else
+	{
+	    return PARTIAL;	    
+	}
+    }
+    else if (pP->getMode() == CegoPredDesc::EXISTSCOMP)
+    {
+	return PARTIAL;	    
+    }
+    else if (pP->getMode() == CegoPredDesc::INSUB)
+    {
+	return PARTIAL;	    
+    }
+    else if (pP->getMode() == CegoPredDesc::NOTINSUB)
+    {
+	return PARTIAL;	    
+    }
+    else if (pP->getMode() == CegoPredDesc::NOTPRED)
+    {
+	return PARTIAL;
+    }
     else
     {
 	return PARTIAL;
@@ -699,14 +811,12 @@ bool CegoQueryHelper::evalBetween(const CegoFieldValue& f1, const CegoFieldValue
 void CegoQueryHelper::aggregateTuple(ListT<CegoField>& aggTuple, CegoExpr* pExpr)
 {
 
-    ListT<CegoAggregation*> aggList = pExpr->getAggregationList();
-    
-    CegoAggregation **pAgg = aggList.First();
+    CegoAggregation **pAgg = pExpr->getAggregationList().First();
     
     while ( pAgg )
     {
 	aggregateTuple(aggTuple, *pAgg);	
-	pAgg = aggList.Next();
+	pAgg = pExpr->getAggregationList().Next();
     }
 }
 
@@ -718,15 +828,10 @@ void CegoQueryHelper::aggregateTuple(ListT<CegoField>& aggTuple, CegoAggregation
     
     CegoExpr *pAE = pAgg->getExpr();
     
-    if ( pAE == 0 )
-    {
-	f.setAttrName(Chain("*"));
-	fv.setType(INT_TYPE);
-    }
-    else
+    if ( pAE )
     {
+	pAE->setFieldListArray( &aggTuple );
 
-	pAE->setFieldListArray( &aggTuple, 1 );
 	fv = pAE->evalFieldValue();
 		
 	if ( fv.getValue() == 0 && pAgg->getType() != CegoAggregation::MIN )
@@ -738,6 +843,11 @@ void CegoQueryHelper::aggregateTuple(ListT<CegoField>& aggTuple, CegoAggregation
 		fv = CegoFieldValue(INT_TYPE, Chain("0"));
 		break;
 	    }
+	    case LONG_TYPE:
+	    {			
+		fv = CegoFieldValue(LONG_TYPE, Chain("0"));
+		break;
+	    }
 	    case VARCHAR_TYPE:
 	    {
 		fv = CegoFieldValue(VARCHAR_TYPE, Chain(""));
@@ -775,16 +885,20 @@ void CegoQueryHelper::aggregateTuple(ListT<CegoField>& aggTuple, CegoAggregation
     {
 	if ( pAgg->getFieldValue().getValue() == 0 )
 	{
-	    pAgg->setFieldValue( CegoFieldValue(INT_TYPE, Chain("1")) );
+	    pAgg->setFieldValue( CegoFieldValue(LONG_TYPE, Chain("1")) );
 	}
 	else
 	{
-	    // int *pI = (int*)(pAgg->getFieldValue().getValue());
-	    // (*pI)++;
-	    int i;
-	    memcpy(&i, pAgg->getFieldValue().getValue(), sizeof(int));
-	    i++;
-	    memcpy(pAgg->getFieldValue().getValue(), &i, sizeof(int));
+	    
+	    (*(((long*)(pAgg->getFieldValue().getValue()))))++;
+	    
+	    /*
+	    long l;
+	    memcpy(&l, pAgg->getFieldValue().getValue(), sizeof(long));
+	    l++;
+	    memcpy(pAgg->getFieldValue().getValue(), &l, sizeof(long));
+	    */
+	    
 	}
 	break;
     }
@@ -840,7 +954,7 @@ void CegoQueryHelper::aggregateTuple(ListT<CegoField>& aggTuple, CegoAggregation
     }    
 }
 
-void CegoQueryHelper::prepareFieldValue(CegoField* pFV, CegoFieldValue& fv, int pos)
+void CegoQueryHelper::prepareFieldValue(CegoField* pFV, CegoFieldValue& fv, CegoTableManager* pTM, int tabSetId)
 {
     if ( pFV->isNullable() == false && fv.isNull() )
     {
@@ -848,10 +962,14 @@ void CegoQueryHelper::prepareFieldValue(CegoField* pFV, CegoFieldValue& fv, int
 			+ pFV->getAttrName() 
 			+ Chain("> in value list"));	
     }
+
     if ( fv.getType() != NULL_TYPE && pFV->getType() != fv.getType() )	
     {
- 
-	if ( fv.castTo(pFV->getType()) == false )
+
+	if ( string2Clob(pFV, fv, pTM, tabSetId) )
+	     return;
+	     
+	if ( fv.castTo(pFV->getType(), pFV->getLength()) == false )
 	{
 	    throw Exception(EXLOC, Chain("Mismatched datatype <") 
 			    + CEGO_TYPE_MAP[(int)fv.getType()] 
@@ -862,14 +980,7 @@ void CegoQueryHelper::prepareFieldValue(CegoField* pFV, CegoFieldValue& fv, int
     }
     
     if ( fv.getValue() != 0 )
-    {
-	if ( pFV->getType() == FIXED_TYPE )
-	{		
-	    BigDecimal d ( fv.valAsChain() );
-	    if ( d.getScale() != pFV->getLength())
-		throw Exception(EXLOC, "Mismatched decimal scale in value list");
-	}
-	
+    {	
 	if ( pFV->getType() == VARCHAR_TYPE  
 	     && pFV->getLength() < fv.getLength() )
 	{
@@ -888,6 +999,42 @@ void CegoQueryHelper::prepareFieldValue(CegoField* pFV, CegoFieldValue& fv, int
 		memcpy(fv.getValue(), &dtval, sizeof(int));	   	    
 	    }
 	}
+
+	if ( pFV->getType() == BLOB_TYPE || pFV->getType() == CLOB_TYPE )
+	{
+	    int fileId;
+	    int pageId;
+	    memcpy(&fileId, fv.getValue(), sizeof(int));
+	    memcpy(&pageId, (void*)((long long)fv.getValue() + sizeof(int)), sizeof(int));
+
+	    try
+	    {
+		if ( pTM->getDBMng()->isClaimed(fileId, pageId, pTM->getLockHandler()))
+		{
+		    
+		    CegoBufferPage bp;
+		    pTM->getDBMng()->bufferFix(bp, tabSetId, fileId, pageId, CegoBufferPool::SYNC, pTM->getLockHandler());
+		    pTM->getDBMng()->bufferUnfix(bp, true, pTM->getLockHandler());
+		    
+		    if (  pFV->getType() == BLOB_TYPE && bp.getType() == CegoBufferPage::BLOB )
+		    {
+			// ok !
+		    }
+		    else if (  pFV->getType() == CLOB_TYPE && bp.getType() == CegoBufferPage::CLOB )
+		    {
+			// ok !
+		    }
+		    else
+		    {
+			throw Exception(EXLOC, Chain("Invalid lob page"));
+		    }		    
+		}
+	    }
+	    catch ( Exception e )
+	    {
+		throw Exception(EXLOC, Chain("Invalid lob value"));
+	    }
+	}
     }
 }
 
@@ -916,7 +1063,7 @@ void CegoQueryHelper::encodeFL(const ListT<CegoField>& fvl, char* &pBufBase, int
     
     if (pBufBase == 0)
     {
-	throw Exception(EXLOC, "malloc system error");	
+	throw Exception(EXLOC, "malloc system error");
     }
 
     char *pBuf = pBufBase;
@@ -1010,20 +1157,26 @@ void CegoQueryHelper::decodeFL(ListT<CegoField>& fvl, char* pc, int len)
     }
 }
 
-void CegoQueryHelper::encodeFVL(int tid, int tastep, CegoTupleState ts, const ListT<CegoField>& fvl, char* &pBufBase, int &buflen)
+void CegoQueryHelper::encodeFVL(unsigned long long tid, unsigned long long tastep, CegoTupleState ts, const ListT<CegoField>& fvl, char* &pBufBase, int &buflen)
 {
     ListT<CegoBlob> blobList;
-    encodeFVL(tid, tastep, ts, fvl, blobList, pBufBase, buflen);
+    ListT<CegoClob> clobList;
+    
+    encodeFVL(tid, tastep, ts, fvl, blobList, clobList, pBufBase, buflen);
 }
 
-void CegoQueryHelper::encodeFVL(int tid, int tastep, CegoTupleState ts, const ListT<CegoField>& fvl, const ListT<CegoBlob>& blobList, char* &pBufBase, int &buflen)
+void CegoQueryHelper::encodeFVL(unsigned long long tid, unsigned long long tastep, CegoTupleState ts, const ListT<CegoField>& fvl, const ListT<CegoBlob>& blobList, const ListT<CegoClob>& clobList, char* &pBufBase, int &buflen)
 {
 
-    buflen = sizeof(int); // for tid
-    buflen += sizeof(int); // for tastep
-    buflen += sizeof(CegoTupleState); // for tuplestate
+    // buflen = sizeof(unsigned long long); // for tid
+    // buflen += sizeof(unsigned long long); // for tastep
+    // buflen += sizeof(CegoTupleState); // for tuplestate
+
+    buflen = skipTupleHeader();
 
     int blobIdx = 0;
+    int clobIdx = 0;
+    
     CegoField* pF = fvl.First();
     while (pF)
     {
@@ -1040,14 +1193,18 @@ void CegoQueryHelper::encodeFVL(int tid, int tastep, CegoTupleState ts, const Li
 	    buflen += sizeof(int); // id
 	    buflen += sizeof(CegoDataType);
 
-
 	    if ( dt == BLOB_TYPE && blobList.Size() > 0)
-	    {
-		
+	    {	
 		buflen += sizeof(long); // blobsize
 		buflen += blobList[blobIdx].getSize();
 		blobIdx++;
 	    }
+	    else if ( dt == CLOB_TYPE && clobList.Size() > 0)
+	    {		
+		buflen += sizeof(long); // clobsize
+		buflen += clobList[clobIdx].getSize();
+		clobIdx++;
+	    }
 	    else
 	    {
 		
@@ -1066,7 +1223,7 @@ void CegoQueryHelper::encodeFVL(int tid, int tastep, CegoTupleState ts, const Li
     }
 
     buflen += sizeof(int); // terminating id
-
+    
     pBufBase = (char*) malloc(buflen);
     
     if (pBufBase == 0)
@@ -1076,16 +1233,12 @@ void CegoQueryHelper::encodeFVL(int tid, int tastep, CegoTupleState ts, const Li
 
     char *pBuf = pBufBase;
 
-    memcpy( pBuf, &tid, sizeof(int));
-    pBuf += sizeof(int);
-
-    memcpy( pBuf, &tastep, sizeof(int));
-    pBuf += sizeof(int);
-
-    memcpy( pBuf, &ts, sizeof(CegoTupleState));
-    pBuf += sizeof(CegoTupleState);
+    int toff = encodeTupleHeader(tid, tastep, ts, pBuf);
+    pBuf += toff;
 
     blobIdx = 0;
+    clobIdx = 0;
+    
     pF = fvl.First();
     while (pF)
     {
@@ -1100,7 +1253,6 @@ void CegoQueryHelper::encodeFVL(int tid, int tastep, CegoTupleState ts, const Li
 	
 	if ( dt != NULL_TYPE )
 	{
-	    
 	    // cout << "ID = " << id << " Encoding value " << pF->getValue().valAsChain() << endl;
 
 	    memcpy( pBuf, &id, sizeof(int));
@@ -1111,12 +1263,20 @@ void CegoQueryHelper::encodeFVL(int tid, int tastep, CegoTupleState ts, const Li
 	    
 	    if ( dt == BLOB_TYPE && blobList.Size() > 0)
 	    {
-		long blobSize = blobList[blobIdx].getSize();
-		memcpy( pBuf, &blobSize, sizeof(long));
-		pBuf += sizeof(long);
+		unsigned long long blobSize = blobList[blobIdx].getSize();
+		memcpy( pBuf, &blobSize, sizeof(unsigned long long));
+		pBuf += sizeof(unsigned long long);
 		memcpy( pBuf, blobList[blobIdx].getBufPtr(), blobList[blobIdx].getSize());
 		pBuf += blobList[blobIdx].getSize();
 	    }
+	    else if ( dt == CLOB_TYPE && clobList.Size() > 0)
+	    {
+		unsigned long long clobSize = clobList[clobIdx].getSize();
+		memcpy( pBuf, &clobSize, sizeof(unsigned long long));
+		pBuf += sizeof(long);
+		memcpy( pBuf, clobList[clobIdx].getBufPtr(), clobList[clobIdx].getSize());
+		pBuf += clobList[clobIdx].getSize();
+	    }
 	    else
 	    {
 		if ( dt == VARCHAR_TYPE 
@@ -1139,18 +1299,43 @@ void CegoQueryHelper::encodeFVL(int tid, int tastep, CegoTupleState ts, const Li
 
     int termId=0;
     memcpy( pBuf, &termId, sizeof(int));
-    // pBuf += sizeof(int);
+
 }
 
-void CegoQueryHelper::decodeFVL(ListT<CegoField>* fvlArray, int pos, char* pc, int len, int &tid, int &tastep, CegoTupleState& ts)
+// for performance reasons, we also provide a method to just return the 
+// tuple header len
+int CegoQueryHelper::skipTupleHeader()
 {
-    decodeFVL(fvlArray[pos], pc, len, tid, tastep, ts, true);
+    return sizeof(unsigned long long) + sizeof(unsigned long long) + sizeof(CegoTupleState);
 }
 
-void CegoQueryHelper::decodeFVL(ListT<CegoField>& fvl, char* pc, int len, int &tid, int &tastep, CegoTupleState& ts, bool doReset)
+int CegoQueryHelper::encodeTupleHeader(unsigned long long tid, unsigned long long tastep, CegoTupleState ts, char* p)
 {
+    memcpy(p, &tid, sizeof(unsigned long long));
+    p += sizeof(unsigned long long);
+    memcpy(p, &tastep, sizeof(unsigned long long));
+    p += sizeof(unsigned long long);
+    memcpy(p, &ts, sizeof(CegoTupleState));
+    
+    return sizeof(unsigned long long) + sizeof(unsigned long long) + sizeof(CegoTupleState);
+}
 
+int CegoQueryHelper::decodeTupleHeader(unsigned long long& tid, unsigned long long& tastep, CegoTupleState& ts, char* p)
+{
+    memcpy(&tid, p, sizeof(unsigned long long));
 
+    char* ptastep = p + sizeof(unsigned long long);
+    memcpy(&tastep, ptastep, sizeof(unsigned long long));
+    
+    char* pts = p + sizeof(unsigned long long) + sizeof(unsigned long long);
+    memcpy(&ts, pts, sizeof(CegoTupleState));
+	
+    return sizeof(unsigned long long) + sizeof(unsigned long long) + sizeof(CegoTupleState);
+}
+
+void CegoQueryHelper::decodeFVL(ListT<CegoField>& fvl, char* pc, int len, bool doReset)
+{
+    
     if ( doReset )
     {
 	// make sure no field values are set    
@@ -1165,92 +1350,101 @@ void CegoQueryHelper::decodeFVL(ListT<CegoField>& fvl, char* pc, int len, int &t
 
     char* pBase = pc;
     
-    memcpy(&tid, pc, sizeof(int));
-    pc += sizeof(int);
-
-    memcpy(&tastep, pc, sizeof(int));
-    pc += sizeof(int);
-
-    memcpy(&ts, pc, sizeof(CegoTupleState));
-    pc += sizeof(CegoTupleState);
-
     bool eot = false;
 
+    int fldCount = fvl.Size();
 
-    CegoField* pF = fvl.First(); 
-
-    while (pc - pBase < len && eot == false && pF)
+    while (pc - pBase < len && eot == false && fldCount > 0 )
     {
 	int id;     
 	memcpy(&id, pc, sizeof(int));
-
-	// cout << "Decoding id " << id << endl;
-
-	// cout << "PF Id =  " << pF->getId() << endl;
-
-	while ( pF && id > pF->getId() )
+	
+	if ( id > 0 )
 	{
-	    pF = fvl.Next();
-	}
-	if ( pF == 0 )
-	    return;
-	
-	pc += sizeof(int);
-	
-	CegoDataType dt;
-	memcpy(&dt, pc, sizeof(CegoDataType));
-	
-	pc += sizeof(CegoDataType);
-	
-	int flen;
-	
-	if (dt == VARCHAR_TYPE
-	    || dt == BIGINT_TYPE
-	    || dt == DECIMAL_TYPE
-	    || dt == FIXED_TYPE )
-	{
-	    memcpy(&flen, pc, sizeof(int));
+	    
 	    pc += sizeof(int);
+	    
+	    CegoDataType dt;
+	    memcpy(&dt, pc, sizeof(CegoDataType));
+	    
+	    pc += sizeof(CegoDataType);
+	    
+	    int flen;
+	    
+	    if (dt == VARCHAR_TYPE
+		|| dt == BIGINT_TYPE
+		|| dt == DECIMAL_TYPE
+		|| dt == FIXED_TYPE )
+	    {
+		memcpy(&flen, pc, sizeof(int));
+		pc += sizeof(int);
+	    }
+	    else
+	    {
+		CegoTypeConverter tc;
+		flen = tc.getTypeLen(dt);	    
+	    }
+	    
+	    CegoField* pF = fvl.First(); 
+	    while ( pF && id != pF->getId() )
+	    {
+		pF = fvl.Next();
+	    }
+	    
+	    if ( pF )
+	    {
+
+		/*
+		
+		pF->getValue().setType(dt);
+		pF->getValue().setLength(flen);
+		
+		if ( flen > 0)
+		    pF->getValue().setValue(pc);
+		else
+		    pF->getValue().setValue(0);
+		pF->getValue().setLocalCopy(false);
+	
+		since CegoField::getValue() has changed to const, this must be done vie setValue()	
+		
+		*/
+		
+		CegoFieldValue fv;
+		fv.setType(dt);
+		fv.setLength(flen);
+
+		if ( flen > 0)
+		    fv.setValue(pc);
+		else
+		    fv.setValue(0);
+		fv.setLocalCopy(false);
+
+		pF->setValue(fv);
+
+		
+		fldCount--;
+		
+	    }
+	    
+	    if ( flen > 0 )
+	    {	    
+		pc += flen;
+	    }
 	}
 	else
 	{
-	    CegoTypeConverter tc;
-	    flen = tc.getTypeLen(dt);	    
+	    eot = true;
 	}
-		
-	if (pF->getId() == id )
-	{
-	    pF->getValue().setType(dt);
-	    pF->getValue().setLength(flen);
-	    // pF->setId(id);
 	    
-	    if ( flen > 0)
-		pF->getValue().setValue(pc);
-	    else
-		pF->getValue().setValue(0);
-	    pF->getValue().setLocalCopy(false);
-	    
-	    // cout << "Decoded valued " << pF->getValue() << endl;
-	    
-	    pF = fvl.Next();
-	}
+    }
 	
-	if ( flen > 0 )
-	{	    
-	    pc += flen;
-	}
-    }
-    /*
-    if ( pF )
-    {
-	cout << "Decoding not complete !!!!!!! => " << pF->getId() << endl;
-    }
-    */
+    /* fldCount is the number of not encoded values => null values */
+
 }
 
+/* field list decoding with included blob decoding. This is used by the recovery manager for redo log recovery  */
 
-
-void CegoQueryHelper::decodeFVL(ListT<CegoField>& fvl, ListT<CegoBlob>& blobList, char* pc, int len, int &tid, int &tastep, CegoTupleState& ts)
+void CegoQueryHelper::decodeFVL(ListT<CegoField>& fvl, ListT<CegoBlob>& blobList, ListT<CegoClob>& clobList, char* pc, int len)
 {
     
     // make sure no field values are set
@@ -1264,15 +1458,6 @@ void CegoQueryHelper::decodeFVL(ListT<CegoField>& fvl, ListT<CegoBlob>& blobList
 
     char* pBase = pc;
     
-    memcpy(&tid, pc, sizeof(int));
-    pc += sizeof(int);
-
-    memcpy(&tastep, pc, sizeof(int));
-    pc += sizeof(int);
-
-    memcpy(&ts, pc, sizeof(CegoTupleState));
-    pc += sizeof(CegoTupleState);
-
     bool eot = false;
     while (pc - pBase < len && eot == false )
     {
@@ -1288,7 +1473,8 @@ void CegoQueryHelper::decodeFVL(ListT<CegoField>& fvl, ListT<CegoBlob>& blobList
 	    pc += sizeof(CegoDataType);
 	    
 	    int flen=0;
-	    long blobSize=0;
+	    unsigned long long blobSize=0;
+	    unsigned long long clobSize=0;
 	    
 	    if (dt == VARCHAR_TYPE
 		|| dt == BIGINT_TYPE
@@ -1300,8 +1486,13 @@ void CegoQueryHelper::decodeFVL(ListT<CegoField>& fvl, ListT<CegoBlob>& blobList
 	    }
 	    else if (dt == BLOB_TYPE )
 	    {
-		memcpy(&blobSize, pc, sizeof(long));
-		pc += sizeof(long);
+		memcpy(&blobSize, pc, sizeof(unsigned long long));
+		pc += sizeof(unsigned long long);
+	    }
+	    else if (dt == CLOB_TYPE )
+	    {
+		memcpy(&clobSize, pc, sizeof(unsigned long long));
+		pc += sizeof(unsigned long long);
 	    }
 	    else
 	    {
@@ -1315,16 +1506,36 @@ void CegoQueryHelper::decodeFVL(ListT<CegoField>& fvl, ListT<CegoBlob>& blobList
 	    {
 		if (pF->getId() == id )
 		{
+
+
+		    
 		    if ( dt == BLOB_TYPE )
 		    {
-			pF->getValue().setType(dt);
+			// pF->getValue().setType(dt);
+			// since CegoField::getValue() has changed to const, this must be done vie setValue()
+			CegoFieldValue fv = pF->getValue();
+			fv.setType(dt);
+			pF->setValue(fv);
+			
 			unsigned char *blobBuf = (unsigned char*)malloc(blobSize);
 			memcpy(blobBuf, pc, blobSize);
 			blobList.Insert( CegoBlob(0,0, blobBuf, blobSize) );
 		    }
+		    else if ( dt == CLOB_TYPE )
+		    {
+			// pF->getValue().setType(dt);
+			// since CegoField::getValue() has changed to const, this must be done vie setValue()
+			CegoFieldValue fv = pF->getValue();
+			fv.setType(dt);
+			pF->setValue(fv);
+
+			char *clobBuf = (char*)malloc(clobSize);
+			memcpy(clobBuf, pc, clobSize);
+			clobList.Insert( CegoClob(0,0, clobBuf, clobSize) );
+		    }
 		    else
 		    {
-			
+			/*
 			pF->getValue().setType(dt);
 			pF->getValue().setLength(flen);
 			pF->setId(id);
@@ -1334,6 +1545,22 @@ void CegoQueryHelper::decodeFVL(ListT<CegoField>& fvl, ListT<CegoBlob>& blobList
 			else
 			    pF->getValue().setValue(0);
 			pF->getValue().setLocalCopy(false);
+
+			since CegoField::getValue() has changed to const, this must be done vie setValue()
+
+			*/
+
+			CegoFieldValue fv = pF->getValue();
+			fv.setType(dt);			
+			fv.setLength(flen);			
+			if ( flen > 0)
+			    fv.setValue(pc);
+			else
+			    fv.setValue(0);
+			fv.setLocalCopy(false);
+			
+			pF->setValue(fv);
+			pF->setId(id);
 			
 			
 		    }
@@ -1357,21 +1584,11 @@ void CegoQueryHelper::decodeFVL(ListT<CegoField>& fvl, ListT<CegoBlob>& blobList
     }
 }
 
-void CegoQueryHelper::decodeNativeFVL(ListT<CegoFieldValue>& fvl, ListT<CegoBlob>& blobList, char* pc, int len, int &tid, int& tastep, CegoTupleState& ts)
+void CegoQueryHelper::decodeNativeFVL(ListT<CegoFieldValue>& fvl, ListT<CegoBlob>& blobList, ListT<CegoClob>& clobList, char* pc, int len)
 {
 		
     char* pBase = pc;
 
-    memcpy(&tid, pc, sizeof(int));    
-    pc = pc + sizeof(int);
-
-    memcpy(&tastep, pc, sizeof(int));    
-    pc = pc + sizeof(int);
-
-    memcpy(&ts, pc, sizeof(CegoTupleState));
-    pc = pc + sizeof(CegoTupleState);
-
-
     bool eot = false;    
     while (pc - pBase < len && eot == false)
     {		    
@@ -1388,7 +1605,8 @@ void CegoQueryHelper::decodeNativeFVL(ListT<CegoFieldValue>& fvl, ListT<CegoBlob
 	    pc += sizeof(CegoDataType);
 	    
 	    int flen=0;
-	    long blobSize=0;
+	    unsigned long long blobSize=0;
+	    unsigned long long clobSize=0;
 	    
 	    if (dt == VARCHAR_TYPE
 		|| dt == BIGINT_TYPE
@@ -1400,8 +1618,13 @@ void CegoQueryHelper::decodeNativeFVL(ListT<CegoFieldValue>& fvl, ListT<CegoBlob
 	    }
 	    else if (dt == BLOB_TYPE )
 	    {
-		memcpy(&blobSize, pc, sizeof(long));
-		pc += sizeof(long);
+		memcpy(&blobSize, pc, sizeof(unsigned long long));
+		pc += sizeof(unsigned long long);
+	    }
+	    else if (dt == CLOB_TYPE )
+	    {
+		memcpy(&clobSize, pc, sizeof(unsigned long long));
+		pc += sizeof(unsigned long long);
 	    }
 	    else
 	    {
@@ -1418,6 +1641,16 @@ void CegoQueryHelper::decodeNativeFVL(ListT<CegoFieldValue>& fvl, ListT<CegoBlob
 		
 		fvl.Insert(fv);
 	    }
+	    else if ( dt == CLOB_TYPE )
+	    {
+		CegoFieldValue fv(dt, Chain("[0,0]"));
+		char *clobBuf = (char*)malloc(clobSize);
+		memcpy(clobBuf, pc, clobSize);
+		clobList.Insert( CegoClob(0,0, clobBuf, clobSize) );
+		
+		fvl.Insert(fv);
+	    }
+	    
 	    else
 	    {
 		CegoFieldValue fv(dt, pc, flen);
@@ -1539,7 +1772,7 @@ void CegoQueryHelper::decodeUpdRec(Chain& tableAlias,
 				   ListT<CegoField>& updList, 
 				   ListT<CegoExpr*>& exprList,
 				   char* pBuf, int buflen,
-				   CegoDistManager *pGTM)
+				   CegoDistManager *pGTM, int tabSetId)
 {
 
     int aliasLen;
@@ -1555,7 +1788,7 @@ void CegoQueryHelper::decodeUpdRec(Chain& tableAlias,
 
     if ( c == 1 )
     {
-	pPred = new CegoPredDesc(pBuf, pGTM);
+	pPred = new CegoPredDesc(pBuf, pGTM, tabSetId);
 	pBuf += pPred->getEncodingLength();
     }
     else
@@ -1578,7 +1811,7 @@ void CegoQueryHelper::decodeUpdRec(Chain& tableAlias,
     int pos = 0;
     while ( pos < expBufLen  )
     {
-	CegoExpr *pExpr = new CegoExpr(pBuf, pGTM);
+	CegoExpr *pExpr = new CegoExpr(pBuf, pGTM, tabSetId);
 
 	int len = pExpr->getEncodingLength();
 	pos += len;
@@ -1609,6 +1842,7 @@ void CegoQueryHelper::encodeDelRec(const Chain& tableAlias,
 	char c = 1;
 	memcpy(pP, &c, sizeof(char));
 	pP = pP + sizeof(char);
+
 	pPred->encode(pP);
 	
     }
@@ -1645,7 +1879,7 @@ void CegoQueryHelper::encodeDelRec(const Chain& tableAlias,
 void CegoQueryHelper::decodeDelRec(Chain& tableAlias,
 				   CegoPredDesc* &pPred,
 				   char* pBuf, int buflen,
-				   CegoDistManager *pGTM)
+				   CegoDistManager *pGTM, int tabSetId)
 {        
     int aliasLen;
     memcpy(&aliasLen, pBuf, sizeof(int));
@@ -1660,7 +1894,7 @@ void CegoQueryHelper::decodeDelRec(Chain& tableAlias,
 
     if ( c == 1 )
     {
-	pPred = new CegoPredDesc(pBuf, pGTM);
+	pPred = new CegoPredDesc(pBuf, pGTM, tabSetId);
 	pBuf += pPred->getEncodingLength();
     }
     else
@@ -1670,112 +1904,6 @@ void CegoQueryHelper::decodeDelRec(Chain& tableAlias,
 
 }
 
-
-void CegoQueryHelper::evalSelection(ListT<CegoExpr*>& exprList, ListT<CegoField>* joinBuf, int jbufSize, bool isAgg, ListT<CegoField>& ol) 
-{
-
-    if ( exprList.isEmpty() )
-    {
-	
-	for ( int i = 0; 
-	      i < jbufSize; 
-	      i++ )
-	{
-	    CegoField* pF = joinBuf[i].First();
-	    while ( pF ) 
-	    {				
-		ol.Insert(*pF);
-		pF = joinBuf[i].Next();
-	    }
-	}
-	return;
-    }
-
-    if ( isAgg )
-    {
-	CegoExpr** pExpr = exprList.First();
-	while ( pExpr )
-	{
-	    
-	    (*pExpr)->setFieldListArray( joinBuf, jbufSize);
-	    
-	    ListT<CegoField> fl = (*pExpr)->getFieldList();
-
-	    CegoField *pF = fl.First();
-	    while ( pF )
-	    {
-		int i =0;	       
-		CegoField *pJF = 0;
-		while ( i < jbufSize && pJF == 0 )
-		{
-		    CegoField *pJF = joinBuf[i].Find(*pF);
-		    if ( pJF )
-		    {
-			pF->setValue(pJF->getValue());			
-		    }	       
-		    i++;
-		}
-		
-		pF = fl.Next();
-	    }
-	    
-	    ol = ol + fl;
-
-	    pExpr = exprList.Next();
-	}
-    }
-    else
-    {
-
-	CegoExpr** pExpr = exprList.First();
-	while ( pExpr )
-	{
-
-	    CegoAttrDesc* pAttrDesc = (*pExpr)->checkAttr();
-	    if ( pAttrDesc )
-	    {
-		if ( pAttrDesc->getAttrName() == Chain(SELECTION_WILDCARD))
-		{
-		    
-		    int i=0;
-		    while ( i < jbufSize )
-		    {		    
-			CegoField *pF = joinBuf[i].First();
-			while ( pF )
-			{
-			    if ( (Chain)pF->getTableName() ==  (Chain)pAttrDesc->getTableName()
-				 || (Chain)pF->getTableAlias() ==  (Chain)pAttrDesc->getTableName() )
-			    {			    
-				ol.Insert(*pF);
-			    }
-			    pF = joinBuf[i].Next();
-			}
-			i++;
-		    }
-		}
-		else
-		{
-		    (*pExpr)->setFieldListArray( joinBuf, jbufSize);
-		    CegoField f = (*pExpr)->evalField();
-		    f.setValue ( (*pExpr)->evalFieldValue() ); 
-		    ol.Insert ( f );
-		}
-	    }	
-	    else
-	    {
-		(*pExpr)->setFieldListArray( joinBuf, jbufSize);
-		CegoField f = (*pExpr)->evalField();
-		f.setValue ( (*pExpr)->evalFieldValue() ); 
-		ol.Insert ( f );
-	    }
-	    pExpr = exprList.Next();
-	}
-    }
-       
-    return;
-    
-}
-
 void CegoQueryHelper::createConjunctionList(CegoPredDesc* pPred, ListT<CegoPredDesc*>& conjunctionList)
 {
     if (pPred)
@@ -1800,3 +1928,21 @@ void CegoQueryHelper::createConjunctionList(CegoPredDesc* pPred, ListT<CegoPredD
     }
 }
 
+bool CegoQueryHelper::string2Clob(CegoField* pFV, CegoFieldValue& fv, CegoTableManager* pTM, int tabSetId)
+{
+    if ( pFV->getType() == CLOB_TYPE && fv.getType() == VARCHAR_TYPE && pTM )
+    {
+	char* clobData = (char*)fv.getValue();
+	unsigned long long clobSize = fv.getLength();
+	int fileId, pageId;
+	pTM->putClobData(tabSetId, clobData, clobSize, fileId, pageId);
+
+	// cout << "Clob created with " << fileId << " " << pageId << endl;
+
+	fv = CegoFieldValue(CLOB_TYPE, Chain("[") + Chain(fileId) + Chain(",") + Chain(pageId) + Chain("]"));
+
+	// cout << "Clob is " << fv << endl;
+	return true;
+    }
+    return false;
+}
