@@ -6,30 +6,15 @@
 //     
 // Design and Implementation by Bjoern Lemke               
 //     
-// (C)opyright 2000-2010 Bjoern Lemke
-//
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
-// 
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; see the file COPYING.  If not, write to
-// the Free Software Foundation, 59 Temple Place - Suite 330,
-// Boston, MA 02111-1307, USA.
+// (C)opyright 2000-2016 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
 // Class: CegoTableObject
 // 
-// Description: 
+// Description: Cego table object structure
 //
-// Status: QG-2.6
+// Status: CLEAN
 //
 ///////////////////////////////////////////////////////////////////////////////
 
@@ -296,6 +281,10 @@ bool CegoTableObject::operator == ( const CegoTableObject& oe)
     return CegoContentObject::operator==(oe);
 }
 
+Chain CegoTableObject::getId() const
+{
+    return getTabName() + Chain("@") + getTableSet() + Chain("@") + getName();
+}
 
 Chain CegoTableObject::toChain() const
 {
@@ -315,13 +304,13 @@ Element* CegoTableObject::getElement() const
     case CegoObject::TABLE:
 	pRoot->setAttribute(XML_OBJTYPE_ATTR, XML_TABOBJ_VALUE);	
 	break;
-    case CegoObject::PINDEX:
+    case CegoObject::PAVLTREE:
 	pRoot->setAttribute(XML_OBJTYPE_ATTR, XML_PIXOBJ_VALUE);	
 	break;
-    case CegoObject::UINDEX:
+    case CegoObject::UAVLTREE:
 	pRoot->setAttribute(XML_OBJTYPE_ATTR, XML_UIXOBJ_VALUE);	
 	break;
-    case CegoObject::INDEX:
+    case CegoObject::AVLTREE:
 	pRoot->setAttribute(XML_OBJTYPE_ATTR, XML_IDXOBJ_VALUE);	
 	break;
     case CegoObject::VIEW:
@@ -377,11 +366,11 @@ void CegoTableObject::putElement(Element* pElement)
 	else if ( objType == Chain(XML_TABOBJ_VALUE))
 	    setType(CegoObject::TABLE);
 	else if ( objType == Chain(XML_PIXOBJ_VALUE))
-	    setType(CegoObject::PINDEX);
+	    setType(CegoObject::PAVLTREE);
 	else if ( objType == Chain(XML_UIXOBJ_VALUE))
-	    setType(CegoObject::UINDEX);
+	    setType(CegoObject::UAVLTREE);
 	else if ( objType == Chain(XML_IDXOBJ_VALUE))
-	    setType(CegoObject::INDEX);
+	    setType(CegoObject::AVLTREE);
 	else if ( objType == Chain(XML_VIEWOBJ_VALUE))
 	    setType(CegoObject::VIEW);
 	else if ( objType == Chain(XML_FKEYOBJ_VALUE))
@@ -455,19 +444,19 @@ Chain CegoTableObject::getFormatted() const
     switch ( getType() )
     {
     case CegoObject::TABLE:
-	s += Chain("table          "); 
+	s += Chain("table           "); 
 	break;
-    case CegoObject::INDEX:
-	s += Chain("index          ");
+    case CegoObject::AVLTREE:
+	s += Chain("avltree         ");
 	break;
-    case CegoObject::UINDEX:
-	s += Chain("unique index   ");
+    case CegoObject::UAVLTREE:
+	s += Chain("unique avltree  ");
 	break;
-    case CegoObject::PINDEX:
-	s += Chain("primary index  ");
+    case CegoObject::PAVLTREE:
+	s += Chain("primary avltree ");
 	break;
     case CegoObject::VIEW:
-	s += Chain("view           ");
+	s += Chain("view            ");
 	break;
     default:
 	break;
@@ -558,6 +547,11 @@ Chain CegoTableObject::getFormatted() const
 	    s += Chain(" blob          |");
 	    break;
 	}
+	case CLOB_TYPE:
+	{
+	    s += Chain(" clob          |");
+	    break;
+	}
 	default:
 	    break;
 	}
